package com.example.trainingPartner.integration;

import static androidx.test.espresso.Espresso.onView;
import static androidx.test.espresso.action.ViewActions.click;
import static androidx.test.espresso.assertion.ViewAssertions.doesNotExist;
import static androidx.test.espresso.assertion.ViewAssertions.matches;
import static androidx.test.espresso.matcher.ViewMatchers.isDisplayed;
import static androidx.test.espresso.matcher.ViewMatchers.isEnabled;
import static androidx.test.espresso.matcher.ViewMatchers.withId;
import static androidx.test.espresso.matcher.ViewMatchers.withText;
import static org.hamcrest.CoreMatchers.allOf;
import static org.hamcrest.CoreMatchers.containsString;
import static org.hamcrest.CoreMatchers.not;

import android.content.Context;
import android.content.SharedPreferences;
import android.view.View;
import android.widget.NumberPicker;

import androidx.test.espresso.Espresso;
import androidx.test.espresso.UiController;
import androidx.test.espresso.ViewAction;
import androidx.test.espresso.matcher.ViewMatchers;
import androidx.test.ext.junit.runners.AndroidJUnit4;
import androidx.test.rule.ActivityTestRule;

import com.example.mapbox2.R;
import com.example.trainingPartner.controller.MainActivity;
import com.example.trainingPartner.controller.SplashActivity;
import com.example.trainingPartner.dto.PointDTO;
import com.example.trainingPartner.dto.UserInformationDTO;
import com.example.trainingPartner.dto.activityDto.LocationActivityDTO;
import com.example.trainingPartner.utils.Utils;
import com.google.firebase.firestore.CollectionReference;
import com.google.firebase.firestore.FirebaseFirestore;

import org.hamcrest.Matcher;
import org.junit.After;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

/**
 * Instrumented test, which will execute on an Android device.
 *
 * @see <a href="http://d.android.com/tools/testing">Testing documentation</a>
 */
@RunWith(AndroidJUnit4.class)
public class HistoricalDayFragmentIntegrationTest {

    @Rule
    public ActivityTestRule<SplashActivity> signRule = new ActivityTestRule<SplashActivity>(SplashActivity.class);

    @Rule
    public ActivityTestRule<MainActivity> mainRule = new ActivityTestRule<MainActivity>(MainActivity.class, false, false);

    private FirebaseFirestore fb = FirebaseFirestore.getInstance();

    private MainActivity mainActivity;
    private Calendar calendar = Calendar.getInstance();
    private int day = (calendar.get(Calendar.DAY_OF_MONTH))-1;
    private int month = (calendar.get(Calendar.MONTH))+2;
    private UserInformationDTO user;
    private Context context;

    private CollectionReference reference = fb.collection("usuarios").document("usuarioNoExistente").collection("año").
            document(calendar.get(Calendar.YEAR)+"").collection(calendar.get(Calendar.MONTH)+"");

    @Before
    public void setUp() throws InterruptedException {
        if (day <=0){
            day =2;
        }
        if(month>=12){
            month = 1;
        }

        SplashActivity act = signRule.getActivity();
        SharedPreferences userPref = act.getApplicationContext().getSharedPreferences("users",Context.MODE_PRIVATE);
        SharedPreferences.Editor edit = userPref.edit();
        edit.putString("username", "usuarioNoExistente");
        edit.apply();

        user = new UserInformationDTO("Pedro","usuarioNoExistente","Pruebas", new Date(), 65.7, 175, true, "default", "ContraseñaPrueba@");
        fb.collection("usuarios").document("usuarioNoExistente").set(user.getInformation());

        List<PointDTO> points = new ArrayList<>();
        points.add(new PointDTO(43.5378458, -5.6390675));
        LocationActivityDTO dto = new LocationActivityDTO(points,Double.valueOf(1), calendar);
        reference.document(day+"").collection("actividades").document(dto.getHour()).set(dto.setValuesToSave());
        HashMap<String, Integer> st = new HashMap<>();
        st.put("steps", 12);
        st.put("calories",2);
        reference.document(day+"").set(st);

        SharedPreferences stepsPref = act.getApplicationContext().getSharedPreferences("stepCounter",Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = stepsPref.edit();
        editor.putInt("stepCount", 15);
        editor.apply();
        signRule.finishActivity();
        Thread.sleep(2000);
        mainRule.launchActivity(null);
        mainActivity = mainRule.getActivity();
        Thread.sleep(1000);
        onView(withId(R.id.history)).perform(click());

    }
    @After
    public void tearDown(){
        mainRule.finishActivity();
        Utils.deleteCollection(reference);
        fb.collection("usuarios").document("usuarioNoExistente").delete();
        Utils.deleteSharedPreferences(mainActivity);
        Utils.deleteCollection( reference.document(day+"").collection("actividades"));
        Utils.deleteStepsSharedPreferences(mainActivity);
        Utils.deleteCollection(fb.collection("usuarios").document("usuarioNoExistente").collection("records"));
        mainActivity=null;

    }


    @Test
    public void allInTheirPlaceTest() throws InterruptedException {
        Thread.sleep(1000);
        onView(allOf(withText(containsString(day+"")), isEnabled())).perform(click());
        onView(withId(R.id.buttonVisualize)).perform(click());
        Thread.sleep(2000);
        onView(withId(R.id.textViewTitleHistoricalDay)).check(matches(isDisplayed()));
        onView(allOf(withId(R.id.textViewHistoricalTitle), withText(containsString(day+"/"+(calendar.get(Calendar.MONTH)+1)+"/"+calendar.get(Calendar.YEAR))))).check(matches(isDisplayed()));
        onView(allOf(withId(R.id.textViewCalories), withText("2.0"))).check(matches(isDisplayed()));
        onView(allOf(withId(R.id.textViewSteps), withText("12"))).check(matches(isDisplayed()));
        onView(withText(containsString("AUTOMÁTICO"))).check(matches(isDisplayed()));
    }

    @Test
    public void openActivityTest() throws InterruptedException {
        Thread.sleep(1000);
        onView(allOf(withText(containsString(day+"")), isEnabled())).perform(click());
        onView(withId(R.id.buttonVisualize)).perform(click());
        Thread.sleep(2000);
        onView(withText(containsString("AUTOMÁTICO"))).check(matches(isDisplayed()));
        onView(withText(containsString("AUTOMÁTICO"))).perform(click());
        Thread.sleep(1000);
        onView(withId(R.id.mapView)).check(matches(isDisplayed()));

    }

    @Test
    public void onBackPressedTest() throws InterruptedException {
        Thread.sleep(1000);
        onView(allOf(withText(containsString(day+"")), isEnabled())).perform(click());
        onView(withId(R.id.buttonVisualize)).perform(click());
        Thread.sleep(2000);
        onView(withText(containsString("AUTOMÁTICO"))).check(matches(isDisplayed()));
        Espresso.pressBack();
        Thread.sleep(1000);
        onView(withId(R.id.calendarView2)).check(matches(isDisplayed()));
    }











}