package com.example.trainingPartner.integration;

import static androidx.test.espresso.Espresso.onView;
import static androidx.test.espresso.action.ViewActions.clearText;
import static androidx.test.espresso.action.ViewActions.click;
import static androidx.test.espresso.action.ViewActions.typeText;
import static androidx.test.espresso.assertion.ViewAssertions.matches;
import static androidx.test.espresso.matcher.ViewMatchers.isChecked;
import static androidx.test.espresso.matcher.ViewMatchers.isDisplayed;
import static androidx.test.espresso.matcher.ViewMatchers.isNotChecked;
import static androidx.test.espresso.matcher.ViewMatchers.withId;
import static androidx.test.espresso.matcher.ViewMatchers.withText;

import androidx.test.espresso.Espresso;
import androidx.test.ext.junit.runners.AndroidJUnit4;
import androidx.test.filters.LargeTest;
import androidx.test.rule.ActivityTestRule;

import com.example.mapbox2.R;
import com.example.trainingPartner.controller.LogInActivity;
import com.example.trainingPartner.controller.SignInActivity;
import com.example.trainingPartner.dto.UserInformationDTO;
import com.example.trainingPartner.utils.Utils;
import com.google.firebase.firestore.FirebaseFirestore;

import org.junit.After;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * Instrumented test, which will execute on an Android device.
 *
 * @see <a href="http://d.android.com/tools/testing">Testing documentation</a>
 */
@LargeTest
@RunWith(AndroidJUnit4.class)
public class LogInIntegrationTest {
    @Rule
    public ActivityTestRule<SignInActivity> iaActivityTestRule= new ActivityTestRule<SignInActivity>(SignInActivity.class);
    @Rule
    public ActivityTestRule<LogInActivity> log= new ActivityTestRule<LogInActivity>(LogInActivity.class);

    private SignInActivity ia;
    private LogInActivity l;
    private FirebaseFirestore fb = FirebaseFirestore.getInstance();
    @Before
    public void setUp() throws Exception {

        ia= iaActivityTestRule.getActivity();
        fb.collection("usuarios").document("usuarioNoExistente").delete();
        Utils.deleteSharedPreferences(ia);
        ia.userRegistered("usuarioNoRegistrado");
        ia.registerUser(new UserInformationDTO("Pedro","usuarioNoExistente","Pruebas", new Date(), 65.7, 175, true, "default", "ContraPrueba@"));
        iaActivityTestRule.finishActivity();
        l = log.getActivity();
    }

    @After
    public void tearDown() throws Exception {

        fb.collection("usuarios").document("usuarioNoExistente").delete();
        Utils.deleteSharedPreferences(ia);
        ia=null;
        l = null;
    }

    @Test
    public void allInTheirPlaceTest() {
        onView(withId(R.id.imageViewLogo)).check(matches(isDisplayed()));
        onView(withId(R.id.editTextTextUserNameLog)).check(matches(isDisplayed()));
        onView(withText("Inicio de sesión")).check(matches(isDisplayed()));
        onView(withId(R.id.buttonGoToSigIn)).check(matches(isDisplayed()));
        onView(withId(R.id.buttonLogIn)).check(matches(isDisplayed()));
    }

    @Test
    public void allFieldsCompleteTest() {
        Espresso.closeSoftKeyboard();
        onView(withId(R.id.buttonLogIn)).perform(click());
        onView(withText("Todos los campos deben de estar completos")).check(matches(isDisplayed()));
        onView(withId(R.id.editTextTextUserNameLog)).perform(typeText("usuarioNoExistente"));
        Espresso.closeSoftKeyboard();
        onView(withId(R.id.buttonLogIn)).perform(click());
        onView(withText("Todos los campos deben de estar completos")).check(matches(isDisplayed()));
        onView(withId(R.id.editTextTextUserNameLog)).perform(clearText());
        onView(withId(R.id.editTextTextPasswordLog)).perform(typeText("ContraPrueba@"));
        Espresso.closeSoftKeyboard();
        onView(withId(R.id.buttonLogIn)).perform(click());
        onView(withText("Todos los campos deben de estar completos")).check(matches(isDisplayed()));
    }

    @Test
    public void userNotExists() throws InterruptedException {
        onView(withId(R.id.editTextTextUserNameLog)).perform(typeText("usuariNoExistente"));
        onView(withId(R.id.editTextTextPasswordLog)).perform(typeText("ContraPrueba@"));
        Espresso.closeSoftKeyboard();
        onView(withId(R.id.buttonLogIn)).perform(click());
        Thread.sleep(1000);
        onView(withText("El usuario no existe")).check(matches(isDisplayed()));
        onView(withId(R.id.editTextTextUserNameLog)).perform(clearText());
        onView(withId(R.id.editTextTextUserNameLog)).perform(typeText("usuarioNoExistente"));
        Espresso.closeSoftKeyboard();
        onView(withId(R.id.buttonLogIn)).perform(click());
        Thread.sleep(5000);
        onView(withText("Pedro Pruebas")).check(matches(isDisplayed()));

    }

    @Test
    public void wrongPasswordTest() throws InterruptedException {
        onView(withId(R.id.editTextTextUserNameLog)).perform(typeText("usuarioNoExistente"));
        onView(withId(R.id.editTextTextPasswordLog)).perform(typeText("Contrase"));
        Espresso.closeSoftKeyboard();
        onView(withId(R.id.buttonLogIn)).perform(click());
        Thread.sleep(1000);
        onView(withText("El nombre de usuario o contraseña no es correcto")).check(matches(isDisplayed()));
        onView(withId(R.id.editTextTextPasswordLog)).perform(clearText());
        onView(withId(R.id.editTextTextPasswordLog)).perform(typeText("ContraPrueba@"));
        Espresso.closeSoftKeyboard();
        onView(withId(R.id.buttonLogIn)).perform(click());
        Thread.sleep(5000);
        onView(withText("Pedro Pruebas")).check(matches(isDisplayed()));
    }




    @Test
    public void testMoveToSignIn() throws InterruptedException {;
        onView(withId(R.id.buttonGoToSigIn)).check(matches(isDisplayed()));
        onView(withId(R.id.buttonGoToSigIn)).perform(click());
        onView(withText("Registro")).check(matches(isDisplayed()));
        onView(withId(R.id.buttonGoToLogIn)).check(matches(isDisplayed()));
        onView(withId(R.id.buttonGoToLogIn)).perform(click());
        onView(withText("Inicio de sesión")).check(matches(isDisplayed()));
        onView(withId(R.id.editTextTextPasswordLog)).check(matches(withText("")));
        onView(withId(R.id.editTextTextUserNameLog)).check(matches(withText("")));
    }



}