package com.example.trainingPartner.integration;

import static androidx.test.espresso.Espresso.onView;
import static androidx.test.espresso.action.ViewActions.click;
import static androidx.test.espresso.assertion.ViewAssertions.doesNotExist;
import static androidx.test.espresso.assertion.ViewAssertions.matches;
import static androidx.test.espresso.matcher.ViewMatchers.isDisplayed;
import static androidx.test.espresso.matcher.ViewMatchers.isNotSelected;
import static androidx.test.espresso.matcher.ViewMatchers.isSelected;
import static androidx.test.espresso.matcher.ViewMatchers.withId;
import static androidx.test.espresso.matcher.ViewMatchers.withText;
import static org.hamcrest.CoreMatchers.not;
import static org.junit.Assert.assertEquals;

import android.content.Context;
import android.content.SharedPreferences;

import androidx.test.espresso.Espresso;
import androidx.test.ext.junit.runners.AndroidJUnit4;
import androidx.test.rule.ActivityTestRule;

import com.example.mapbox2.R;
import com.example.trainingPartner.controller.MainActivity;
import com.example.trainingPartner.controller.SplashActivity;
import com.example.trainingPartner.dto.UserInformationDTO;
import com.example.trainingPartner.model.DailyInformationModel;
import com.example.trainingPartner.model.MyCallbBack;
import com.example.trainingPartner.utils.Utils;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.firebase.firestore.CollectionReference;
import com.google.firebase.firestore.DocumentSnapshot;
import com.google.firebase.firestore.FirebaseFirestore;

import org.junit.After;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;

import java.util.Calendar;
import java.util.Date;

/**
 * Instrumented test, which will execute on an Android device.
 *
 * @see <a href="http://d.android.com/tools/testing">Testing documentation</a>
 */
@RunWith(AndroidJUnit4.class)
public class MainActivityIntegrationTest {

    @Rule
    public ActivityTestRule<SplashActivity> signRule = new ActivityTestRule<SplashActivity>(SplashActivity.class);

    @Rule
    public ActivityTestRule<MainActivity> mainRule = new ActivityTestRule<MainActivity>(MainActivity.class, false, false);

    private FirebaseFirestore fb = FirebaseFirestore.getInstance();

    private MainActivity mainActivity;
    private UserInformationDTO user;
    private Context context;
    private Calendar calendar = Calendar.getInstance();
    private CollectionReference reference = fb.collection("usuarios").document("usuarioNoExistente").collection("año").
            document(calendar.get(Calendar.YEAR)+"").collection(calendar.get(Calendar.MONTH)+"");

    @Before
    public void setUp() throws InterruptedException {
        SplashActivity act = signRule.getActivity();
        SharedPreferences userPref = act.getApplicationContext().getSharedPreferences("users",Context.MODE_PRIVATE);
        SharedPreferences.Editor edit = userPref.edit();
        edit.putString("username", "usuarioNoExistente");
        edit.apply();
        user = new UserInformationDTO("Pedro","usuarioNoExistente","Pruebas", new Date(), 65.7, 175, true, "default", "ContraseñaPrueba@");
        fb.collection("usuarios").document("usuarioNoExistente").set(user.getInformation());
        SharedPreferences stepsPref = act.getApplicationContext().getSharedPreferences("stepCounter",Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = stepsPref.edit();
        editor.putInt("stepCount", 15);
        editor.apply();
        signRule.finishActivity();
        Thread.sleep(2000);
        mainRule.launchActivity(null);
        mainActivity = mainRule.getActivity();
    }
    @After
    public void tearDown(){
        mainRule.finishActivity();
        Utils.deleteCollection(reference);
        fb.collection("usuarios").document("usuarioNoExistente").delete();
        Utils.deleteSharedPreferences(mainActivity);
        fb.collection("registro").document("usuarioNoExistente").delete();
        Utils.deleteStepsSharedPreferences(mainActivity);
        mainActivity=null;

    }

    @Test
    public void allInTheirPlace() throws InterruptedException {
        Thread.sleep(5000);
        onView(withId(R.id.imageButtonProfile)).check(matches(isDisplayed()));
        onView(withText("Pedro Pruebas")).check(matches(isDisplayed()));
        onView(withId(R.id.imageViewSignOut)).check(matches(isDisplayed()));
        onView(withId(R.id.imageViewWeight)).check(matches(isDisplayed()));
        onView(withId(R.id.imageViewHeight)).check(matches(isDisplayed()));
        onView(withId(R.id.imageViewSteps)).check(matches(isDisplayed()));
        onView(withId(R.id.imageViewCalories)).check(matches(isDisplayed()));
        onView(withText("65.7 kg")).check(matches(isDisplayed()));
        onView(withText("1.75 m")).check(matches(isDisplayed()));
    }

    /**
     * Ha habido que introducir el previousMenuItem para que funcionara
     * @throws InterruptedException
     */
    @Test
    public void changeFragment() throws InterruptedException {
        Thread.sleep(5000);
        onView(withId(R.id.imageButtonProfile)).check(matches(isDisplayed()));
        onView(withText("Pedro Pruebas")).check(matches(isDisplayed()));
        onView(withId(R.id.main_window)).check(matches(isDisplayed()));
        onView(withId(R.id.main_window)).check(matches(isSelected()));
        onView(withId(R.id.history)).check(matches(isDisplayed()));
        onView(withId(R.id.history)).perform(click());
        Thread.sleep(1000);
        onView(withId(R.id.textViewHistory)).check(matches(isDisplayed()));
        onView(withId(R.id.main_window)).check(matches(isNotSelected()));
        onView(withId(R.id.history)).check(matches(isSelected()));
        Espresso.pressBack();
        Thread.sleep(1000);
        onView(withText("Pedro Pruebas")).check(matches(isDisplayed()));
        onView(withId(R.id.main_window)).check(matches(isSelected()));
        onView(withId(R.id.history)).check(matches(isNotSelected()));
        Espresso.pressBack();
        Thread.sleep(1000);
        onView(withId(R.id.textViewHistory)).check(matches(isDisplayed()));
        onView(withId(R.id.main_window)).check(matches(isNotSelected()));
        onView(withId(R.id.history)).check(matches(isSelected()));
        onView(withId(R.id.predictions)).perform(click());
        Thread.sleep(1000);
        onView(withId(R.id.history)).check(matches(isNotSelected()));
        onView(withId(R.id.predictions)).check(matches(isSelected()));
        onView(withId(R.id.textViewPredictions)).check(matches(isDisplayed()));
        onView(withId(R.id.record)).perform(click());
        onView(withText(R.string.selectMode)).check(matches(isDisplayed()));
        Espresso.pressBack();
        Thread.sleep(1000);
        onView(withId(R.id.record)).check(matches(isNotSelected()));
        onView(withId(R.id.predictions)).check(matches(isSelected()));
        onView(withId(R.id.textViewPredictions)).check(matches(isDisplayed()));
    }





}