package com.example.trainingPartner.integration;

import static androidx.test.espresso.Espresso.onView;
import static androidx.test.espresso.action.ViewActions.click;
import static androidx.test.espresso.action.ViewActions.typeText;
import static androidx.test.espresso.assertion.ViewAssertions.doesNotExist;
import static androidx.test.espresso.assertion.ViewAssertions.matches;
import static androidx.test.espresso.matcher.ViewMatchers.hasBackground;
import static androidx.test.espresso.matcher.ViewMatchers.isDisplayed;
import static androidx.test.espresso.matcher.ViewMatchers.isEnabled;
import static androidx.test.espresso.matcher.ViewMatchers.isNotEnabled;
import static androidx.test.espresso.matcher.ViewMatchers.isRoot;
import static androidx.test.espresso.matcher.ViewMatchers.withId;
import static androidx.test.espresso.matcher.ViewMatchers.withText;
import static com.google.common.util.concurrent.Striped.lock;
import static org.hamcrest.CoreMatchers.containsString;
import static org.hamcrest.CoreMatchers.not;
import static org.junit.Assert.assertEquals;

import android.content.Context;
import android.content.SharedPreferences;
import android.os.SystemClock;
import android.view.View;
import android.widget.NumberPicker;

import androidx.test.espresso.Espresso;
import androidx.test.espresso.IdlingResource;
import androidx.test.espresso.UiController;
import androidx.test.espresso.ViewAction;
import androidx.test.espresso.ViewInteraction;
import androidx.test.espresso.action.GeneralClickAction;
import androidx.test.espresso.action.GeneralLocation;
import androidx.test.espresso.action.GeneralSwipeAction;
import androidx.test.espresso.action.Press;
import androidx.test.espresso.action.Swipe;
import androidx.test.espresso.action.Tap;
import androidx.test.espresso.matcher.ViewMatchers;
import androidx.test.ext.junit.runners.AndroidJUnit4;
import androidx.test.rule.ActivityTestRule;

import com.example.mapbox2.R;
import com.example.trainingPartner.controller.ManualRecordActivity;
import com.example.trainingPartner.controller.SplashActivity;
import com.example.trainingPartner.dto.UserInformationDTO;
import com.example.trainingPartner.dto.activityDto.ActivityDTO;
import com.example.trainingPartner.dto.activityDto.ManualActivityDTO;
import com.example.trainingPartner.model.MyCallbBack;
import com.example.trainingPartner.model.TrainDetailsModel;
import com.example.trainingPartner.utils.Utils;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.firebase.firestore.CollectionReference;
import com.google.firebase.firestore.DocumentSnapshot;
import com.google.firebase.firestore.FirebaseFirestore;

import org.hamcrest.Matcher;
import org.junit.After;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;

import java.util.Calendar;
import java.util.Date;

/**
 * Instrumented test, which will execute on an Android device.
 *
 * @see <a href="http://d.android.com/tools/testing">Testing documentation</a>
 */
@RunWith(AndroidJUnit4.class)
public class ManualRecordActivityIntegrationTest {


    @Rule
    public ActivityTestRule<ManualRecordActivity> manualRule = new ActivityTestRule<ManualRecordActivity>(ManualRecordActivity.class, false, false);

    private FirebaseFirestore fb = FirebaseFirestore.getInstance();

    private ManualRecordActivity manualRecordActivity;
    private UserInformationDTO user;
    private Context context;
    private Calendar calendar = Calendar.getInstance();
    private CollectionReference reference = fb.collection("usuarios").document("usuarioNoExistente").collection("año").
            document(calendar.get(Calendar.YEAR)+"").collection(calendar.get(Calendar.MONTH)+"");

    @Before
    public void setUp(){
        manualRule.launchActivity(null);
        manualRecordActivity = manualRule.getActivity();
        SharedPreferences userPref = manualRecordActivity.getApplicationContext().getSharedPreferences("users",Context.MODE_PRIVATE);
        SharedPreferences.Editor edit = userPref.edit();
        edit.putString("username", "usuarioNoExistente");
        edit.apply();
        user = new UserInformationDTO("Pedro","usuarioNoExistente","Pruebas", new Date(), 65.7, 175, true, "default", "ContraseñaPrueba@");
        fb.collection("usuarios").document("usuarioNoExistente").set(user.getInformation());



    }
    @After
    public void tearDown() throws InterruptedException {
        Utils.deleteCollection( reference.document(calendar.get(Calendar.DAY_OF_MONTH)+"").collection("actividades"));
        Utils.deleteCollection(reference);
        fb.collection("usuarios").document("usuarioNoExistente").delete();
        Utils.deleteCollection(fb.collection("usuarios").document("usuarioNoExistente").collection("records"));
        Utils.deleteSharedPreferences(manualRecordActivity);
        manualRecordActivity= null;
    }

    @Test
    public void allInTheirPlace(){
        onView(withId(R.id.editTextSelectDistance)).check(matches(isDisplayed()));
        onView(withId(R.id.numberPickerMinutes)).check(matches(isDisplayed()));
        onView(withId(R.id.numberPickerSeconds)).check(matches(isDisplayed()));
        onView(withId(R.id.numberPickerKm)).check(matches(isDisplayed()));
        onView(withId(R.id.numberPickerm)).check(matches(isDisplayed()));
        onView(withId(R.id.numberPickerHours)).check(matches(isDisplayed()));
        onView(withId(R.id.buttonSaveManualActivity)).check(matches(isDisplayed()));
        onView(withId(R.id.editTextSelectTime)).check(matches(isDisplayed()));
    }

    @Test
    public void onBackPressedTest() throws InterruptedException {
        onView(withId(R.id.editTextSelectDistance)).check(matches(isDisplayed()));
        Espresso.pressBack();
        Thread.sleep(2000);
        onView(withText("Pedro Pruebas")).check(matches(isDisplayed()));
        onView(withText("Manual")).check(doesNotExist());
    }

    @Test
    public void buttonActivateTest() throws InterruptedException {
        onView(withId(R.id.numberPickerMinutes)).check(matches(isDisplayed()));
        onView(withId(R.id.numberPickerSeconds)).check(matches(isDisplayed()));
        onView(withId(R.id.numberPickerKm)).check(matches(isDisplayed()));
        onView(withId(R.id.numberPickerm)).check(matches(isDisplayed()));
        onView(withId(R.id.numberPickerHours)).check(matches(isDisplayed()));
        onView(withId(R.id.numberPickerm)).perform(new Action(2));
        onView(withId(R.id.numberPickerMinutes)).perform(new GeneralSwipeAction(Swipe.FAST, GeneralLocation.TOP_CENTER, GeneralLocation.BOTTOM_CENTER, Press.FINGER));
        onView(withId(R.id.buttonSaveManualActivity)).check(matches(isEnabled()));
        onView(withId(R.id.numberPickerMinutes)).perform(new Action(0));
        onView(withId(R.id.numberPickerm)).perform(new GeneralSwipeAction(Swipe.FAST, GeneralLocation.TOP_CENTER, GeneralLocation.BOTTOM_CENTER, Press.FINGER));
        onView(withId(R.id.buttonSaveManualActivity)).check(matches(isNotEnabled()));
    }

    @Test
    public void buttonNotActivateTest() throws InterruptedException {
        onView(withId(R.id.numberPickerMinutes)).check(matches(isDisplayed()));
        onView(withId(R.id.numberPickerSeconds)).check(matches(isDisplayed()));
        onView(withId(R.id.numberPickerKm)).check(matches(isDisplayed()));
        onView(withId(R.id.numberPickerm)).check(matches(isDisplayed()));
        onView(withId(R.id.numberPickerHours)).check(matches(isDisplayed()));
        onView(withId(R.id.numberPickerm)).perform(new Action(0));
        onView(withId(R.id.numberPickerMinutes)).perform(new GeneralSwipeAction(Swipe.FAST, GeneralLocation.TOP_CENTER, GeneralLocation.BOTTOM_CENTER, Press.FINGER));
        onView(withId(R.id.buttonSaveManualActivity)).check(matches(isNotEnabled()));
        onView(withId(R.id.numberPickerMinutes)).perform(new GeneralSwipeAction(Swipe.FAST, GeneralLocation.TOP_CENTER, GeneralLocation.BOTTOM_CENTER, Press.FINGER));
    }

    @Test
    public void saveManualActivityTest() throws InterruptedException {

        onView(withId(R.id.numberPickerMinutes)).check(matches(isDisplayed()));
        onView(withId(R.id.numberPickerSeconds)).check(matches(isDisplayed()));
        onView(withId(R.id.numberPickerKm)).check(matches(isDisplayed()));
        onView(withId(R.id.numberPickerm)).check(matches(isDisplayed()));
        onView(withId(R.id.numberPickerHours)).check(matches(isDisplayed()));

        onView(withId(R.id.numberPickerMinutes)).perform(new Action(2));
        onView(withId(R.id.numberPickerKm)).perform(new GeneralSwipeAction(Swipe.FAST, GeneralLocation.BOTTOM_CENTER, GeneralLocation.TOP_CENTER, Press.FINGER));
        onView(withId(R.id.buttonSaveManualActivity)).perform(click());
        Thread.sleep(1000);
        onView(withId(R.id.imageView)).check(matches(isDisplayed()));


    }




    class Action implements ViewAction {
        private int number;
        public Action(int number){
            this.number = number;
        }
        @Override
        public Matcher<View> getConstraints() {
            return ViewMatchers.isAssignableFrom(NumberPicker.class);
        }

        @Override
        public String getDescription() {
            return "Automatiza numberPicker";
        }

        @Override
        public void perform(UiController uiController, View view) {
            NumberPicker picker = (NumberPicker) view;
            picker.setValue(number);

        }
    }



}