package com.example.trainingPartner.integration;

import static androidx.test.espresso.Espresso.onView;
import static androidx.test.espresso.action.ViewActions.click;
import static androidx.test.espresso.assertion.ViewAssertions.matches;
import static androidx.test.espresso.matcher.ViewMatchers.isDisplayed;
import static androidx.test.espresso.matcher.ViewMatchers.isNotSelected;
import static androidx.test.espresso.matcher.ViewMatchers.isSelected;
import static androidx.test.espresso.matcher.ViewMatchers.withId;
import static androidx.test.espresso.matcher.ViewMatchers.withText;

import static org.hamcrest.CoreMatchers.not;

import android.content.Context;
import android.content.SharedPreferences;
import android.icu.text.AlphabeticIndex;

import androidx.test.espresso.Espresso;
import androidx.test.ext.junit.runners.AndroidJUnit4;
import androidx.test.rule.ActivityTestRule;

import com.example.mapbox2.R;
import com.example.trainingPartner.controller.MainActivity;
import com.example.trainingPartner.controller.ManualRecordActivity;
import com.example.trainingPartner.controller.SplashActivity;
import com.example.trainingPartner.dto.PointDTO;
import com.example.trainingPartner.dto.UserInformationDTO;
import com.example.trainingPartner.dto.activityDto.LocationActivityDTO;
import com.example.trainingPartner.dto.activityDto.ManualActivityDTO;
import com.example.trainingPartner.model.ActivitiesModel;
import com.example.trainingPartner.model.MyCallbBack;
import com.example.trainingPartner.model.RecordModel;
import com.example.trainingPartner.utils.Utils;
import com.google.firebase.firestore.CollectionReference;
import com.google.firebase.firestore.FirebaseFirestore;

import org.junit.After;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

/**
 * Instrumented test, which will execute on an Android device.
 *
 * @see <a href="http://d.android.com/tools/testing">Testing documentation</a>
 */
@RunWith(AndroidJUnit4.class)
public class PredictionsFragmentIntegrationTest {

    @Rule
    public ActivityTestRule<SplashActivity> signRule = new ActivityTestRule<SplashActivity>(SplashActivity.class);

    @Rule
    public ActivityTestRule<MainActivity> mainRule = new ActivityTestRule<MainActivity>(MainActivity.class, false, false);

    private FirebaseFirestore fb = FirebaseFirestore.getInstance();

    private MainActivity mainActivity;
    private UserInformationDTO user;
    private Context context;
    private Calendar calendar = Calendar.getInstance();
    private CollectionReference reference = fb.collection("usuarios").document("usuarioNoExistente").collection("año").
            document(calendar.get(Calendar.YEAR)+"").collection(calendar.get(Calendar.MONTH)+"");

    @Before
    public void setUp() throws InterruptedException {

        SplashActivity act = signRule.getActivity();
        SharedPreferences userPref = act.getApplicationContext().getSharedPreferences("users",Context.MODE_PRIVATE);
        SharedPreferences.Editor edit = userPref.edit();
        edit.putString("username", "usuarioNoExistente");
        edit.apply();

        user = new UserInformationDTO("Pedro","usuarioNoExistente","Pruebas", new Date(), 65.7, 175, true, "default", "ContraseñaPrueba@");
        fb.collection("usuarios").document("usuarioNoExistente").set(user.getInformation());

        ManualActivityDTO dto = new ManualActivityDTO(3, 720 , calendar);

        ManualActivityDTO dto2 = new ManualActivityDTO(4, 920 , calendar);

       RecordModel model = new RecordModel("usuarioNoExistente");
        model.saveActivity(dto, new MyCallbBack<Boolean>() {
            @Override
            public void onCallback(Boolean element) {

            }
        });
        model.saveActivity(dto2, new MyCallbBack<Boolean>() {
            @Override
            public void onCallback(Boolean element) {

            }
        });
        Thread.sleep(8000);



        signRule.finishActivity();
        Thread.sleep(2000);
        mainRule.launchActivity(null);
        mainActivity = mainRule.getActivity();
        Thread.sleep(1000);
        onView(withId(R.id.predictions)).perform(click());

    }
    @After
    public void tearDown(){
        mainRule.finishActivity();
        Utils.deleteCollection(reference);
        fb.collection("usuarios").document("usuarioNoExistente").delete();
        Utils.deleteSharedPreferences(mainActivity);
        Utils.deleteCollection( reference.document(calendar.get(Calendar.DAY_OF_MONTH)+"").collection("actividades"));
        Utils.deleteStepsSharedPreferences(mainActivity);
        Utils.deleteCollection(fb.collection("usuarios").document("usuarioNoExistente").collection("records"));
        mainActivity=null;
    }

    @Test
    public void allInTheirPlace() throws InterruptedException {
        Thread.sleep(5000);
        onView(withId(R.id.textViewPredictions)).check(matches(isDisplayed()));
        onView(withId(R.id.notEnoughActivities)).check(matches(isDisplayed()));
        onView(withId(R.id.one_pred)).check(matches(not(isDisplayed())));
    }

   @Test
   public void allInTheirPlaceAddActivity() throws InterruptedException {
       Thread.sleep(5000);
       onView(withId(R.id.textViewPredictions)).check(matches(isDisplayed()));
       onView(withId(R.id.main_window)).perform(click());
       RecordModel model = new RecordModel("usuarioNoExistente");
       ManualActivityDTO dto2 = new ManualActivityDTO(4, 920 , calendar);
       model.saveActivity(dto2, new MyCallbBack<Boolean>() {
           @Override
           public void onCallback(Boolean element) {


           }
       });
       Thread.sleep(7000);
       onView(withId(R.id.predictions)).perform(click());
       onView(withId(R.id.textViewPredictions)).check(matches(isDisplayed()));
       onView(withId(R.id.notEnoughActivities)).check(matches(not(isDisplayed())));
       onView(withId(R.id.one_pred)).check(matches(isDisplayed()));
       onView(withId(R.id.one_pred)).check(matches(isDisplayed()));

   }





}