package com.example.trainingPartner.integration;

import static androidx.test.espresso.Espresso.onView;
import static androidx.test.espresso.action.ViewActions.click;
import static androidx.test.espresso.assertion.ViewAssertions.doesNotExist;
import static androidx.test.espresso.assertion.ViewAssertions.matches;
import static androidx.test.espresso.matcher.ViewMatchers.isDisplayed;
import static androidx.test.espresso.matcher.ViewMatchers.withId;
import static androidx.test.espresso.matcher.ViewMatchers.withText;
import static org.hamcrest.CoreMatchers.containsString;
import static org.hamcrest.CoreMatchers.not;
import static org.junit.Assert.assertEquals;

import android.content.Context;
import android.content.SharedPreferences;
import android.os.SystemClock;
import android.view.View;
import android.widget.EditText;
import android.widget.TextView;

import androidx.test.espresso.Espresso;
import androidx.test.espresso.ViewInteraction;
import androidx.test.ext.junit.runners.AndroidJUnit4;
import androidx.test.rule.ActivityTestRule;

import com.example.mapbox2.R;
import com.example.trainingPartner.controller.RecordActivity;
import com.example.trainingPartner.controller.SignInActivity;
import com.example.trainingPartner.controller.SplashActivity;
import com.example.trainingPartner.dto.UserInformationDTO;
import com.example.trainingPartner.utils.Utils;
import com.google.firebase.firestore.CollectionReference;
import com.google.firebase.firestore.FirebaseFirestore;

import org.hamcrest.Description;
import org.hamcrest.TypeSafeMatcher;
import org.junit.After;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;

import java.util.Calendar;
import java.util.Date;

/**
 * Instrumented test, which will execute on an Android device.
 *
 * @see <a href="http://d.android.com/tools/testing">Testing documentation</a>
 */
@RunWith(AndroidJUnit4.class)
public class RecordActivityIntegrationTest {
    @Rule
    public ActivityTestRule<SplashActivity> signRule = new ActivityTestRule<SplashActivity>(SplashActivity.class);

    @Rule
    public ActivityTestRule<RecordActivity> recordRule = new ActivityTestRule<RecordActivity>(RecordActivity.class, false, false);

    private FirebaseFirestore fb = FirebaseFirestore.getInstance();

    private RecordActivity recordActivity;
    private SplashActivity act;
    private UserInformationDTO user;
    private Calendar calendar = Calendar.getInstance();
    private CollectionReference reference = fb.collection("usuarios").document("usuarioNoExistente").collection("año").
            document(calendar.get(Calendar.YEAR)+"").collection(calendar.get(Calendar.MONTH)+"");

    @Before
    public void setUp() throws InterruptedException {
        Thread.sleep(5000);
        user = new UserInformationDTO("Pedro","usuarioNoExistente","Pruebas", new Date(), 65.7, 175, true, "default", "ContraseñaPrueba@");
        fb.collection("usuarios").document("usuarioNoExistente").set(user.getInformation());
        act = signRule.getActivity();
        Utils.deleteSharedPreferences(act);
        SharedPreferences userPref = act.getApplicationContext().getSharedPreferences("users",Context.MODE_PRIVATE);
        SharedPreferences.Editor edit = userPref.edit();
        edit.putString("username", "usuarioNoExistente");
        edit.apply();

        signRule.finishActivity();
        Thread.sleep(1000);
        act = null;
        recordRule.launchActivity(null);
        recordActivity = recordRule.getActivity();
    }
    @After
    public void tearDown() throws InterruptedException {
        recordRule.finishActivity();
        Utils.deleteCollection( reference.document(calendar.get(Calendar.DAY_OF_MONTH)+"").collection("actividades"));
        Utils.deleteCollection(reference);
        fb.collection("usuarios").document("usuarioNoExistente").delete();
        Utils.deleteSharedPreferences(recordActivity);
        recordActivity= null;
    }

    @Test
    public void allInTheirPlaceTest() throws InterruptedException {
        onView(withId(R.id.textViewDistance)).check(matches(isDisplayed()));
        onView(withId(R.id.textViewDistanceValue)).check(matches(isDisplayed()));
        onView(withId(R.id.textViewVelocity)).check(matches(isDisplayed()));
        onView(withId(R.id.textViewVelocityValue)).check(matches(isDisplayed()));
        onView(withId(R.id.textView)).check(matches(isDisplayed()));
        onView(withId(R.id.textViewMinKmValue)).check(matches(isDisplayed()));
        onView(withId(R.id.textView)).check(matches(isDisplayed()));
        onView(withId(R.id.textViewMinKmValue)).check(matches(isDisplayed()));
        onView(withId(R.id.textViewTime)).check(matches(isDisplayed()));
        onView(withId(R.id.textViewTimeValue)).check(matches(isDisplayed()));
        onView(withId(R.id.imageButtonStart)).check(matches(isDisplayed()));
        onView(withId(R.id.imageButtonEnd)).check(matches(not(isDisplayed())));
        onView(withId(R.id.imageButtonStop)).check(matches(not(isDisplayed())));
    }

    @Test
    public void startActivityTest() throws InterruptedException {
        onView(withId(R.id.textViewTimeValue)).check(matches(hasText("00:00:00")));
        onView(withId(R.id.imageButtonStart)).check(matches(isDisplayed()));
        onView(withId(R.id.imageButtonEnd)).check(matches(not(isDisplayed())));
        onView(withId(R.id.imageButtonStop)).check(matches(not(isDisplayed())));
        onView(withId(R.id.imageButtonStart)).perform(click());
        onView(withId(R.id.imageButtonEnd)).check(matches(isDisplayed()));
        onView(withId(R.id.imageButtonStop)).check(matches(isDisplayed()));
        onView(withId(R.id.imageButtonStart)).check(matches(not(isDisplayed())));
        onView(withId(R.id.textViewTimeValue)).check(matches(not(hasText("00:00:00"))));
        onView(withId(R.id.imageButtonStop)).perform(click());
        onView(withId(R.id.imageButtonStart)).check(matches(isDisplayed()));
        onView(withId(R.id.imageButtonEnd)).check(matches(not(isDisplayed())));
        onView(withId(R.id.imageButtonStop)).check(matches(not(isDisplayed())));
        onView(withId(R.id.imageButtonStart)).perform(click());
        onView(withId(R.id.imageButtonEnd)).perform(click());

    }

    /**
     * Probar a ir hacia atras con la actividad sin iniciar
     * @throws InterruptedException
     */
    @Test
    public void onBackPressedActivityNotIniciatedTest() throws InterruptedException {
        Thread.sleep(2000);
        onView(withId(R.id.textViewTimeValue)).check(matches(hasText("00:00:00")));
        onView(withId(R.id.imageButtonStart)).check(matches(isDisplayed()));
        onView(withId(R.id.imageButtonEnd)).check(matches(not(isDisplayed())));
        onView(withId(R.id.imageButtonStop)).check(matches(not(isDisplayed())));
        Espresso.pressBack();
        onView(withText(containsString("AUTOMÁTICO"))).check(doesNotExist());
    }

    /**
     * Probar a ir hacia atrás con la actividad iniciada
     * @throws InterruptedException
     */
    @Test
    public void onBackPressedActivityIniciatedTest() throws InterruptedException {
        onView(withId(R.id.textViewTimeValue)).check(matches(hasText("00:00:00")));
        onView(withId(R.id.imageButtonStart)).check(matches(isDisplayed()));
        onView(withId(R.id.imageButtonEnd)).check(matches(not(isDisplayed())));
        onView(withId(R.id.imageButtonStop)).check(matches(not(isDisplayed())));
        onView(withId(R.id.imageButtonStart)).perform(click());
        onView(withId(R.id.imageButtonEnd)).check(matches(isDisplayed()));
        onView(withId(R.id.imageButtonStop)).check(matches(isDisplayed()));
        onView(withId(R.id.imageButtonStart)).check(matches(not(isDisplayed())));
        Espresso.pressBack();
        onView(withText(containsString("AUTOMÁTICO"))).check(doesNotExist());
    }

    /**
     * Probar a ir hacia atrás con la actividad pausada
     * @throws InterruptedException
     */
    @Test
    public void onBackPressedActivityPausedTest() throws InterruptedException {
        Thread.sleep(2000);
        onView(withId(R.id.textViewTimeValue)).check(matches(hasText("00:00:00")));
        onView(withId(R.id.imageButtonStart)).check(matches(isDisplayed()));
        onView(withId(R.id.imageButtonEnd)).check(matches(not(isDisplayed())));
        onView(withId(R.id.imageButtonStop)).check(matches(not(isDisplayed())));
        onView(withId(R.id.imageButtonStart)).perform(click());
        onView(withId(R.id.imageButtonEnd)).check(matches(isDisplayed()));
        onView(withId(R.id.imageButtonStop)).check(matches(isDisplayed()));
        onView(withId(R.id.imageButtonStart)).check(matches(not(isDisplayed())));
        onView(withId(R.id.imageButtonStop)).perform(click());
        onView(withId(R.id.imageButtonStart)).check(matches(isDisplayed()));
        onView(withId(R.id.imageButtonEnd)).check(matches(not(isDisplayed())));
        onView(withId(R.id.imageButtonStop)).check(matches(not(isDisplayed())));
        Espresso.pressBack();
        onView(withText(containsString("AUTOMÁTICO"))).check(doesNotExist());
    }

    /**
     * Añadi un callback en saveactivity
     * @throws InterruptedException
     */
    @Test
    public void saveActivityTest() throws InterruptedException {
        Thread.sleep(2000);
        onView(withId(R.id.textViewTimeValue)).check(matches(hasText("00:00:00")));
        onView(withId(R.id.imageButtonStart)).check(matches(isDisplayed()));
        onView(withId(R.id.imageButtonEnd)).check(matches(not(isDisplayed())));
        onView(withId(R.id.imageButtonStop)).check(matches(not(isDisplayed())));
        onView(withId(R.id.imageButtonStart)).perform(click());
        Thread.sleep(1000);
        onView(withId(R.id.imageButtonEnd)).perform(click());
        Thread.sleep(4000);
        onView(withId(R.id.textViewDistancia)).check(matches(isDisplayed()));
    }



    private TypeSafeMatcher<View> hasText(String content){
        return new TypeSafeMatcher<View>() {
            @Override
            public void describeTo(Description description) {
                description.appendText("Has EditText/TextView the value:  " + content);
            }

            @Override
            public boolean matchesSafely(View view) {
                if (!(view instanceof TextView) && !(view instanceof EditText)) {
                    return false;
                }
                if (view != null) {
                    String text;
                    if (view instanceof TextView) {
                        text = ((TextView) view).getText().toString();
                    } else {
                        text = ((EditText) view).getText().toString();
                    }

                    return (text.equalsIgnoreCase(content));
                }
                return false;
            }
        };
    }





}