package com.example.trainingPartner.integration;

import static androidx.test.espresso.Espresso.onView;
import static androidx.test.espresso.action.ViewActions.click;
import static androidx.test.espresso.action.ViewActions.typeText;
import static androidx.test.espresso.matcher.RootMatchers.isDialog;
import static androidx.test.espresso.matcher.ViewMatchers.isChecked;
import static androidx.test.espresso.matcher.ViewMatchers.isDisplayed;
import static androidx.test.espresso.matcher.ViewMatchers.isNotChecked;
import static androidx.test.espresso.matcher.ViewMatchers.isSelected;
import static androidx.test.espresso.matcher.ViewMatchers.withClassName;
import static androidx.test.espresso.matcher.ViewMatchers.withContentDescription;
import static androidx.test.espresso.matcher.ViewMatchers.withId;
import static androidx.test.espresso.matcher.ViewMatchers.withText;
import static org.hamcrest.CoreMatchers.equalTo;
import static org.junit.Assert.assertEquals;

import static androidx.test.espresso.assertion.ViewAssertions.matches;

import android.app.DatePickerDialog;
import android.view.View;
import android.widget.DatePicker;

import androidx.fragment.app.DialogFragment;
import androidx.test.espresso.Espresso;
import androidx.test.espresso.UiController;
import androidx.test.espresso.ViewAction;
import androidx.test.espresso.matcher.RootMatchers;
import androidx.test.ext.junit.runners.AndroidJUnit4;
import androidx.test.filters.LargeTest;
import androidx.test.rule.ActivityTestRule;

import com.example.mapbox2.R;
import com.example.trainingPartner.controller.SignInActivity;
import com.example.trainingPartner.controller.fragment.DatePickerFragment;
import com.example.trainingPartner.utils.Utils;
import com.google.firebase.firestore.FirebaseFirestore;

import org.hamcrest.Matcher;
import org.junit.After;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;

import java.util.Calendar;
import java.util.HashMap;
import java.util.Map;

/**
 * Instrumented test, which will execute on an Android device.
 *
 * @see <a href="http://d.android.com/tools/testing">Testing documentation</a>
 */
@LargeTest
@RunWith(AndroidJUnit4.class)
public class SignInIntegrationTest {
    @Rule
    public ActivityTestRule<SignInActivity> iaActivityTestRule= new ActivityTestRule<SignInActivity>(SignInActivity.class);

    private SignInActivity ia;
    private FirebaseFirestore fb = FirebaseFirestore.getInstance();
    @Before
    public void setUp() throws Exception {
        ia= iaActivityTestRule.getActivity();
        fb.collection("usuarios").document("username").delete();
        Utils.deleteSharedPreferences(ia);
    }

    @After
    public void tearDown() throws Exception {

        fb.collection("usuarios").document("username").delete();
        Utils.deleteSharedPreferences(ia);
        ia=null;
    }

    @Test
    public void allInTheirPlaceTest() {
        onView(withId(R.id.imageViewLogo)).check(matches(isDisplayed()));
        onView(withId(R.id.textViewUserName)).check(matches(isDisplayed()));
        onView(withId(R.id.radioButtonMale)).check(matches(isChecked()));
        onView(withId(R.id.radioButtonFemale)).check(matches(isNotChecked()));
        onView(withText("Registro")).check(matches(isDisplayed()));
        onView(withId(R.id.buttonGoToLogIn)).check(matches(isDisplayed()));
        onView(withId(R.id.buttonSignIn)).check(matches(isDisplayed()));
    }

    @Test
    public void testHasTextError() {
        onView(withId(R.id.buttonSignIn)).perform(click());
        onView(withText("Todos los campos deben de estar completos")).check(matches(isDisplayed()));
        onView(withId(R.id.editTextTextUserName)).perform(typeText("username"));
        onView(withId(R.id.editTextTextPersonName)).perform(typeText("Usuario"));
        onView(withId(R.id.editTextTextPersonSurName)).perform(typeText("Aguilar"));
        onView(withId(R.id.editTextDate)).perform(typeText("Holaa"));
        onView(withText("Aceptar")).perform(click());
        Espresso.closeSoftKeyboard();
        onView(withId(R.id.buttonSignIn)).perform(click());
        onView(withText("Todos los campos deben de estar completos")).check(matches(isDisplayed()));
    }

    @Test
    public void testWeightError() throws InterruptedException {

        onView(withId(R.id.editTextTextUserName)).perform(typeText("username"));
        onView(withId(R.id.editTextTextPersonName)).perform(typeText("Usuario"));
        onView(withId(R.id.editTextTextPersonSurName)).perform(typeText("Aguilar"));
        onView(withId(R.id.editTextDate)).perform(typeText("Holaa"));
        onView(withText("Aceptar")).perform(click());
        onView(withId(R.id.editTextWeightSignIn)).perform(typeText("65432"));
        onView(withId(R.id.editTextHeightSignIn)).perform(typeText("175"));
        onView(withId(R.id.editTextTextPassword)).perform(typeText("PedroPascal@"));
        Espresso.closeSoftKeyboard();
        onView(withId(R.id.buttonSignIn)).perform(click());
        Thread.sleep(2000);
        onView(withText("Formato de peso incorrecto, dos decimales como máximo")).check(matches(isDisplayed()));
        onView(withId(R.id.editTextWeightSignIn)).check(matches(withText("")));
        onView(withId(R.id.editTextWeightSignIn)).perform(typeText("65.432"));
        Espresso.closeSoftKeyboard();
        onView(withId(R.id.buttonSignIn)).perform(click());
        Thread.sleep(2000);
        onView(withText("Formato de peso incorrecto, dos decimales como máximo")).check(matches(isDisplayed()));
        onView(withId(R.id.editTextWeightSignIn)).check(matches(withText("")));
        onView(withId(R.id.editTextWeightSignIn)).perform(typeText("2"));
        Espresso.closeSoftKeyboard();
        onView(withId(R.id.buttonSignIn)).perform(click());
        Thread.sleep(2000);
        onView(withText("Peso incorrecto")).check(matches(isDisplayed()));
        onView(withId(R.id.editTextWeightSignIn)).check(matches(withText("")));
        onView(withId(R.id.editTextWeightSignIn)).perform(typeText("65.43"));
        Espresso.closeSoftKeyboard();
        onView(withId(R.id.buttonSignIn)).perform(click());
        Thread.sleep(3000);
        onView(withText("Usuario Aguilar")).check(matches(isDisplayed()));
    }

    @Test
    public void testHeightError() throws InterruptedException {

        onView(withId(R.id.editTextTextUserName)).perform(typeText("username"));
        onView(withId(R.id.editTextTextPersonName)).perform(typeText("Usuario"));
        onView(withId(R.id.editTextTextPersonSurName)).perform(typeText("Aguilar"));
        onView(withId(R.id.editTextDate)).perform(typeText("Holaa"));
        onView(withText("Aceptar")).perform(click());
        onView(withId(R.id.editTextWeightSignIn)).perform(typeText("65"));
        onView(withId(R.id.editTextHeightSignIn)).perform(typeText("17"));
        onView(withId(R.id.editTextTextPassword)).perform(typeText("PedroPascal@"));
        Espresso.closeSoftKeyboard();
        onView(withId(R.id.buttonSignIn)).perform(click());
        Thread.sleep(2000);
        onView(withText("La altura no es correcta")).check(matches(isDisplayed()));
        onView(withId(R.id.editTextHeightSignIn)).check(matches(withText("")));
        onView(withId(R.id.editTextHeightSignIn)).perform(typeText("500"));
        Espresso.closeSoftKeyboard();
        onView(withId(R.id.buttonSignIn)).perform(click());
        Thread.sleep(2000);
        onView(withText("La altura no es correcta")).check(matches(isDisplayed()));
        onView(withId(R.id.editTextHeightSignIn)).check(matches(withText("")));
        onView(withId(R.id.editTextHeightSignIn)).perform(typeText("175"));
        Espresso.closeSoftKeyboard();
        onView(withId(R.id.buttonSignIn)).perform(click());
        Thread.sleep(3000);
        onView(withText("Usuario Aguilar")).check(matches(isDisplayed()));
    }

    @Test
    public void testUserIdentificado() throws InterruptedException {
        Map<String, Object> ex = new HashMap<>();
        ex.put("Hola", 12);
        fb.collection("usuarios").document("usuarioPrueba").set(ex);
        onView(withId(R.id.editTextTextUserName)).perform(typeText("usuarioPrueba"));
        onView(withId(R.id.editTextTextPersonName)).perform(typeText("Usuario"));
        onView(withId(R.id.editTextTextPersonSurName)).perform(typeText("Aguilar"));
        onView(withId(R.id.editTextDate)).perform(click());
        onView(withText("Aceptar")).perform(click());
        onView(withId(R.id.editTextWeightSignIn)).perform(typeText("65"));
        onView(withId(R.id.editTextHeightSignIn)).perform(typeText("175"));
        onView(withId(R.id.editTextTextPassword)).perform(typeText("PedroPascal@"));
        Espresso.closeSoftKeyboard();
        onView(withId(R.id.buttonSignIn)).perform(click());
        Thread.sleep(2000);
        onView(withText("El nombre de usuario ya está registrado en el sistema")).check(matches(isDisplayed()));
        onView(withId(R.id.editTextTextUserName)).check(matches(withText("")));
        onView(withId(R.id.editTextTextUserName)).perform(typeText("username"));
        Espresso.closeSoftKeyboard();
        onView(withId(R.id.buttonSignIn)).perform(click());
        Thread.sleep(2000);
        onView(withText("Usuario Aguilar")).check(matches(isDisplayed()));
        fb.collection("usuarios").document("usuarioPrueba").delete();
    }

    @Test
    public void testPassword() throws InterruptedException {;
        onView(withId(R.id.editTextTextUserName)).perform(typeText("username"));
        onView(withId(R.id.editTextTextPersonName)).perform(typeText("Usuario"));
        onView(withId(R.id.editTextTextPersonSurName)).perform(typeText("Aguilar"));
        onView(withId(R.id.editTextDate)).perform(click());
        onView(withText("Aceptar")).perform(click());
        onView(withId(R.id.editTextWeightSignIn)).perform(typeText("65"));
        onView(withId(R.id.editTextHeightSignIn)).perform(typeText("175"));
        errorEIntentar("P_edr", "La contraseña debe de contener 8 caracteres como mínimo");
        errorEIntentar("p_edropascal", "La contraseña debe de contener caracteres en mayusculas y minusculas");
        errorEIntentar("Pedropascal", "La contraseña debe de contener al menos un caracter especial (@#$%^&+=!_-)");
        errorEIntentar("Pedr_pas cal", "La contraseña contiene algun caracter vacío");
        onView(withId(R.id.editTextTextPassword)).perform(typeText("Pedropas_cal"));
        Espresso.closeSoftKeyboard();
        onView(withId(R.id.buttonSignIn)).perform(click());
        Thread.sleep(2000);
        onView(withText("Usuario Aguilar")).check(matches(isDisplayed()));
    }

    @Test
    public void testMoveToLogin() throws InterruptedException {;
        onView(withId(R.id.buttonGoToLogIn)).check(matches(isDisplayed()));
        onView(withId(R.id.buttonGoToLogIn)).perform(click());
        onView(withText("Inicio de sesión")).check(matches(isDisplayed()));
        onView(withId(R.id.buttonGoToSigIn)).check(matches(isDisplayed()));
        onView(withId(R.id.buttonGoToSigIn)).perform(click());
        onView(withText("Registro")).check(matches(isDisplayed()));
        onView(withId(R.id.editTextTextPassword)).check(matches(withText("")));
        onView(withId(R.id.editTextTextUserName)).check(matches(withText("")));
    }

    private void errorEIntentar(String pass, String message) throws InterruptedException {
        onView(withId(R.id.editTextTextPassword)).perform(typeText(pass));
        Espresso.closeSoftKeyboard();
        onView(withId(R.id.buttonSignIn)).perform(click());
        Thread.sleep(2000);
        onView(withText(message)).check(matches(isDisplayed()));
        onView(withId(R.id.editTextTextPassword)).check(matches(withText("")));
    }


}