package com.example.trainingPartner.integration;

import static androidx.test.espresso.Espresso.onView;
import static androidx.test.espresso.action.ViewActions.click;
import static androidx.test.espresso.assertion.ViewAssertions.doesNotExist;
import static androidx.test.espresso.assertion.ViewAssertions.matches;
import static androidx.test.espresso.matcher.ViewMatchers.isDisplayed;
import static androidx.test.espresso.matcher.ViewMatchers.withId;
import static androidx.test.espresso.matcher.ViewMatchers.withText;
import static org.hamcrest.CoreMatchers.containsString;
import static org.junit.Assert.assertEquals;

import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;

import androidx.test.espresso.Espresso;
import androidx.test.ext.junit.runners.AndroidJUnit4;
import androidx.test.rule.ActivityTestRule;

import com.example.mapbox2.R;
import com.example.trainingPartner.controller.SplashActivity;
import com.example.trainingPartner.controller.TrainDetailsActivity;
import com.example.trainingPartner.dto.PointDTO;
import com.example.trainingPartner.dto.UserInformationDTO;
import com.example.trainingPartner.dto.activityDto.LocationActivityDTO;
import com.example.trainingPartner.utils.Utils;
import com.google.firebase.firestore.CollectionReference;
import com.google.firebase.firestore.FirebaseFirestore;
import com.mapbox.geojson.Point;

import org.junit.After;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Instrumented test, which will execute on an Android device.
 *
 * @see <a href="http://d.android.com/tools/testing">Testing documentation</a>
 */
@RunWith(AndroidJUnit4.class)
public class TrainDetailsActivityIntegrationTest {

    @Rule
    public ActivityTestRule<SplashActivity> signRule = new ActivityTestRule<SplashActivity>(SplashActivity.class);

    @Rule
    public ActivityTestRule<TrainDetailsActivity> trainRule = new ActivityTestRule<TrainDetailsActivity>(TrainDetailsActivity.class, false, false);

    private FirebaseFirestore fb = FirebaseFirestore.getInstance();
    private SplashActivity act;

    private TrainDetailsActivity details;
    private UserInformationDTO user;
    private Context context;
    private Calendar calendar = Calendar.getInstance();
    private CollectionReference reference = fb.collection("usuarios").document("usuarioNoExistente").collection("año").
            document(calendar.get(Calendar.YEAR)+"").collection(calendar.get(Calendar.MONTH)+"");

    @Before
    public void setUp(){

        act = signRule.getActivity();
        SharedPreferences userPref = act.getApplicationContext().getSharedPreferences("users",Context.MODE_PRIVATE);
        SharedPreferences.Editor edit = userPref.edit();
        edit.putString("username", "usuarioNoExistente");
        edit.apply();
        user = new UserInformationDTO("Pedro","usuarioNoExistente","Pruebas", new Date(), 65.7, 175, true, "default", "ContraseñaPrueba@");
        fb.collection("usuarios").document("usuarioNoExistente").set(user.getInformation());



    }
    @After
    public void tearDown(){
        //trainRule.finishActivity();
        Utils.deleteSharedPreferences(details);
        Utils.deleteCollection( reference.document(calendar.get(Calendar.DAY_OF_MONTH)+"").collection("actividades"));
        Utils.deleteCollection(reference);
        fb.collection("usuarios").document("usuarioNoExistente").delete();
        details= null;

    }

    @Test
    public void allInTheirPlaceTest() throws InterruptedException {
        List<PointDTO> points = new ArrayList<>();
        points.add(new PointDTO(43.5378458, -5.6390675));
        Intent intent = new Intent(act.getApplicationContext(), TrainDetailsActivity.class);
        LocationActivityDTO dto = new LocationActivityDTO(points,Double.valueOf(1), calendar);
        intent.putExtra("hora", dto.getHour());
        intent.putExtra("year",dto.getYear());
        intent.putExtra("month",dto.getMonth());
        intent.putExtra("day",dto.getDay());
        reference.document(calendar.get(Calendar.DAY_OF_MONTH)+"").collection("actividades").document(dto.getHour()).set(dto.setValuesToSave());
        trainRule.launchActivity(intent);
        details = trainRule.getActivity();
        onView(withId(R.id.mapView)).check(matches(isDisplayed()));
        onView(withId(R.id.textViewDistancia)).check(matches(isDisplayed()));
        onView(withId(R.id.textViewTiempo)).check(matches(isDisplayed()));
        onView(withId(R.id.textViewVelocidad)).check(matches(isDisplayed()));
        onView(withId(R.id.textViewVelocityValueDetails)).check(matches(isDisplayed()));
        onView(withId(R.id.buttonGuardar)).check(matches(isDisplayed()));
        onView(withId(R.id.buttonBorrar)).check(matches(isDisplayed()));
        Thread.sleep(1000);
        onView(withText("0.0 km/h")).check(matches(isDisplayed()));
        onView(withText("0.0 km")).check(matches(isDisplayed()));
        onView(withText("00:00:01")).check(matches(isDisplayed()));
        onView(withText("--:-- min/km")).check(matches(isDisplayed()));


    }

    @Test
    public void testDataWithManyPointsIntegration() throws InterruptedException {
        List<PointDTO> points = new ArrayList<>();
        points.add(new PointDTO(43.5378458, -5.6390675));
        points.add(new PointDTO(43.5379317, -5.6397083));
        points.add(new PointDTO(43.5378458, -5.6390675));
        points.add(new PointDTO(43.5379317, -5.6397083));
        Map<String, Double> kilometers = new HashMap<>();
        kilometers.put("1", Double.valueOf(286));
        kilometers.put("2", Double.valueOf(175));
        Intent intent = new Intent(act.getApplicationContext(), TrainDetailsActivity.class);
        LocationActivityDTO dto = new LocationActivityDTO(points,kilometers, Double.valueOf(1.63), Double.valueOf(461), calendar);
        intent.putExtra("hora", dto.getHour());
        intent.putExtra("year",dto.getYear());
        intent.putExtra("month",dto.getMonth());
        intent.putExtra("day",dto.getDay());
        reference.document(calendar.get(Calendar.DAY_OF_MONTH)+"").collection("actividades").document(dto.getHour()).set(dto.setValuesToSave());
        trainRule.launchActivity(intent);
        details = trainRule.getActivity();
        onView(withId(R.id.mapView)).check(matches(isDisplayed()));
        onView(withId(R.id.textViewVelocityValueDetails)).check(matches(isDisplayed()));
        Thread.sleep(1000);
        onView(withText("04:37 min/km")).check(matches(isDisplayed()));
        onView(withText("1.63 km")).check(matches(isDisplayed()));
        onView(withText("04:46 min/km")).check(matches(isDisplayed()));
        onView(withText("04:42 min/km")).check(matches(isDisplayed()));

    }

    @Test
    public void testDataWithStopsIntegration() throws InterruptedException {
        List<PointDTO> points = new ArrayList<>();
        points.add(new PointDTO(0, 0));
        points.add(new PointDTO(43.5378458, -5.6390675));
        points.add(new PointDTO(43.5379317, -5.6397083));
        points.add(new PointDTO(0, 0));
        points.add(new PointDTO(43.5378458, -5.6390675));
        points.add(new PointDTO(43.5379317, -5.6397083));
        points.add(new PointDTO(0, 0));
        points.add(new PointDTO(43.5379317, -5.6397083));
        Map<String, Double> kilometers = new HashMap<>();
        kilometers.put("1", Double.valueOf(286));
        kilometers.put("2", Double.valueOf(175));
        Intent intent = new Intent(act.getApplicationContext(), TrainDetailsActivity.class);
        LocationActivityDTO dto = new LocationActivityDTO(points,kilometers, Double.valueOf(1.63), Double.valueOf(461), calendar);
        intent.putExtra("hora", dto.getHour());
        intent.putExtra("year",dto.getYear());
        intent.putExtra("month",dto.getMonth());
        intent.putExtra("day",dto.getDay());
        reference.document(calendar.get(Calendar.DAY_OF_MONTH)+"").collection("actividades").document(dto.getHour()).set(dto.setValuesToSave());
        trainRule.launchActivity(intent);
        details = trainRule.getActivity();
        onView(withId(R.id.mapView)).check(matches(isDisplayed()));
        onView(withId(R.id.textViewVelocityValueDetails)).check(matches(isDisplayed()));
        Thread.sleep(1000);
        onView(withText("04:37 min/km")).check(matches(isDisplayed()));
        onView(withText("1.63 km")).check(matches(isDisplayed()));
        onView(withText("04:46 min/km")).check(matches(isDisplayed()));
        onView(withText("04:42 min/km")).check(matches(isDisplayed()));
    }

    @Test
    public void testDeleteActivity() throws InterruptedException {
        List<PointDTO> points = new ArrayList<>();
        points.add(new PointDTO(43.5378458, -5.6390675));
        points.add(new PointDTO(43.5379317, -5.6397083));
        points.add(new PointDTO(43.5378458, -5.6390675));
        points.add(new PointDTO(43.5379317, -5.6397083));
        Map<String, Double> kilometers = new HashMap<>();
        kilometers.put("1", Double.valueOf(286));
        kilometers.put("2", Double.valueOf(175));
        Intent intent = new Intent(act.getApplicationContext(), TrainDetailsActivity.class);
        LocationActivityDTO dto = new LocationActivityDTO(points,kilometers, Double.valueOf(1.63), Double.valueOf(461), calendar);
        intent.putExtra("hora", dto.getHour());
        intent.putExtra("year",dto.getYear());
        intent.putExtra("month",dto.getMonth());
        intent.putExtra("day",dto.getDay());
        reference.document(calendar.get(Calendar.DAY_OF_MONTH)+"").collection("actividades").document(dto.getHour()).set(dto.setValuesToSave());
        trainRule.launchActivity(intent);
        details = trainRule.getActivity();
        onView(withId(R.id.mapView)).check(matches(isDisplayed()));
        onView(withId(R.id.textViewVelocityValueDetails)).check(matches(isDisplayed()));
        onView(withId(R.id.buttonBorrar)).perform(click());
        Thread.sleep(500);
        onView(withText("ACEPTAR")).perform(click());
        Thread.sleep(5000);
        onView(withText(containsString("AUTOMÁTICO"))).check(doesNotExist());

    }

    @Test
    public void testSaveActivity() throws InterruptedException {
        List<PointDTO> points = new ArrayList<>();
        points.add(new PointDTO(43.5378458, -5.6390675));
        points.add(new PointDTO(43.5379317, -5.6397083));
        points.add(new PointDTO(43.5378458, -5.6390675));
        points.add(new PointDTO(43.5379317, -5.6397083));
        Map<String, Double> kilometers = new HashMap<>();
        kilometers.put("1", Double.valueOf(286));
        kilometers.put("2", Double.valueOf(175));
        Intent intent = new Intent(act.getApplicationContext(), TrainDetailsActivity.class);
        LocationActivityDTO dto = new LocationActivityDTO(points,kilometers, Double.valueOf(1.63), Double.valueOf(461), calendar);
        intent.putExtra("hora", dto.getHour());
        intent.putExtra("year",dto.getYear());
        intent.putExtra("month",dto.getMonth());
        intent.putExtra("day",dto.getDay());
        reference.document(calendar.get(Calendar.DAY_OF_MONTH)+"").collection("actividades").document(dto.getHour()).set(dto.setValuesToSave());
        trainRule.launchActivity(intent);
        details = trainRule.getActivity();
        onView(withId(R.id.mapView)).check(matches(isDisplayed()));
        onView(withId(R.id.textViewVelocityValueDetails)).check(matches(isDisplayed()));
        onView(withId(R.id.buttonGuardar)).perform(click());
        Thread.sleep(5000);
        onView(withText(containsString("AUTOMÁTICO"))).check(matches(isDisplayed()));

    }

    @Test
    public void testOnBackPressedIntegration() throws InterruptedException {
        List<PointDTO> points = new ArrayList<>();
        points.add(new PointDTO(43.5378458, -5.6390675));
        points.add(new PointDTO(43.5379317, -5.6397083));
        points.add(new PointDTO(43.5378458, -5.6390675));
        points.add(new PointDTO(43.5379317, -5.6397083));
        Map<String, Double> kilometers = new HashMap<>();
        kilometers.put("1", Double.valueOf(286));
        kilometers.put("2", Double.valueOf(175));
        Intent intent = new Intent(act.getApplicationContext(), TrainDetailsActivity.class);
        LocationActivityDTO dto = new LocationActivityDTO(points,kilometers, Double.valueOf(1.63), Double.valueOf(461), calendar);
        intent.putExtra("hora", dto.getHour());
        intent.putExtra("year",dto.getYear());
        intent.putExtra("month",dto.getMonth());
        intent.putExtra("day",dto.getDay());
        reference.document(calendar.get(Calendar.DAY_OF_MONTH)+"").collection("actividades").document(dto.getHour()).set(dto.setValuesToSave());
        trainRule.launchActivity(intent);
        details = trainRule.getActivity();
        onView(withId(R.id.mapView)).check(matches(isDisplayed()));
        onView(withId(R.id.textViewVelocityValueDetails)).check(matches(isDisplayed()));
        Espresso.pressBack();
        Thread.sleep(5000);
        onView(withText(containsString("AUTOMÁTICO"))).check(matches(isDisplayed()));

    }



}