package com.example.trainingPartner.unitary;


import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

import static java.lang.Thread.sleep;

import android.app.ProgressDialog;
import android.content.Context;
import android.content.SharedPreferences;
import android.os.Bundle;

import androidx.fragment.app.FragmentTransaction;
import androidx.fragment.app.testing.FragmentScenario;
import androidx.lifecycle.Lifecycle;
import androidx.test.ext.junit.runners.AndroidJUnit4;
import androidx.test.platform.app.InstrumentationRegistry;
import androidx.test.rule.ActivityTestRule;

import com.example.mapbox2.R;
import com.example.trainingPartner.controller.MainActivity;
import com.example.trainingPartner.controller.SignInActivity;
import com.example.trainingPartner.controller.fragment.HistoryFragment;
import com.example.trainingPartner.dto.UserInformationDTO;
import com.example.trainingPartner.utils.Utils;
import com.google.android.gms.tasks.Task;
import com.google.api.core.ApiFuture;
import com.google.firebase.firestore.CollectionReference;
import com.google.firebase.firestore.DocumentReference;
import com.google.firebase.firestore.DocumentSnapshot;
import com.google.firebase.firestore.FirebaseFirestore;
import com.google.firebase.firestore.QueryDocumentSnapshot;
import com.google.firebase.firestore.QuerySnapshot;
import com.prolificinteractive.materialcalendarview.CalendarDay;

import org.junit.After;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.TestRule;
import org.junit.runner.Description;
import org.junit.runner.RunWith;
import org.junit.runners.model.Statement;

import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;


/**
 * Instrumented test, which will execute on an Android device.
 *
 * @see <a href="http://d.android.com/tools/testing">Testing documentation</a>
 */
@RunWith(AndroidJUnit4.class)
public class HistoryFragmentTest {


    @Rule
    public ActivityTestRule<SignInActivity> myCustom = new ActivityTestRule<SignInActivity>(SignInActivity.class);

    FragmentScenario<HistoryFragment> fragmentScenario ;

    private static SignInActivity si;
    private FirebaseFirestore fb= FirebaseFirestore.getInstance();;
    private SharedPreferences.Editor edit;

    private Calendar calendar = Calendar.getInstance();
    private CollectionReference reference = fb.collection("usuarios").document("usuarioNoExistente").collection("año").
            document(calendar.get(Calendar.YEAR)+"").collection(calendar.get(Calendar.MONTH)+"");




    @Before
    public void setUp() throws Exception {
        si = myCustom.getActivity();
        si.userRegistered("usuarioNoExistente");
        si.registerUser(new UserInformationDTO("Pedro","usuarioNoExistente","Pruebas", new Date(), 65.7, 175, true, "default", "ContraseñaPrueba@"));
        SharedPreferences userPref = si.getApplicationContext().getSharedPreferences("users",Context.MODE_PRIVATE);
        edit = userPref.edit();
        edit.putString("username", "usuarioNoExistente");
        edit.apply();
        fragmentScenario= FragmentScenario.launchInContainer(HistoryFragment.class);
    }


    @After
    public void tearDown(){
        Utils.deleteCollection(reference);
        Utils.deleteCollection(fb.collection("usuarios").document("usuarioNoExistente").collection("año"));
        fb.collection("usuarios").document("usuarioNoExistente").delete();
        Utils.deleteSharedPreferences(si);
        fragmentScenario.recreate();
        si=null;
    }
    /**
     * No hay ningún día activado
     */
    @Test
    public void desActivateDatesTestWithoutDays() throws InterruptedException {
        Thread.sleep(10000);
         fragmentScenario.onFragment(fragment -> {
         Context fragmentContext = fragment.requireContext();
         //fragmentScenario1.moveToState(Lifecycle.State.CREATED);

         assertEquals(0, fragment.getDaysUsed());
         });





    }

    /**
     * SE añaden dos días y solo hay 2 días activados
     */
    @Test
    public void desActivateDatesTest() throws InterruptedException {
        HashMap<String, Integer> steps = new HashMap<>();
        steps.put("steps", 12);
        steps.put("calories", 15);
        int day =2;
        if(calendar.get(Calendar.DAY_OF_MONTH) == 2){
                day = 4;
        }
        reference.document(day+"").set(steps);
        reference.document((day-1)+"").set(steps);
        Thread.sleep(5000);

        fragmentScenario.onFragment(fragment -> {
            // Accede al fragmento y verifica el comportamiento esperado utilizando el contexto de la actividad
            Context fragmentContext = fragment.requireContext();
            //fragmentScenario.moveToState(Lifecycle.State.CREATED);
            synchronized (fragment){
                assertEquals(2, fragment.getDaysUsed());
            }


        });


    }

    /**
     * Comprobar que si solo se tiene en cuenta el día actual y otro, en el calendario
     * sólo aparecerá el otro día.
     */
    @Test
    public void testDesActivateDatesToday() throws InterruptedException {
        HashMap<String, Integer> steps = new HashMap<>();
        steps.put("steps", 12);
        steps.put("calories", 15);
        int day =0;
        if(calendar.get(Calendar.DAY_OF_MONTH) == 1){
            day = calendar.get(Calendar.DAY_OF_MONTH)-1;
        }
        reference.document(day+"").set(steps);
        reference.document(calendar.get(Calendar.DAY_OF_MONTH)+"").set(steps);

        Thread.sleep(5000);
        fragmentScenario.onFragment(fragment -> {
            // Accede al fragmento y verifica el comportamiento esperado utilizando el contexto de la actividad
            Context fragmentContext = fragment.requireContext();
            synchronized (fragment){
                assertEquals(1, fragment.getDaysUsed());
            }
        });

    }









}