package com.example.trainingPartner.unitary;

import static org.junit.Assert.assertEquals;

import android.content.Context;
import android.content.SharedPreferences;
import android.widget.EditText;

import androidx.fragment.app.testing.FragmentScenario;
import androidx.lifecycle.Lifecycle;
import androidx.test.core.app.ActivityScenario;
import androidx.test.ext.junit.runners.AndroidJUnit4;
import androidx.test.platform.app.InstrumentationRegistry;
import androidx.test.rule.ActivityTestRule;

import com.example.trainingPartner.controller.MainActivity;
import com.example.trainingPartner.controller.SignInActivity;
import com.example.trainingPartner.controller.SplashActivity;
import com.example.trainingPartner.controller.fragment.HistoryFragment;
import com.example.trainingPartner.dto.UserInformationDTO;
import com.example.trainingPartner.model.DailyInformationModel;
import com.example.trainingPartner.model.MyCallbBack;
import com.example.trainingPartner.utils.Utils;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.firebase.firestore.CollectionReference;
import com.google.firebase.firestore.DocumentReference;
import com.google.firebase.firestore.DocumentSnapshot;
import com.google.firebase.firestore.FirebaseFirestore;
import com.google.firebase.firestore.QuerySnapshot;

import org.junit.After;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.TestRule;
import org.junit.runner.Description;
import org.junit.runner.RunWith;
import org.junit.runners.model.Statement;

import java.util.Calendar;
import java.util.Date;

/**
 * Instrumented test, which will execute on an Android device.
 *
 * @see <a href="http://d.android.com/tools/testing">Testing documentation</a>
 */
@RunWith(AndroidJUnit4.class)
public class MainActivityTest {

    @Rule
    public ActivityTestRule<SplashActivity> signRule = new ActivityTestRule<SplashActivity>(SplashActivity.class);

    @Rule
    public ActivityTestRule<MainActivity> mainRule = new ActivityTestRule<MainActivity>(MainActivity.class, false, false);

    private FirebaseFirestore fb = FirebaseFirestore.getInstance();

    private MainActivity mainActivity;
    private UserInformationDTO user;
    private Context context;
    private Calendar calendar = Calendar.getInstance();
    private CollectionReference reference = fb.collection("usuarios").document("usuarioNoExistente").collection("año").
            document(calendar.get(Calendar.YEAR)+"").collection(calendar.get(Calendar.MONTH)+"");

    @Before
    public void setUp(){
        SplashActivity act = signRule.getActivity();
        SharedPreferences userPref = act.getApplicationContext().getSharedPreferences("users",Context.MODE_PRIVATE);
        SharedPreferences.Editor edit = userPref.edit();
        edit.putString("username", "usuarioNoExistente");
        edit.apply();
        user = new UserInformationDTO("Pedro","usuarioNoExistente","Pruebas", new Date(), 65.7, 175, true, "default", "ContraseñaPrueba@");
        fb.collection("usuarios").document("usuarioNoExistente").set(user.getInformation());
        SharedPreferences stepsPref = act.getApplicationContext().getSharedPreferences("stepCounter",Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = stepsPref.edit();
        editor.putInt("stepCount", 15);
        editor.apply();



    }
    @After
    public void tearDown(){
        mainRule.finishActivity();
        Utils.deleteCollection(reference);
        fb.collection("usuarios").document("usuarioNoExistente").delete();
        Utils.deleteSharedPreferences(mainActivity);
        fb.collection("registro").document("usuarioNoExistente").delete();
        Utils.deleteStepsSharedPreferences(mainActivity);
        mainActivity=null;

    }

    @Test
    public void testLogOut(){
        mainRule.launchActivity(null);
        mainActivity = mainRule.getActivity();
        long steps = mainActivity.saveStepsLogOut();
        reference.document(calendar.get(Calendar.DAY_OF_MONTH)+"").get().addOnSuccessListener(new OnSuccessListener<DocumentSnapshot>() {
            @Override
            public void onSuccess(DocumentSnapshot documentSnapshot) {
                assertEquals(steps, documentSnapshot.get("steps"));
            }
    });

    }
    @Test
    public void testSaveRegistry() {
        mainRule.launchActivity(null);
        mainActivity = mainRule.getActivity();
        DailyInformationModel model = new DailyInformationModel("usuarioNoExistente");
        model.getLastConnection(new MyCallbBack<Calendar>() {
            @Override
            public void onCallback(Calendar element) {
                assertEquals(element, Calendar.getInstance());
            }
        });

    }

    /**
     * Este test comprueba que si se inicia la aplicación un día más tarde, se actualizará el registro
     * y los pasos se iniciarán a 0.
     * Me dio problemas porque tenía mal puesto el saveRegistry en el mainActivity.
     * Ahora esta dentro del callback
     * @throws InterruptedException
     */
    @Test
    public void testSaveStepsAndRegistry() throws InterruptedException {
        fb.collection("registro").document("usuarioNoExistente").delete();
        //Se inicia con 15 pasos en la aplicación.
        //Peeero el registro esta en el día anterior
        DailyInformationModel model = new DailyInformationModel("usuarioNoExistente");
        model.saveRegistry(calendar.get(Calendar.YEAR), calendar.get(Calendar.MONTH), calendar.get(Calendar.DAY_OF_MONTH)-1);
        Thread.sleep(10000);
        mainRule.launchActivity(null);
        mainActivity = mainRule.getActivity();

        long steps = mainActivity.saveStepsLogOut();
        Thread.sleep(5000);
        reference.document(calendar.get(Calendar.DAY_OF_MONTH)+"").get().addOnSuccessListener(new OnSuccessListener<DocumentSnapshot>() {
            @Override
            public void onSuccess(DocumentSnapshot documentSnapshot) {
                assertEquals(steps, documentSnapshot.get("steps"));

                assertEquals((long)0, documentSnapshot.get("steps"));
            }
        });
        int day = calendar.get(Calendar.DAY_OF_MONTH);
        reference.document((day-1)+"").get().addOnSuccessListener(new OnSuccessListener<DocumentSnapshot>() {
            @Override
            public void onSuccess(DocumentSnapshot documentSnapshot) {
                assertEquals((long)15, documentSnapshot.get("steps"));
                mainRule.finishActivity();
            }
        });

    }

}