package com.example.trainingPartner.unitary;

import static org.junit.Assert.assertEquals;

import android.content.Context;
import android.content.SharedPreferences;

import androidx.fragment.app.testing.FragmentScenario;
import androidx.test.ext.junit.runners.AndroidJUnit4;
import androidx.test.rule.ActivityTestRule;

import com.example.trainingPartner.controller.SignInActivity;
import com.example.trainingPartner.controller.fragment.HistoryFragment;
import com.example.trainingPartner.controller.fragment.MainWindowFragment;
import com.example.trainingPartner.dto.UserInformationDTO;
import com.example.trainingPartner.dto.activityDto.ActivityDTO;
import com.example.trainingPartner.dto.activityDto.ManualActivityDTO;
import com.example.trainingPartner.utils.Utils;
import com.google.firebase.firestore.CollectionReference;
import com.google.firebase.firestore.FirebaseFirestore;

import org.junit.After;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;

import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;


/**
 * Instrumented test, which will execute on an Android device.
 *
 * @see <a href="http://d.android.com/tools/testing">Testing documentation</a>
 */
@RunWith(AndroidJUnit4.class)
public class MainWindowFragmentTest {


    @Rule
    public ActivityTestRule<SignInActivity> myCustom = new ActivityTestRule<SignInActivity>(SignInActivity.class);

    FragmentScenario<MainWindowFragment> fragmentScenario ;

    private static SignInActivity si;
    private FirebaseFirestore fb= FirebaseFirestore.getInstance();;
    private SharedPreferences.Editor edit;

    private Calendar calendar = Calendar.getInstance();
    private CollectionReference reference = fb.collection("usuarios").document("usuarioNoExistente").collection("año").
            document(calendar.get(Calendar.YEAR)+"").collection(calendar.get(Calendar.MONTH)+"").document(calendar.get(Calendar.DAY_OF_MONTH)+"").collection("actividades");




    @Before
    public void setUp() throws Exception {
        si = myCustom.getActivity();
        si.userRegistered("usuarioNoExistente");
        si.registerUser(new UserInformationDTO("Pedro","usuarioNoExistente","Pruebas", new Date(), 65.7, 175, true, "default", "ContraseñaPrueba@"));
        SharedPreferences userPref = si.getApplicationContext().getSharedPreferences("users",Context.MODE_PRIVATE);
        edit = userPref.edit();
        edit.putString("username", "usuarioNoExistente");
        edit.apply();

        fragmentScenario= FragmentScenario.launchInContainer(MainWindowFragment.class);
    }


    @After
    public void tearDown(){
        myCustom.finishActivity();
        Utils.deleteCollection(reference);
        Utils.deleteCollection(fb.collection("usuarios").document("usuarioNoExistente").collection("año"));
        fb.collection("usuarios").document("usuarioNoExistente").delete();
        Utils.deleteSharedPreferences(si);
        si=null;
    }
    /**
     * No hay ningún día activado
     */
    @Test
    public void getInfOfTheUser() throws InterruptedException {
        Thread.sleep(5000);
        fragmentScenario.onFragment(fragment -> {
            Context fragmentContext = fragment.requireContext();
            UserInformationDTO dto = fragment.getUserInfo();
            assertEquals("Pedro", dto.getName());
            assertEquals("Pruebas", dto.getSurname());
            assertEquals((Double)65.7, dto.getWeight());
            assertEquals((Double)1.75, dto.getHeight());
            assertEquals("default", dto.getPhoto());
            assertEquals(true, dto.isSex());
        });

    }
    /**
     * No hay ningún día activado
     */
    @Test
    public void getActivitiesOfTheUser() throws InterruptedException {
        ActivityDTO actividad = new ManualActivityDTO(1,500,Calendar.getInstance());
        reference.document(actividad.getFormatHour()).set(actividad.setValuesToSave());
        Thread.sleep(5000);
        fragmentScenario.onFragment(fragment -> {
            Context fragmentContext = fragment.requireContext();
            assertEquals(1, fragment.getActivities().size());
        });





    }

    /**
     * No hay ningún día activado
     */
    @Test
    public void getActivitiesOfTheUserWithActivities() throws InterruptedException {
        Thread.sleep(5000);
        fragmentScenario.onFragment(fragment -> {
            Context fragmentContext = fragment.requireContext();

            assertEquals(0, fragment.getActivities().size());
        });





    }


}