package com.example.trainingPartner.unitary;

import static org.junit.Assert.assertEquals;

import android.content.Context;
import android.content.SharedPreferences;

import androidx.test.ext.junit.runners.AndroidJUnit4;
import androidx.test.rule.ActivityTestRule;

import com.example.trainingPartner.controller.MainActivity;
import com.example.trainingPartner.controller.ManualRecordActivity;
import com.example.trainingPartner.controller.SplashActivity;
import com.example.trainingPartner.controller.TrainDetailsManualActivity;
import com.example.trainingPartner.dto.UserInformationDTO;
import com.example.trainingPartner.dto.activityDto.ActivityDTO;
import com.example.trainingPartner.dto.activityDto.ManualActivityDTO;
import com.example.trainingPartner.model.DailyInformationModel;
import com.example.trainingPartner.model.MyCallbBack;
import com.example.trainingPartner.model.TrainDetailsModel;
import com.example.trainingPartner.utils.Utils;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.firebase.firestore.CollectionReference;
import com.google.firebase.firestore.DocumentSnapshot;
import com.google.firebase.firestore.FirebaseFirestore;

import org.junit.After;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;

import java.util.Calendar;
import java.util.Date;

/**
 * Instrumented test, which will execute on an Android device.
 *
 * @see <a href="http://d.android.com/tools/testing">Testing documentation</a>
 */
@RunWith(AndroidJUnit4.class)
public class ManualRecordActivityTest {

    @Rule
    public ActivityTestRule<SplashActivity> signRule = new ActivityTestRule<SplashActivity>(SplashActivity.class);

    @Rule
    public ActivityTestRule<ManualRecordActivity> manualRule = new ActivityTestRule<ManualRecordActivity>(ManualRecordActivity.class, false, false);

    private FirebaseFirestore fb = FirebaseFirestore.getInstance();

    private ManualRecordActivity manualRecordActivity;
    private UserInformationDTO user;
    private Context context;
    private Calendar calendar = Calendar.getInstance();
    private CollectionReference reference = fb.collection("usuarios").document("usuarioNoExistente").collection("año").
            document(calendar.get(Calendar.YEAR)+"").collection(calendar.get(Calendar.MONTH)+"");

    @Before
    public void setUp(){
        manualRule.launchActivity(null);
        manualRecordActivity = manualRule.getActivity();
        SplashActivity act = signRule.getActivity();
        SharedPreferences userPref = act.getApplicationContext().getSharedPreferences("users",Context.MODE_PRIVATE);
        SharedPreferences.Editor edit = userPref.edit();
        edit.putString("username", "usuarioNoExistente");
        edit.apply();
        user = new UserInformationDTO("Pedro","usuarioNoExistente","Pruebas", new Date(), 65.7, 175, true, "default", "ContraseñaPrueba@");
        fb.collection("usuarios").document("usuarioNoExistente").set(user.getInformation());



    }
    @After
    public void tearDown(){
        manualRule.finishActivity();
        Utils.deleteCollection( reference.document(calendar.get(Calendar.DAY_OF_MONTH)+"").collection("actividades"));
        Utils.deleteCollection(reference);
        fb.collection("usuarios").document("usuarioNoExistente").delete();
        Utils.deleteSharedPreferences(manualRecordActivity);
        manualRecordActivity= null;

    }

    @Test
    public void testSaveActivity() throws InterruptedException {
        ActivityDTO dto = new ManualActivityDTO(1, 4000, Calendar.getInstance());
        manualRecordActivity.saveActivity(dto);
        Thread.sleep(2000);
        TrainDetailsModel model = new TrainDetailsModel(dto.getYear(), dto.getMonth(), dto.getDay(),dto.getHour(), Utils.getCalendar( dto.getDay(),dto.getMonth(),dto.getYear()), "usuarioNoExistente");

        model.getData(new MyCallbBack<ActivityDTO>() {
            @Override
            public void onCallback(ActivityDTO element) {
                Double d = Double.valueOf(1);
                assertEquals(d,element.distance);
            }
        });
        reference.document(calendar.get(Calendar.DAY_OF_MONTH)+"").collection("actividades").document(dto.getHour()).get().addOnSuccessListener(new OnSuccessListener<DocumentSnapshot>() {
            @Override
            public void onSuccess(DocumentSnapshot documentSnapshot) {
                Double distance = documentSnapshot.getDouble("distance");
                Double d = Double.valueOf(1);
                assertEquals(d, distance);
            }
        });


    }


}