package com.example.trainingPartner.unitary;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import android.content.Context;
import android.content.SharedPreferences;

import androidx.fragment.app.testing.FragmentScenario;
import androidx.test.ext.junit.runners.AndroidJUnit4;
import androidx.test.rule.ActivityTestRule;

import com.example.trainingPartner.controller.ManualRecordActivity;
import com.example.trainingPartner.controller.SignInActivity;
import com.example.trainingPartner.controller.SplashActivity;
import com.example.trainingPartner.controller.fragment.MainWindowFragment;
import com.example.trainingPartner.controller.fragment.PredictionsFragment;
import com.example.trainingPartner.dto.UserInformationDTO;
import com.example.trainingPartner.dto.activityDto.ActivityDTO;
import com.example.trainingPartner.dto.activityDto.ManualActivityDTO;
import com.example.trainingPartner.model.MyCallbBack;
import com.example.trainingPartner.model.PredictionsModel;
import com.example.trainingPartner.utils.Utils;
import com.google.firebase.firestore.CollectionReference;
import com.google.firebase.firestore.FirebaseFirestore;

import org.junit.After;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;

import java.util.Calendar;
import java.util.Date;


/**
 * Instrumented test, which will execute on an Android device.
 *
 * @see <a href="http://d.android.com/tools/testing">Testing documentation</a>
 */
@RunWith(AndroidJUnit4.class)
public class PredictionsFragmentTest {


    @Rule
    public ActivityTestRule<SplashActivity> myCustom = new ActivityTestRule<SplashActivity>(SplashActivity.class);

    @Rule
    public ActivityTestRule<ManualRecordActivity> manualRule = new ActivityTestRule<ManualRecordActivity>(ManualRecordActivity.class, false, false);

    FragmentScenario<PredictionsFragment> fragmentScenario ;

    private FirebaseFirestore fb= FirebaseFirestore.getInstance();;
    private SharedPreferences.Editor edit;

    private Calendar calendar = Calendar.getInstance();
    private CollectionReference reference = fb.collection("usuarios").document("usuarioNoExistente").collection("año").
            document(calendar.get(Calendar.YEAR)+"").collection(calendar.get(Calendar.MONTH)+"").document(calendar.get(Calendar.DAY_OF_MONTH)+"").collection("actividades");

    private SplashActivity act;


    @Before
    public void setUp() throws Exception {
        act = myCustom.getActivity();
        Utils.deleteSharedPreferences(act);
        SharedPreferences userPref = act.getApplicationContext().getSharedPreferences("users",Context.MODE_PRIVATE);
        SharedPreferences.Editor edit = userPref.edit();
        edit.putString("username", "usuarioNoExistente");
        edit.apply();
        UserInformationDTO user = new UserInformationDTO("Pedro","usuarioNoExistente","Pruebas", new Date(), 65.7, 175, true, "default", "ContraseñaPrueba@");
        fb.collection("usuarios").document("usuarioNoExistente").set(user.getInformation());

        fragmentScenario= FragmentScenario.launchInContainer(PredictionsFragment.class);
    }


    @After
    public void tearDown(){
        myCustom.finishActivity();
        Utils.deleteCollection(reference);
        Utils.deleteCollection(fb.collection("usuarios").document("usuarioNoExistente").collection("año"));
        Utils.deleteCollection(fb.collection("usuarios").document("usuarioNoExistente").collection("records"));
        fb.collection("usuarios").document("usuarioNoExistente").delete();
        Utils.deleteSharedPreferences(act);
    }

    @Test
    public void getPredictionsWithoutThreeActivities() throws InterruptedException {
        fragmentScenario.onFragment(fragment -> {
            Context fragmentContext = fragment.requireContext();
            PredictionsModel model = fragment.getModel();
            model.moreThanThreeActivities(new MyCallbBack<Boolean>() {
                @Override
                public void onCallback(Boolean element) {
                    assertFalse(element);
                }
            });

        });


    }

    @Test
    public void getPredictions() throws InterruptedException {
        ManualActivityDTO manual1 = new ManualActivityDTO(3,900, calendar);
        ManualActivityDTO manual2 = new ManualActivityDTO(3,900, calendar);
        ManualActivityDTO manual3 = new ManualActivityDTO(3, 900, calendar);
        manualRule.launchActivity(null);
        ManualRecordActivity manualRecordActivity = manualRule.getActivity();
        manualRecordActivity.saveActivity(manual1);
        Thread.sleep(5000);
        manualRecordActivity.saveActivity(manual2);
        Thread.sleep(5000);
        manualRecordActivity.saveActivity(manual3);
        Thread.sleep(5000);
        fragmentScenario.onFragment(fragment -> {
            Context fragmentContext = fragment.requireContext();
            PredictionsModel model = fragment.getModel();
            model.moreThanThreeActivities(new MyCallbBack<Boolean>() {
                @Override
                public void onCallback(Boolean element) {
                    assertTrue(element);
                }
            });

        });


    }






}