package com.example.trainingPartner.unitary;

import static org.junit.Assert.assertEquals;

import android.content.Context;
import android.content.SharedPreferences;

import androidx.test.ext.junit.runners.AndroidJUnit4;
import androidx.test.rule.ActivityTestRule;

import com.example.trainingPartner.controller.ManualRecordActivity;
import com.example.trainingPartner.controller.RecordActivity;
import com.example.trainingPartner.controller.SplashActivity;
import com.example.trainingPartner.dto.UserInformationDTO;
import com.example.trainingPartner.dto.activityDto.ActivityDTO;
import com.example.trainingPartner.dto.activityDto.ManualActivityDTO;
import com.example.trainingPartner.model.MyCallbBack;
import com.example.trainingPartner.model.TrainDetailsModel;
import com.example.trainingPartner.utils.Utils;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.firebase.firestore.CollectionReference;
import com.google.firebase.firestore.DocumentSnapshot;
import com.google.firebase.firestore.FirebaseFirestore;

import org.junit.After;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;

import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Optional;

/**
 * Instrumented test, which will execute on an Android device.
 *
 * @see <a href="http://d.android.com/tools/testing">Testing documentation</a>
 */
@RunWith(AndroidJUnit4.class)
public class RecordActivityTest {

    @Rule
    public ActivityTestRule<SplashActivity> signRule = new ActivityTestRule<SplashActivity>(SplashActivity.class);

    @Rule
    public ActivityTestRule<RecordActivity> recordRule = new ActivityTestRule<RecordActivity>(RecordActivity.class, false, false);

    private FirebaseFirestore fb = FirebaseFirestore.getInstance();

    private RecordActivity recordActivity;
    private UserInformationDTO user;
    private Context context;
    private Calendar calendar = Calendar.getInstance();
    private CollectionReference reference = fb.collection("usuarios").document("usuarioNoExistente").collection("año").
            document(calendar.get(Calendar.YEAR)+"").collection(calendar.get(Calendar.MONTH)+"");

    @Before
    public void setUp(){
        recordRule.launchActivity(null);
        recordActivity = recordRule.getActivity();
        SplashActivity act = signRule.getActivity();
        SharedPreferences userPref = act.getApplicationContext().getSharedPreferences("users",Context.MODE_PRIVATE);
        SharedPreferences.Editor edit = userPref.edit();
        edit.putString("username", "usuarioNoExistente");
        edit.apply();
        user = new UserInformationDTO("Pedro","usuarioNoExistente","Pruebas", new Date(), 65.7, 175, true, "default", "ContraseñaPrueba@");
        fb.collection("usuarios").document("usuarioNoExistente").set(user.getInformation());



    }
    @After
    public void tearDown(){
        recordRule.finishActivity();
        Utils.deleteCollection( reference.document(calendar.get(Calendar.DAY_OF_MONTH)+"").collection("actividades"));
        Utils.deleteCollection(reference);
        fb.collection("usuarios").document("usuarioNoExistente").delete();
        Utils.deleteSharedPreferences(recordActivity);
        recordActivity= null;

    }


    @Test
    public void testAddTheRestantKilometerLessThanOneKilometer() throws InterruptedException {
        recordActivity.setTime(201);
        HashMap<String, Double> parcials = new HashMap<>();
        HashMap<String,Double> fin = recordActivity.addTheRestantKilometer(0.57, parcials);
        assertEquals(1, fin.size());
        assertEquals(Double.valueOf(201), fin.get("1"));
    }

    @Test
    public void testAddTheRestantKilometerMoreThanOneKilometer() throws InterruptedException {
        recordActivity.setTime(461);
        HashMap<String, Double> parcials = new HashMap<>();
        parcials.put("1", Double.valueOf(286));
        HashMap<String,Double> fin = recordActivity.addTheRestantKilometer(1.63, parcials);
        assertEquals(2, fin.size());
        assertEquals(Double.valueOf(286), fin.get("1"));
        assertEquals(Double.valueOf(175), fin.get("2"));
    }


}