package com.example.trainingPartner.unitary;

import android.content.Context;
import android.content.SharedPreferences;
import android.widget.EditText;

import androidx.test.core.app.ActivityScenario;
import androidx.test.platform.app.InstrumentationRegistry;
import androidx.test.ext.junit.runners.AndroidJUnit4;
import androidx.test.rule.ActivityTestRule;


import org.junit.After;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;

import static org.junit.Assert.*;

import static java.lang.Thread.sleep;

import com.example.trainingPartner.controller.MainActivity;
import com.example.trainingPartner.controller.SignInActivity;
import com.example.trainingPartner.dto.UserInformationDTO;
import com.example.trainingPartner.model.LogInSignInModel;
import com.example.trainingPartner.model.MyCallbBack;
import com.example.trainingPartner.utils.Utils;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.firebase.firestore.DocumentSnapshot;
import com.google.firebase.firestore.FirebaseFirestore;
import com.google.firebase.firestore.QuerySnapshot;

import java.util.Date;
import java.util.List;

/**
 * Instrumented test, which will execute on an Android device.
 *
 * @see <a href="http://d.android.com/tools/testing">Testing documentation</a>
 */
@RunWith(AndroidJUnit4.class)
public class SignInTest {
    @Rule
    public ActivityTestRule<SignInActivity> iaActivityTestRule= new ActivityTestRule<SignInActivity>(SignInActivity.class);

    private SignInActivity ia;
    private FirebaseFirestore fb= FirebaseFirestore.getInstance();;
    @Before
    public void setUp() throws Exception {
        ia= iaActivityTestRule.getActivity();

    }

    @After
    public void tearDown() throws Exception {
        iaActivityTestRule.finishActivity();

        ia=null;
    }

    /**
     * Test de comprobación de las contraseñas.
     */
    @Test
    public void correctPasswordTest() {
        assertEquals(false, ia.correctPassword("hola"));
        assertEquals(false, ia.correctPassword(""));
        assertEquals(false, ia.correctPassword(" "));
        assertEquals(false, ia.correctPassword("alfcoke20"));
        assertEquals(false, ia.correctPassword("alfcoke_20"));
        assertEquals(false, ia.correctPassword("ALFCOKE_20"));
        assertEquals(false, ia.correctPassword("aA_29"));
        assertEquals(true, ia.correctPassword("Alfcoke_20"));
    }

    @Test
    public void hasTextTest(){
        assertEquals(false, Utils.hasStringText(""));
        assertEquals(false, Utils.hasStringText(" "));
        assertEquals(false, Utils.hasStringText(null));
        assertEquals(true, Utils.hasStringText("a"));
        assertEquals(true, Utils.hasStringText(" a"));
    }

    @Test
    public void checkWeightTest(){
        EditText text = new EditText(ia.getApplicationContext());
        assertEquals(false, ia.checkWeight(null));
        text.setText(".12");
        assertEquals(false, ia.checkWeight(text));
        text.setText("12");
        assertEquals(false, ia.checkWeight(text));
        text.setText("65.4");
        assertEquals(true, ia.checkWeight(text));
        text.setText("12345");
        assertEquals(false, ia.checkWeight(text));
        text.setText("123.4");
        assertEquals(true, ia.checkWeight(text));
        text.setText("123.65");
        assertEquals(true, ia.checkWeight(text));
        text.setText("70.654");
        assertEquals(false, ia.checkWeight(text));
        text.setText("70.65");
        assertEquals(true, ia.checkWeight(text));
    }

    @Test
    public void checkHeightTest(){
        EditText text = new EditText(ia.getApplicationContext());
        assertEquals(false, ia.checkHeight(null));
        text.setText(".12");
        assertEquals(false, ia.checkHeight(text));
        text.setText("12");
        assertEquals(false, ia.checkHeight(text));
        text.setText("655");
        assertEquals(false, ia.checkHeight(text));
        text.setText("175");
    }

    /**
     * Devuelve true si el usuario NO esta registrado en el sistema
     */
    @Test
    public void registerUser() throws InterruptedException {
        ia.userRegistered("usuarioNoExistente");
        ia.registerUser(new UserInformationDTO("Pedro","usuarioNoExistente","Pruebas", new Date(), 65.7, 175, true, "default", "ahavhdhgyshagahgdadw"));
        Thread.sleep(5000);
        LogInSignInModel model = new LogInSignInModel("usuarioNoExistente");
        model.getUserNames(new MyCallbBack<List<String>>() {
            @Override
            public void onCallback(List<String> element) {
                assertTrue(element.contains("usuarioNoExistente"));
                fb.collection("usuarios").document("usuarioNoExistente").delete();
            }
        });



    }


}