package com.example.trainingPartner.unitary;

import static org.junit.Assert.assertEquals;

import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;

import androidx.test.ext.junit.runners.AndroidJUnit4;
import androidx.test.rule.ActivityTestRule;

import com.example.trainingPartner.controller.MainActivity;
import com.example.trainingPartner.controller.RecordActivity;
import com.example.trainingPartner.controller.SplashActivity;
import com.example.trainingPartner.controller.TrainDetailsActivity;
import com.example.trainingPartner.dto.PointDTO;
import com.example.trainingPartner.dto.UserInformationDTO;
import com.example.trainingPartner.dto.activityDto.ActivityDTO;
import com.example.trainingPartner.dto.activityDto.LocationActivityDTO;
import com.example.trainingPartner.utils.Utils;
import com.google.firebase.firestore.CollectionReference;
import com.google.firebase.firestore.FirebaseFirestore;
import com.mapbox.geojson.Point;

import org.junit.After;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Instrumented test, which will execute on an Android device.
 *
 * @see <a href="http://d.android.com/tools/testing">Testing documentation</a>
 */
@RunWith(AndroidJUnit4.class)
public class TrainDetailsActivityTest {

    @Rule
    public ActivityTestRule<SplashActivity> signRule = new ActivityTestRule<SplashActivity>(SplashActivity.class);

    @Rule
    public ActivityTestRule<TrainDetailsActivity> trainRule = new ActivityTestRule<TrainDetailsActivity>(TrainDetailsActivity.class, false, false);

    private FirebaseFirestore fb = FirebaseFirestore.getInstance();
    private SplashActivity act;

    private TrainDetailsActivity details;
    private UserInformationDTO user;
    private Context context;
    private Calendar calendar = Calendar.getInstance();
    private CollectionReference reference = fb.collection("usuarios").document("usuarioNoExistente").collection("año").
            document(calendar.get(Calendar.YEAR)+"").collection(calendar.get(Calendar.MONTH)+"");

    @Before
    public void setUp(){

        act = signRule.getActivity();
        SharedPreferences userPref = act.getApplicationContext().getSharedPreferences("users",Context.MODE_PRIVATE);
        SharedPreferences.Editor edit = userPref.edit();
        edit.putString("username", "usuarioNoExistente");
        edit.apply();
        user = new UserInformationDTO("Pedro","usuarioNoExistente","Pruebas", new Date(), 65.7, 175, true, "default", "ContraseñaPrueba@");
        fb.collection("usuarios").document("usuarioNoExistente").set(user.getInformation());



    }
    @After
    public void tearDown(){
        trainRule.finishActivity();
        Utils.deleteCollection( reference.document(calendar.get(Calendar.DAY_OF_MONTH)+"").collection("actividades"));
        Utils.deleteCollection(reference);
        fb.collection("usuarios").document("usuarioNoExistente").delete();
        Utils.deleteSharedPreferences(details);
        details= null;

    }

    @Test
    public void testData() throws InterruptedException {
        List<PointDTO> points = new ArrayList<>();
        points.add(new PointDTO(43.5378458, -5.6390675));
        Intent intent = new Intent(act.getApplicationContext(), TrainDetailsActivity.class);
        LocationActivityDTO dto = new LocationActivityDTO(points,Double.valueOf(1), calendar);
        intent.putExtra("hora", dto.getHour());
        intent.putExtra("year",dto.getYear());
        intent.putExtra("month",dto.getMonth());
        intent.putExtra("day",dto.getDay());
        reference.document(calendar.get(Calendar.DAY_OF_MONTH)+"").collection("actividades").document(dto.getHour()).set(dto.setValuesToSave());
        trainRule.launchActivity(intent);
        details = trainRule.getActivity();
        LocationActivityDTO dtoApp = details.getDto();
        assertEquals(Double.valueOf(1), dtoApp.time);
        assertEquals(points.size(), dtoApp.coordinates.size());
        assertEquals(points.get(0), dtoApp.coordinates.get(0));
        assertEquals(Double.valueOf(0), dtoApp.distance);

    }

    @Test
    public void testDataWithManyPoints() throws InterruptedException {
        List<PointDTO> points = new ArrayList<>();
        points.add(new PointDTO(43.5378458, -5.6390675));
        points.add(new PointDTO(43.5379317, -5.6397083));
        points.add(new PointDTO(43.5378458, -5.6390675));
        points.add(new PointDTO(43.5379317, -5.6397083));
        Map<String, Double> kilometers = new HashMap<>();
        kilometers.put("1", Double.valueOf(286));
        kilometers.put("2", Double.valueOf(175));
        Intent intent = new Intent(act.getApplicationContext(), TrainDetailsActivity.class);
        LocationActivityDTO dto = new LocationActivityDTO(points,kilometers, Double.valueOf(1.63), Double.valueOf(461), calendar);
        intent.putExtra("hora", dto.getHour());
        intent.putExtra("year",dto.getYear());
        intent.putExtra("month",dto.getMonth());
        intent.putExtra("day",dto.getDay());
        reference.document(calendar.get(Calendar.DAY_OF_MONTH)+"").collection("actividades").document(dto.getHour()).set(dto.setValuesToSave());
        trainRule.launchActivity(intent);
        details = trainRule.getActivity();
        LocationActivityDTO dtoApp = details.getDto();
        assertEquals(Double.valueOf(461), dtoApp.time);
        assertEquals(points.size(), dtoApp.coordinates.size());
        assertEquals(points.get(0), dtoApp.coordinates.get(0));
        assertEquals(points.get(1), dtoApp.coordinates.get(1));
        assertEquals(Double.valueOf(1.63), dtoApp.distance);

    }

    @Test
    public void testDataWithStops() throws InterruptedException {
        List<PointDTO> points = new ArrayList<>();
        points.add(new PointDTO(0, 0));
        points.add(new PointDTO(43.5378458, -5.6390675));
        points.add(new PointDTO(43.5379317, -5.6397083));
        points.add(new PointDTO(0, 0));
        points.add(new PointDTO(43.5378458, -5.6390675));
        points.add(new PointDTO(43.5379317, -5.6397083));
        points.add(new PointDTO(0, 0));
        points.add(new PointDTO(43.5379317, -5.6397083));
        Map<String, Double> kilometers = new HashMap<>();
        kilometers.put("1", Double.valueOf(286));
        kilometers.put("2", Double.valueOf(175));
        Intent intent = new Intent(act.getApplicationContext(), TrainDetailsActivity.class);
        LocationActivityDTO dto = new LocationActivityDTO(points,kilometers, Double.valueOf(1.63), Double.valueOf(461), calendar);
        intent.putExtra("hora", dto.getHour());
        intent.putExtra("year",dto.getYear());
        intent.putExtra("month",dto.getMonth());
        intent.putExtra("day",dto.getDay());
        reference.document(calendar.get(Calendar.DAY_OF_MONTH)+"").collection("actividades").document(dto.getHour()).set(dto.setValuesToSave());
        trainRule.launchActivity(intent);
        details = trainRule.getActivity();
        LocationActivityDTO dtoApp = details.getDto();
        assertEquals(Double.valueOf(461), dtoApp.time);
        assertEquals(8, dtoApp.coordinates.size());
        assertEquals(points.get(0), dtoApp.coordinates.get(0));
        assertEquals(points.get(1), dtoApp.coordinates.get(1));
        assertEquals(Double.valueOf(1.63), dtoApp.distance);

    }

    /**
     * Refinamiento del método splitIfIsNecessary
     * @throws InterruptedException
     */
    @Test
    public void testSplitIfIsNecessary() throws InterruptedException {
        List<PointDTO> points = new ArrayList<>();
        points.add(new PointDTO(0, 0));
        points.add(new PointDTO(43.5378458, -5.6390675));
        points.add(new PointDTO(43.5379317, -5.6397083));
        points.add(new PointDTO(0, 0));
        points.add(new PointDTO(43.5378458, -5.6390675));
        points.add(new PointDTO(43.5379317, -5.6397083));
        points.add(new PointDTO(0, 0));
        points.add(new PointDTO(43.5379317, -5.6397083));
        Map<String, Double> kilometers = new HashMap<>();
        kilometers.put("1", Double.valueOf(286));
        kilometers.put("2", Double.valueOf(175));
        Intent intent = new Intent(act.getApplicationContext(), TrainDetailsActivity.class);
        LocationActivityDTO dto = new LocationActivityDTO(points,kilometers, Double.valueOf(1.63), Double.valueOf(461), calendar);
        intent.putExtra("hora", dto.getHour());
        intent.putExtra("year",dto.getYear());
        intent.putExtra("month",dto.getMonth());
        intent.putExtra("day",dto.getDay());
        reference.document(calendar.get(Calendar.DAY_OF_MONTH)+"").collection("actividades").document(dto.getHour()).set(dto.setValuesToSave());
        trainRule.launchActivity(intent);
        details = trainRule.getActivity();
        LocationActivityDTO dtoAct = details.getDto();
        List<Point> list = dtoAct.getMapBoxPoints();
        assertEquals(3, details.getListaDeListas().size());
        assertEquals(2, details.getListaDeListas().get(0).size());
        assertEquals(2, details.getListaDeListas().get(1).size());
        assertEquals(1, details.getListaDeListas().get(2).size());



    }



}