package com.example.trainingPartner.unitary;

import static org.junit.Assert.assertEquals;

import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;

import androidx.test.ext.junit.runners.AndroidJUnit4;
import androidx.test.rule.ActivityTestRule;

import com.example.trainingPartner.controller.SplashActivity;
import com.example.trainingPartner.controller.TrainDetailsActivity;
import com.example.trainingPartner.controller.TrainDetailsManualActivity;
import com.example.trainingPartner.dto.PointDTO;
import com.example.trainingPartner.dto.UserInformationDTO;
import com.example.trainingPartner.dto.activityDto.LocationActivityDTO;
import com.example.trainingPartner.dto.activityDto.ManualActivityDTO;
import com.example.trainingPartner.utils.Utils;
import com.google.firebase.firestore.CollectionReference;
import com.google.firebase.firestore.FirebaseFirestore;
import com.mapbox.geojson.Point;

import org.junit.After;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Instrumented test, which will execute on an Android device.
 *
 * @see <a href="http://d.android.com/tools/testing">Testing documentation</a>
 */
@RunWith(AndroidJUnit4.class)
public class TrainDetailsManualActivityTest {

    @Rule
    public ActivityTestRule<SplashActivity> signRule = new ActivityTestRule<SplashActivity>(SplashActivity.class);

    @Rule
    public ActivityTestRule<TrainDetailsManualActivity> trainRule = new ActivityTestRule<TrainDetailsManualActivity>(TrainDetailsManualActivity.class, false, false);

    private FirebaseFirestore fb = FirebaseFirestore.getInstance();
    private SplashActivity act;

    private TrainDetailsManualActivity details;
    private UserInformationDTO user;
    private Context context;
    private Calendar calendar = Calendar.getInstance();
    private CollectionReference reference = fb.collection("usuarios").document("usuarioNoExistente").collection("año").
            document(calendar.get(Calendar.YEAR)+"").collection(calendar.get(Calendar.MONTH)+"");

    @Before
    public void setUp(){

        act = signRule.getActivity();
        SharedPreferences userPref = act.getApplicationContext().getSharedPreferences("users",Context.MODE_PRIVATE);
        SharedPreferences.Editor edit = userPref.edit();
        edit.putString("username", "usuarioNoExistente");
        edit.apply();
        user = new UserInformationDTO("Pedro","usuarioNoExistente","Pruebas", new Date(), 65.7, 175, true, "default", "ContraseñaPrueba@");
        fb.collection("usuarios").document("usuarioNoExistente").set(user.getInformation());



    }
    @After
    public void tearDown(){
        trainRule.finishActivity();
        Utils.deleteCollection( reference.document(calendar.get(Calendar.DAY_OF_MONTH)+"").collection("actividades"));
        Utils.deleteCollection(reference);
        fb.collection("usuarios").document("usuarioNoExistente").delete();
        Utils.deleteSharedPreferences(details);
        details= null;

    }

    @Test
    public void testManualData() throws InterruptedException {
        Intent intent = new Intent(act.getApplicationContext(), TrainDetailsManualActivity.class);
        ManualActivityDTO dto = new ManualActivityDTO(1, 4 , calendar);
        intent.putExtra("hora", dto.getHour());
        intent.putExtra("year",dto.getYear());
        intent.putExtra("month",dto.getMonth());
        intent.putExtra("day",dto.getDay());
        reference.document(calendar.get(Calendar.DAY_OF_MONTH)+"").collection("actividades").document(dto.getHour()).set(dto.setValuesToSave());
        trainRule.launchActivity(intent);
        details = trainRule.getActivity();
        ManualActivityDTO dtoApp = details.getDto();
        assertEquals(Double.valueOf(4), dtoApp.time);
        assertEquals(Double.valueOf(1), dtoApp.distance);

    }

    @Test
    public void testManualDataWithString() throws InterruptedException {
        Intent intent = new Intent(act.getApplicationContext(), TrainDetailsManualActivity.class);
        ManualActivityDTO dto = new ManualActivityDTO(1, 240 , calendar);
        intent.putExtra("hora", dto.getHour());
        intent.putExtra("year",dto.getYear());
        intent.putExtra("month",dto.getMonth());
        intent.putExtra("day",dto.getDay());
        reference.document(calendar.get(Calendar.DAY_OF_MONTH)+"").collection("actividades").document(dto.getHour()).set(dto.setValuesToSave());
        trainRule.launchActivity(intent);
        details = trainRule.getActivity();
        ManualActivityDTO dtoApp = details.getDto();
        assertEquals(Double.valueOf(240), dtoApp.time);
        assertEquals(Double.valueOf(1), dtoApp.distance);
        assertEquals(Double.valueOf(15), (Double) dtoApp.getSpeedMean());

    }





}