package com.example.trainingPartner.controller;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentTransaction;

import android.app.ProgressDialog;
import android.content.BroadcastReceiver;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.ServiceConnection;
import android.content.SharedPreferences;
import android.graphics.Bitmap;
import android.net.Uri;
import android.os.Bundle;
import android.os.IBinder;
import android.view.MenuItem;
import android.view.View;
import android.widget.ImageButton;
import android.widget.TextView;

import com.example.mapbox2.R;
import com.example.trainingPartner.controller.dialog.SelectModeDialog;
import com.example.trainingPartner.controller.fragment.HistoryFragment;
import com.example.trainingPartner.controller.fragment.MainWindowFragment;
import com.example.trainingPartner.model.MyCallbBack;
import com.example.trainingPartner.utils.Utils;
import com.example.trainingPartner.controller.fragment.PredictionsFragment;
import com.example.trainingPartner.controller.service.StepCounterService;
import com.example.trainingPartner.dto.UserInformationDTO;
import com.example.trainingPartner.model.DailyInformationModel;
import com.google.android.material.bottomnavigation.BottomNavigationView;
import com.google.android.material.navigation.NavigationBarView;
import com.theartofdev.edmodo.cropper.CropImage;

import java.io.ByteArrayOutputStream;
import java.io.FileNotFoundException;
import java.util.Calendar;

public class MainActivity extends AppCompatActivity {
    public static SharedPreferences sharedPreferences;
    private DailyInformationModel model;
    private UserInformationDTO dto;
    private BottomNavigationView bar;
    private Fragment fragment;
    private int menuItem;
    private int previousMenuItem;
    private StepCounterService mService= null;
    private boolean mBound = false;
    private Context context;
    private int steps;
    private ProgressDialog dialog;
    private Calendar today;
    private int calories =0;

    @Override
    protected void onCreate(Bundle savedInstanceState) {

        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        sharedPreferences = getSharedPreferences("stepCounter",MODE_PRIVATE);
        steps = sharedPreferences.getInt("stepCount", 0);
        dialog = new ProgressDialog(this, R.style.ProgressDialogStyle);
        dialog.setCancelable(false);
        dialog.show();
        String username = Utils.getUserName(this);
        System.out.println("USERNAME"+ username);
        model = new DailyInformationModel(username);
        bar = findViewById(R.id.navigationView);
        today = Calendar.getInstance();


        fragment = new MainWindowFragment(dialog);
        dto = MainWindowFragment.dto;
        menuItem = R.id.main_window;
        previousMenuItem = R.id.main_window;
        openFragment(fragment);
        bar.setSelectedItemId(menuItem);
        context = this;

        bar.setOnItemSelectedListener(new NavigationBarView.OnItemSelectedListener() {
            @Override
            public boolean onNavigationItemSelected(@NonNull MenuItem item) {
                previousMenuItem = menuItem;
                switch (item.getItemId()) {

                    case R.id.main_window:
                        menuItem = R.id.main_window;
                        dialog.show();
                        fragment = new MainWindowFragment(dialog);
                        openFragment(fragment);
                        break;
                    case R.id.record:
                        new SelectModeDialog(context, menuItem);
                        break;
                    case R.id.history:
                        menuItem = R.id.history;
                        dialog.show();
                        fragment = new HistoryFragment(dialog);
                        openFragment(fragment);
                        break;
                    case R.id.predictions:
                        menuItem=R.id.predictions;
                        dialog.show();
                        fragment = new PredictionsFragment(dialog);
                        openFragment(fragment);
                        break;
                }
                return true;
            }

        });


    }

    public void saveRegistry(){
        model.saveRegistry(today.get(Calendar.YEAR), today.get(Calendar.MONTH), today.get(Calendar.DAY_OF_MONTH));
    }
    @Override
    public void onResume() {
        super.onResume();

        bar.setSelectedItemId(menuItem);
        dismissProgressLogIn();

        dismissProgressSignIn();

    }

    private void dismissProgressLogIn(){
        ProgressDialog dialog = LogInActivity.dialog;
        if (dialog!=null && dialog.isShowing()){
            dialog.dismiss();
        }
    }

    private void dismissProgressSignIn(){
        ProgressDialog dialog = SignInActivity.dialog;
        if (dialog!=null && dialog.isShowing()){
            dialog.dismiss();
        }
    }

    public int saveStepsLogOut(){
        int actualY = today.get(Calendar.YEAR);
        int actualM = today.get(Calendar.MONTH);
        int actualD = today.get(Calendar.DAY_OF_MONTH);
        steps = sharedPreferences.getInt("stepCount", 0);
        model.saveStepsAndCalories(actualY, actualM, actualD, steps, calories );
        return steps;
    }

    public void saveSteps(){

        model.getLastConnection(new MyCallbBack<Calendar>() {
            @Override
            public void onCallback(Calendar lastConnection) {
                int year = lastConnection.get(Calendar.YEAR);
                int day = lastConnection.get(Calendar.DAY_OF_MONTH);
                int month = lastConnection.get(Calendar.MONTH);
                int actualY = today.get(Calendar.YEAR);
                int actualM = today.get(Calendar.MONTH);
                int actualD = today.get(Calendar.DAY_OF_MONTH);

                System.out.println(year+ " "+ month+ " "+ " "+ day+ " "+actualD +"  "+steps );
                if (actualY != year || actualD!= day || actualM!= month){
                    steps = sharedPreferences.getInt("stepCount", 0);
                    UserInformationDTO dto = MainWindowFragment.dto;
                    calories = (int) (dto.getWeight() * 0.0005 * dto.getLongitud_paso() * steps);
                    model.saveStepsAndCalories(year, month, day, steps, calories);
                    steps=0;
                    SharedPreferences.Editor editor =sharedPreferences.edit();
                    editor.putInt("year", actualY);
                    editor.putInt("day", actualD);
                    editor.putInt("month", actualM);
                    editor.putInt("stepCount", steps);
                    editor.apply();

                }
                saveRegistry();

                registerReceiver(updateSteps, new IntentFilter(StepCounterService.STEPS));
                bind();
            }

        });



    }

    private void bind(){
        bindService(new Intent(this, StepCounterService.class), mServiceConnection,
                Context.BIND_AUTO_CREATE);
    }


    public void openFragment(Fragment fragment) {
        FragmentTransaction transaction = getSupportFragmentManager().beginTransaction();
        //this is a helper class that replaces the container with the fragment. You can replace or add fragments.
        transaction.replace(R.id.container, fragment);
        transaction.addToBackStack(null); //if you add fragments it will be added to the backStack. If you replace the fragment it will add only the last fragment
        transaction.commit(); // commit() performs the action

    }


    @Override
    protected void onStart(){
        super.onStart();
        saveSteps();

    }


    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data){
        super.onActivityResult(requestCode,resultCode,data);
        if(requestCode==CropImage.CROP_IMAGE_ACTIVITY_REQUEST_CODE){
            CropImage.ActivityResult result = CropImage.getActivityResult(data);
            if (resultCode==RESULT_OK){
                System.out.println("Todo ha funcionado correctamente");
                Uri resultUri = result.getUri();
                System.out.println(resultUri);
                try {
                    Bitmap b2 = Utils.decodeUri(this, resultUri, 400);
                    Bitmap b3 =Utils.getRoundedCroppedBitmap(b2);
                    ((MainWindowFragment)fragment).getModel().saveImage(b3, new MyCallbBack<UserInformationDTO>() {
                        @Override
                        public void onCallback(UserInformationDTO element) {
                            ((MainWindowFragment)fragment).setImage();
                        }
                    });


                    //openFragment(new HistoryFragment());
                    //openFragment(fragment);
                } catch (FileNotFoundException e) {
                    e.printStackTrace();
                }

            } else if (resultCode == CropImage.CROP_IMAGE_ACTIVITY_RESULT_ERROR_CODE) {
                Exception error = result.getError();
            }



        }
    }



    private BroadcastReceiver updateSteps = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            int steps = intent.getIntExtra(StepCounterService.STEPS_EXTRA,0);

            View customLayout = findViewById(R.id.steps_main);
            if (customLayout!= null){
                UserInformationDTO dto = MainWindowFragment.dto;
                if (dto!= null){
                    calories = (int) (dto.getWeight() * 0.0005 * dto.getLongitud_paso() * steps);
                    View customLayout2 = findViewById(R.id.calorie_main);
                    TextView st2 = customLayout2.findViewById(R.id.textViewCalories);
                    st2.setText(Utils.setStepsString(calories));

                }
                TextView st = customLayout.findViewById(R.id.textViewSteps);
                st.setText(Utils.setStepsString(steps));
            }

        }
    };



    // Monitors the state of the connection to the service.
    private final ServiceConnection mServiceConnection = new ServiceConnection() {

        @Override
        public void onServiceConnected(ComponentName name, IBinder service) {
            steps = sharedPreferences.getInt("stepCount", 0);
            StepCounterService.LocalBinder binder = (StepCounterService.LocalBinder) service;
            mService = binder.getService();
            mService.requestSteps(steps);
            TextView tx = findViewById(R.id.textViewSteps);
            tx.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    mService.stopRequest();
                }
            });
            mBound = true;

        }



        @Override
        public void onServiceDisconnected(ComponentName name) {
            mService = null;
            mBound = false;
        }




    };
    @Override
    public void onStop() {
        super.onStop();
        if (mBound) {
            // Unbind from the service. This signals to the service that this activity is no longer
            // in the foreground, and the service can respond by promoting itself to a foreground
            // service.

            unbindService(mServiceConnection);
            mBound = false;
        }

    }
    public int getStepsSharedPref( ){
        SharedPreferences shared = this.getPreferences(Context.MODE_PRIVATE);
        int s=  shared.getInt("stepCount", 0);
        return s;


    }

    public void setPreviousItem(int previousMenuItem){
        this.previousMenuItem = previousMenuItem;
    }


    @Override
    public void onBackPressed(){
        bar.setSelectedItemId(previousMenuItem);

    }

    public void stopSteps(){
        mService.stopRequest();
    }
}