package com.example.trainingPartner.controller;

import androidx.appcompat.app.AppCompatActivity;

import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.widget.Button;
import android.widget.NumberPicker;

import com.example.mapbox2.R;
import com.example.trainingPartner.model.MyCallbBack;
import com.example.trainingPartner.utils.Utils;
import com.example.trainingPartner.dto.activityDto.ActivityDTO;
import com.example.trainingPartner.dto.activityDto.ManualActivityDTO;
import com.example.trainingPartner.model.RecordModel;

import java.util.Calendar;

public class ManualRecordActivity extends AppCompatActivity {
    private NumberPicker pickerKm;
    private NumberPicker pickerM;
    private NumberPicker pickerHours;
    private NumberPicker pickerMinutes;
    private NumberPicker pickerSeconds;
    private Button buttonSave;
    private Context context;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_manual_record);
        this.context=this;
        setButton();
        selectKm();
        selectM();
        selectHours();
        selectMinutesSeconds();
    }

    private void setButton(){
        buttonSave= findViewById(R.id.buttonSaveManualActivity);
        buttonSave.setEnabled(false);
        buttonSave.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                int hours = pickerHours.getValue();
                int minutes = pickerMinutes.getValue();
                int seconds = pickerSeconds.getValue();
                int km = pickerKm.getValue();
                int m = pickerM.getValue();
                double time = (hours*3600 + minutes*60 + seconds);
                double met = (double)m/1000;
                double distance = (km+met);
                ManualActivityDTO dto = new ManualActivityDTO(distance, time, Calendar.getInstance());

                saveActivity(dto);

            }
        });
    }

    public void saveActivity(ActivityDTO dto){
        String username = Utils.getUserName(context);
        RecordModel model = new RecordModel(username);
        model.saveActivity(dto, new MyCallbBack<Boolean>() {
            @Override
            public void onCallback(Boolean element) {
                startManualTrainDetailsActivity(dto);
                finish();
            }
        });
    }

    private void startMainActivity(){
        Intent intent = new Intent(this, MainActivity.class);
        startActivity(intent);
    }

    @Override
    public void onBackPressed(){
        startMainActivity();
    }

    /**
     * Inicia la actividad de el TrainDetails
     * @param dto
     */
    private void startManualTrainDetailsActivity(ActivityDTO dto){
        Intent intent = new Intent(this, TrainDetailsManualActivity.class);
        intent.putExtra("hora", dto.getHour());
        intent.putExtra("year",dto.getYear());
        intent.putExtra("month",dto.getMonth());
        intent.putExtra("day",dto.getDay());
        startActivity(intent);
    }

    private void selectKm(){
        pickerKm = findViewById(R.id.numberPickerKm);
        pickerKm.setMinValue(0);
        pickerKm.setMaxValue(200);
        pickerKm.setOnValueChangedListener(new Listener());
    }
    private void selectM(){
        pickerM = findViewById(R.id.numberPickerm);
        pickerM.setMinValue(0);
        pickerM.setMaxValue(999);
        pickerM.setOnValueChangedListener(new Listener());

    }

    private void selectHours(){
        pickerHours = findViewById(R.id.numberPickerHours);
        pickerHours.setMinValue(0);
        pickerHours.setMaxValue(200);
        pickerHours.setOnValueChangedListener(new Listener());
    }

    private void selectMinutesSeconds(){
        pickerMinutes= findViewById(R.id.numberPickerMinutes);
        pickerMinutes.setMinValue(0);
        pickerMinutes.setMaxValue(59);
        pickerMinutes.setOnValueChangedListener(new Listener());

        pickerSeconds= findViewById(R.id.numberPickerSeconds);
        pickerSeconds.setMinValue(0);
        pickerSeconds.setMaxValue(59);
        pickerSeconds.setOnValueChangedListener(new Listener());
    }

    private class Listener implements NumberPicker.OnValueChangeListener {

        @Override
        public void onValueChange(NumberPicker picker, int oldVal, int newVal) {
            if (pickerSeconds.getValue()==0 && pickerMinutes.getValue()==0 && pickerHours.getValue()==0 ){
                buttonSave.setEnabled(false);
            }else if (pickerM.getValue()==0 && pickerKm.getValue()==0){
                buttonSave.setEnabled(false);
            }else{
                buttonSave.setEnabled(true);
            }
        }
    }



}