package com.example.trainingPartner.controller;

import android.Manifest;
import android.app.ProgressDialog;
import android.content.BroadcastReceiver;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.ServiceConnection;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.location.Location;
import android.os.Bundle;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;

import androidx.core.app.ActivityCompat;
import androidx.localbroadcastmanager.content.LocalBroadcastManager;

import com.example.mapbox2.R;
import com.example.trainingPartner.model.MyCallbBack;
import com.example.trainingPartner.utils.Utils;
import com.example.trainingPartner.controller.service.TrackingService;
import com.example.trainingPartner.dto.activityDto.LocationActivityDTO;
import com.example.trainingPartner.dto.PointDTO;
import com.example.trainingPartner.model.RecordModel;
import com.mapbox.geojson.Point;

import android.os.IBinder;
import android.preference.PreferenceManager;
import android.util.Log;
import android.view.View;
import android.widget.ImageButton;
import android.widget.TextView;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;

public class RecordActivity extends AppCompatActivity implements
        SharedPreferences.OnSharedPreferenceChangeListener{
    public static ProgressDialog progress;

    private static final String TAG = RecordActivity.class.getSimpleName();
    // Used in checking for runtime permissions.
    private static final int REQUEST_PERMISSIONS_REQUEST_CODE = 34;

    // The BroadcastReceiver used to listen from broadcasts from the service.
    private MyReceiver myReceiver;

    // A reference to the service used to get location updates.
    private TrackingService mService = null;

    // Tracks the bound state of the service.
    private boolean mBound = false;

    // UI elements.
    private  TextView timeTextView;
    private TextView distanceTextView;
    private TextView speedTextView;
    private TextView minKm;
    private ImageButton start ;
    private ImageButton stop;
    private ImageButton end;


    private double time = 0.0;
    private double speed = 0.0;
    private double distance = 0.0;
    private String username ;
    private RecordModel model;
    private List<Point> routeCoordinates = new ArrayList<>();
    private boolean ended = false;

    // Monitors the state of the connection to the service.
    private final ServiceConnection mServiceConnection = new ServiceConnection() {


        @Override
        public void onServiceConnected(ComponentName name, IBinder service) {
            TrackingService.LocalBinder binder = (TrackingService.LocalBinder) service;
            mService = binder.getService();
            mBound = true;
        }

        @Override
        public void onServiceDisconnected(ComponentName name) {
            mService = null;
            mBound = false;
        }
    };

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        myReceiver = new MyReceiver();
        setContentView(R.layout.activity_record);
        username = Utils.getUserName(this);
        model = new RecordModel(username);
        progress = new ProgressDialog(this);
        progress.setMessage("Determinando ubicacion...");
        progress.setCancelable(false);
        if (!checkPermissions()) {
            progress.show();
        }



        // Check that the user hasn't revoked permissions by going to Settings.
        if (Utils.requestingLocationUpdates(this)) {
            if (!checkPermissions()) {
                requestPermissions();
            }
        }
    }
    @Override
    protected void onStart() {
        super.onStart();

        PreferenceManager.getDefaultSharedPreferences(this)
                .registerOnSharedPreferenceChangeListener(this);

        start = (ImageButton) findViewById(R.id.imageButtonStart);
        stop = (ImageButton) findViewById(R.id.imageButtonStop);
        end = (ImageButton) findViewById(R.id.imageButtonEnd);
        timeTextView = findViewById(R.id.textViewTimeValue);
        speedTextView = findViewById(R.id.textViewVelocityValue);
        distanceTextView = findViewById(R.id.textViewDistanceValue);
        minKm = findViewById(R.id.textViewMinKmValue);
        registerReceivers();

        start.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if (!checkPermissions()) {
                    requestPermissions();
                } else {
                    mService.requestLocationUpdates(time);
                }
            }
        });

        stop.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                mService.removeLocationUpdates(false);
                stop.setVisibility(View.GONE);
            }
        });
        end.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                mService.onDestroy();
                stopRequesting();
                ended=true;
            }
        });

        // Restore the state of the buttons when the activity (re)launches.
        //setButtonsState(Util.requestingLocationUpdates(this));

        // Bind to the service. If the service is in foreground mode, this signals to the service
        // that since this activity is in the foreground, the service can exit foreground mode.
        bindService(new Intent(this, TrackingService.class), mServiceConnection,
                Context.BIND_AUTO_CREATE);
    }

    private void stopRequesting(){
        Utils.setRequestingLocationUpdates(this, false);
    }

    @Override
    protected void onResume() {
        super.onResume();

        LocalBroadcastManager.getInstance(this).registerReceiver(myReceiver,
                new IntentFilter(TrackingService.ACTION_BROADCAST));
    }



    @Override
    protected void onPause() {
        LocalBroadcastManager.getInstance(this).unregisterReceiver(myReceiver);
        super.onPause();
    }

    @Override
    protected void onStop() {
        if (mBound) {
            // Unbind from the service. This signals to the service that this activity is no longer
            // in the foreground, and the service can respond by promoting itself to a foreground
            // service.
            unbindService(mServiceConnection);
            mBound = false;
        }
        PreferenceManager.getDefaultSharedPreferences(this)
                .unregisterOnSharedPreferenceChangeListener(this);
        super.onStop();
    }
    /**
     * Registra los recividores de los intents
     */
    private void registerReceivers(){
        registerReceiver(requestLocationPermission, new IntentFilter(TrackingService.REQUESTPERMISSION));
        registerReceiver(updateSpeed, new IntentFilter(TrackingService.TRACKING_SPEED));
        registerReceiver(getRouteCoordinates, new IntentFilter(TrackingService.TRACKING_COORDINATES));
        registerReceiver(updateDistance, new IntentFilter(TrackingService.TRACKING_DISTANCE));
        registerReceiver(updateTime, new IntentFilter(TrackingService.TIMER_UPDATED));
    }

    public void setTime(double time){
        this.time = time;
    }

    private void unregisterReceivers(){
        unregisterReceiver(updateDistance);
        unregisterReceiver(updateSpeed);
        unregisterReceiver(updateTime);
        unregisterReceiver(getRouteCoordinates);
    }
    /**
     * Este recibidor, asignara el tiempo y lo mostrará por pantalla
     */
    private BroadcastReceiver updateTime = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            time = intent.getDoubleExtra(TrackingService.TIME_EXTRA,0.0);
            timeTextView.setText(Utils.returnStringTime(time));
        }
    };

    /**
     * Este recibidor recibirá la distancia y la mostrará por pantalla
     */
    private BroadcastReceiver updateDistance = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            distance = intent.getDoubleExtra(TrackingService.DISTANCE_EXTRA,0.0);
            distanceTextView.setText(convertToKilometers(distance));
        }
    };
    /**
     * Este recibidor recibirá la velocidad y la mostrará por pantalla
     */
    private BroadcastReceiver updateSpeed = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            speed = intent.getDoubleExtra(TrackingService.SPEED_EXTRA,0.0);
            speedTextView.setText(convertToSpeed(speed));
            minKm.setText(Utils.convertToMinKm(speed));
        }
    };

    /**
     * Este recibidor recibirá el permiso de la localización
     */
    private BroadcastReceiver requestLocationPermission = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            if (!checkPermissions()) {
                requestPermissions();
            }
        }
    };

    /**
     * Este recibidor recibe las coordenadas y cuando termina la actividad las guarda
     */
    private BroadcastReceiver getRouteCoordinates = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            List<PointDTO> lista = (List<PointDTO>) intent.getSerializableExtra(TrackingService.COORDINATES_EXTRA);
            HashMap<String, Double> parcials= (HashMap<String, Double>) intent.getSerializableExtra(TrackingService.PARCIALS_EXTRA);
            parcials = addTheRestantKilometer(distance,parcials);
            System.out.println(ended);
            lista = lista.subList(0, lista.size()-1);
            if (ended){
                LocationActivityDTO actividadDTO;
                if (parcials.size()>0){
                    actividadDTO = new LocationActivityDTO(lista, parcials, distance, time, Calendar.getInstance());
                }else{
                    actividadDTO = new LocationActivityDTO(lista, time, Calendar.getInstance());
                }

                model.saveActivity(actividadDTO, new MyCallbBack<Boolean>() {
                    @Override
                    public void onCallback(Boolean element) {
                        startTrainDetailsActivity(actividadDTO);
                        unregisterReceivers();
                        finish();
                    }
                });

            }
        }
    };
    /**
     * Inicia la actividad de el TrainDetails
     * @param dto
     */
    private void startTrainDetailsActivity(LocationActivityDTO dto){
        Intent intent = new Intent(this, TrainDetailsActivity.class);
        intent.putExtra("hora", dto.getHour());
        intent.putExtra("year",dto.getYear());
        intent.putExtra("month",dto.getMonth());
        intent.putExtra("day",dto.getDay());
        startActivity(intent);
    }

    /**
     * Añade el kilometro restante a los parciales
     * Por ejemplo:
     * Haces 3.20 km, añade ese 0.2 con el tiempo que le tomo.
     * @param distance
     * @param p
     * @return
     */
    public HashMap<String, Double> addTheRestantKilometer(double distance,HashMap<String,Double> p){
        HashMap<String, Double> parcials = p;
        int km = (int) distance +1;
        double decimal = distance-((int)distance);
        if (decimal>=0.01){
            String key = km+"";
            if (!parcials.containsKey(key)){
                double time = 0;
                for (double value : parcials.values()){
                    time+=value;
                }
                time = this.time -time;
                parcials.put(key, time);
            }
        }
        return parcials;
    }

    /**
     * Convierte a km para mostrar por pantalla
     * @param distance
     * @return
     */
    private String convertToKilometers(double distance){
        return Utils.roundTwoDecimals(distance) + " km";
    }



    /**
     * Convierte a km/h para mostrar por pantalla
     * @param speed
     * @return
     */
    private String convertToSpeed(double speed){
        return speed + " km/h";
    }


    /**
     * Devuelve el estado de las peticiones necesitadas.
     */
    private boolean checkPermissions() {
        return  PackageManager.PERMISSION_GRANTED == ActivityCompat.checkSelfPermission(this,
                Manifest.permission.ACCESS_FINE_LOCATION);
    }

    private void requestPermissions() {
            Log.i(TAG, "Requesting permission");
            ActivityCompat.requestPermissions(RecordActivity.this,
                    new String[]{Manifest.permission.ACCESS_FINE_LOCATION},
                    REQUEST_PERMISSIONS_REQUEST_CODE);
            progress.dismiss();
    }
    /**
     * Callback recibida cuando la petición de localización es completada.
     */
    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions,
                                           @NonNull int[] grantResults) {
        Log.i(TAG, "onRequestPermissionResult");
        if (requestCode == REQUEST_PERMISSIONS_REQUEST_CODE) {
            if (grantResults.length <= 0) {
                // Si la interacción con el usuario es cancelada.
                Log.i(TAG, "User interaction was cancelled.");
            } else if (grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                //Aquí se inicia el servicio en segundo plano.
                mService.requestLocationUpdates(time);
            } else {
                //Si no se dan permisos se cierra la activity
               finish();
               startMainActivity();
            }
        }
    }


    @Override
    public void onSharedPreferenceChanged(SharedPreferences sharedPreferences, String s) {
        // Update the buttons state depending on whether location updates are being requested.
        if (s.equals(Utils.KEY_REQUESTING_LOCATION_UPDATES)) {
            setButtonsState(sharedPreferences.getBoolean(Utils.KEY_REQUESTING_LOCATION_UPDATES,
                    false));
        }
    }
    private void startMainActivity(){
        mService.removeLocationUpdates(true);
        Utils.setRequestingLocationUpdates(this, false);
        Intent intent = new Intent(this, MainActivity.class);
        startActivity(intent);

    }

    @Override
    public void onBackPressed(){
        startMainActivity();
    }


    private class MyReceiver extends BroadcastReceiver {
        @Override
        public void onReceive(Context context, Intent intent) {
            Location location = intent.getParcelableExtra(TrackingService.EXTRA_LOCATION);
            if (location != null) {

                System.out.println("location== "+ Utils.getLocationText(location));
            }
        }
    }
    private void setButtonsState(boolean requestingLocationUpdates) {
        if (requestingLocationUpdates) {
            start.setEnabled(false);
            end.setEnabled(true);
            end.setVisibility(View.VISIBLE);
            stop.setVisibility(View.VISIBLE);
            start.setVisibility(View.GONE);
        } else {
            start.setEnabled(true);
            end.setEnabled(false);
            end.setVisibility(View.GONE);
            start.setVisibility(View.VISIBLE);
            stop.setVisibility(View.GONE);
        }
    }


}