package com.example.trainingPartner.controller;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;

import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.graphics.Color;
import android.graphics.Typeface;
import android.os.Bundle;
import android.text.Spannable;
import android.text.SpannableString;
import android.text.style.StyleSpan;
import android.view.View;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.TableRow;
import android.widget.TextView;

import com.example.mapbox2.R;
import com.example.trainingPartner.dto.activityDto.ActivityDTO;
import com.example.trainingPartner.model.MyCallbBack;
import com.example.trainingPartner.utils.Utils;
import com.example.trainingPartner.controller.dialog.DeleteDialog;
import com.example.trainingPartner.dto.activityDto.LocationActivityDTO;
import com.example.trainingPartner.model.TrainDetailsModel;
import com.mapbox.geojson.Feature;
import com.mapbox.geojson.FeatureCollection;
import com.mapbox.geojson.LineString;
import com.mapbox.geojson.MultiPoint;
import com.mapbox.geojson.Point;
import com.mapbox.mapboxsdk.Mapbox;
import com.mapbox.mapboxsdk.camera.CameraPosition;
import com.mapbox.mapboxsdk.geometry.LatLng;
import com.mapbox.mapboxsdk.maps.MapView;
import com.mapbox.mapboxsdk.maps.MapboxMap;
import com.mapbox.mapboxsdk.maps.OnMapReadyCallback;
import com.mapbox.mapboxsdk.maps.Style;
import com.mapbox.mapboxsdk.style.layers.LineLayer;
import com.mapbox.mapboxsdk.style.layers.Property;
import com.mapbox.mapboxsdk.style.layers.PropertyFactory;
import com.mapbox.mapboxsdk.style.sources.GeoJsonSource;
import com.prolificinteractive.materialcalendarview.CalendarDay;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class TrainDetailsActivity extends AppCompatActivity implements OnMapReadyCallback {
    private ProgressDialog dialog;
    private MapboxMap mapboxMap;
    private MapView mapView;
    private TrainDetailsModel model;
    private LocationActivityDTO dto;

    private List<List<Point>> listaDeListas = new ArrayList<>();

    private String year, month,day, hour;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        //Se crea una instancia de mapbox con el token de inicio de sesion
        Mapbox.getInstance(this, getString(R.string.acces_token));
        setContentView(R.layout.activity_train_details);
        mapView = findViewById(R.id.mapView);
        mapView.onCreate(savedInstanceState);
        mapView.getMapAsync(this);
        Intent intent = getIntent();

        year = intent.getStringExtra("year");
        month = intent.getStringExtra("month");
        day = intent.getStringExtra("day");
        hour = intent.getStringExtra("hora");
        String username = Utils.getUserName(this);
        model = new TrainDetailsModel(year,month,day,hour, Utils.getCalendar(day,month,year), username);
        model.getData(new MyCallbBack<ActivityDTO>() {
            @Override
            public void onCallback(ActivityDTO element) {
                    dto = (LocationActivityDTO) element;
                    splitIfIsNecessary(dto.getMapBoxPoints());
                    asignarCampos();
                    buttonActions();
                    loadStyle();
                }
        });
        dialog = new ProgressDialog(this, R.style.ProgressDialogStyle);
        dialog.setCancelable(false);

    }

    public LocationActivityDTO getDto(){
        return dto;
    }

    private void buttonActions(){
        Button buttonDelete = findViewById(R.id.buttonBorrar);
        Button buttonSave = findViewById(R.id.buttonGuardar);

        actionDelete(buttonDelete);

        actionSave(buttonSave);
    }

    private void actionSave(Button buttonSave) {
        buttonSave.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                startMainActivity();
                finish();
            }
        });
    }

    private void actionDelete(Button buttonDelete) {
        Context cont = this;
        buttonDelete.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                new DeleteDialog(cont, model, dialog);
            }
        });
    }



    private void startMainActivity(){
        dialog.show();
        Intent intent = new Intent(this, MainActivity.class);
        startActivity(intent);
    }

    @Override
    public void onBackPressed(){
        startMainActivity();
    }

    private void asignarCampos(){
        TextView distance = findViewById(R.id.textViewDistanceValueDetails);
        distance.setText(dto.getDistance() +" km");
        TextView speed = findViewById(R.id.textViewVelocityValueDetails);
        speed.setText(dto.getSpeedMean() + " km/h");
        TextView minKm = findViewById(R.id.textViewMinKmValueDetails);
        minKm.setText(dto.getMinKmMean());
        TextView time = findViewById(R.id.textViewTimeValueDetails);
        time.setText(dto.getFormatHour());

        TextView titulo = findViewById(R.id.textViewTitulo);

        String fecha = day + "/"+(Integer.parseInt(month)+1)+"/"+year;
        titulo.setText("ACTIVIDAD: "+ fecha+  " "+hour);
        setPartials();
    }

    private void setPartials(){
        Map<String, Double> partials = dto.getPartials();
        LinearLayout layoutParciales = findViewById(R.id.layoutParciales);
        if (partials.size()>0){
            setFirstColumn(layoutParciales);
            for (int i = 0; i < partials.size(); i++){
                LinearLayout horizontal = new LinearLayout(this);
                horizontal.setLayoutParams(new LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT, 1f));
                horizontal.setOrientation(LinearLayout.HORIZONTAL);
                int a = i+1;
                double t = partials.get(a+"");
                horizontal.addView(createTextView(a+""));
                horizontal.addView(createTextView(Utils.returnStringTime(t)));
                horizontal.addView(createTextView(dto.getDistanceKilometer(i)));
                horizontal.addView(createTextView(dto.getSpeedKilometer(i,t)));
                horizontal.setPadding(0,10,0,10);


                layoutParciales.addView(horizontal);
            }
        }


    }


    private void setFirstColumn(LinearLayout layoutParciales){
        LinearLayout horizontal = new LinearLayout(this);
        horizontal.setLayoutParams(new LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT, 1f));
        horizontal.setOrientation(LinearLayout.HORIZONTAL);


        horizontal.addView(createTitle("Vuelta"));
        horizontal.addView(createTitle("Tiempo"));
        horizontal.addView(createTitle("Distancia"));
        horizontal.addView(createTitle("Ritmo"));

        layoutParciales.addView(horizontal);
    }

    private TextView createTextView(String text){
        LinearLayout.LayoutParams params = new TableRow.LayoutParams(0, LinearLayout.LayoutParams.WRAP_CONTENT, 1f);
        TextView textV = new TextView(this);
        textV.setText(text);
        textV.setTextAlignment(View.TEXT_ALIGNMENT_CENTER);
        textV.setLayoutParams(params);
        textV.setTextSize(15);
        return textV;
    }

    private TextView createTitle(String text){
        SpannableString spannableString = new SpannableString(text);
        // Creamos un objeto de estilo negrita
        StyleSpan boldStyle = new StyleSpan(Typeface.BOLD);
        spannableString.setSpan(boldStyle, 0, text.length(), Spannable.SPAN_EXCLUSIVE_EXCLUSIVE);
        LinearLayout.LayoutParams params = new TableRow.LayoutParams(0, LinearLayout.LayoutParams.WRAP_CONTENT, 1f);
        TextView textV = new TextView(this);
        textV.setTextAlignment(View.TEXT_ALIGNMENT_CENTER);
        textV.setLayoutParams(params);
        textV.setTextSize(20);
        textV.setText(spannableString);
        return textV;
    }


    @Override
    public void onMapReady(@NonNull MapboxMap mapboxMap) {

        this.mapboxMap = mapboxMap; //Se le asigna al atributo



    }

    private void loadStyle(){
        Point point = dto.getMiddlePoint();
        int puntos = dto.getMapBoxPoints().size();
        int zoomAToApply = 15- (puntos/20);
        CameraPosition cameraPosition = new CameraPosition.Builder()
                .target(new LatLng(point.latitude(), point.longitude()))
                .zoom(zoomAToApply)
                .bearing(0)
                .tilt(0)
                .build();

        mapboxMap.setCameraPosition(cameraPosition);
        mapboxMap.setStyle(Style.MAPBOX_STREETS, new Style.OnStyleLoaded() {
            @Override
            public void onStyleLoaded(@NonNull Style style) {
                int id = 0;
                for (List<Point> element : listaDeListas){
                    Set mySet = new HashSet(element);
                    if (mySet.size()>1){
                        String sourceId = "line-source-" + id;
                        String layerId = "line-layer-" + id;
                        System.out.println(element);
                        id++;
                        style.addSource(new GeoJsonSource(sourceId,
                                FeatureCollection.fromFeatures(new Feature[] {Feature.fromGeometry(
                                        LineString.fromLngLats(MultiPoint.fromLngLats(element))
                                )})));
                        // The layer properties for our line. This is where we make the line dotted, set the
                        // color, etc.
                        style.addLayer(new LineLayer(layerId, sourceId).withProperties(
                                PropertyFactory.lineDasharray(new Float[] {0.01f, 2f}),
                                PropertyFactory.lineCap(Property.LINE_CAP_ROUND),
                                PropertyFactory.lineJoin(Property.LINE_JOIN_ROUND),
                                PropertyFactory.lineWidth(5f),
                                PropertyFactory.lineColor(Color.parseColor("#e55e5e"))
                        ));
                    }
                }




            }
        });
    }

    public void splitIfIsNecessary(List<Point> lista){
        int index = lista.indexOf(Point.fromLngLat(0,0));
        if (index != -1){
            List<Point> lista1 = lista.subList(0, index);
            splitIfIsNecessary(lista1);

            List<Point> lista2 = lista.subList(index+1, lista.size());
            splitIfIsNecessary(lista2);
        }else if (lista.size()>0){
            listaDeListas.add(lista);
        }
    }

    public List<List<Point>> getListaDeListas(){
        int cont =0;
        for (List<Point> list: listaDeListas){
            System.out.println("Lista: "+ cont++);
            for (Point p : list){
                System.out.println("    "+p.latitude()+" "+ p.longitude());
            }
        }
        return listaDeListas;
    }

    @Override
    protected void onDestroy(){

        if (dialog.isShowing()){
            dialog.dismiss();
        }
        super.onDestroy();
    }
}