package com.example.trainingPartner.controller;

import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;

import androidx.appcompat.app.AppCompatActivity;

import com.example.mapbox2.R;
import com.example.trainingPartner.dto.activityDto.ActivityDTO;
import com.example.trainingPartner.model.MyCallbBack;
import com.example.trainingPartner.utils.Utils;
import com.example.trainingPartner.controller.dialog.DeleteDialog;
import com.example.trainingPartner.dto.activityDto.ManualActivityDTO;
import com.example.trainingPartner.model.TrainDetailsModel;
import com.mapbox.mapboxsdk.Mapbox;

public class TrainDetailsManualActivity extends AppCompatActivity  {

    private ProgressDialog dialog;
    private TrainDetailsModel model;
    private ManualActivityDTO dto;

    private String day, hour, year, month;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        //Se crea una instancia de mapbox con el token de inicio de sesion
        Mapbox.getInstance(this, getString(R.string.acces_token));
        setContentView(R.layout.activity_train_manual_details);
        Intent intent = getIntent();
        year = intent.getStringExtra("year");
        month = intent.getStringExtra("month");
        day = intent.getStringExtra("day");
        hour = intent.getStringExtra("hora");
        String username = Utils.getUserName(this);

        dialog = new ProgressDialog(this, R.style.ProgressDialogStyle);
        dialog.setCancelable(false);
        model = new TrainDetailsModel(year,month,day,hour, Utils.getCalendar(day,month,year), username);

        model.getData(new MyCallbBack<ActivityDTO>() {
            @Override
            public void onCallback(ActivityDTO element) {
                dto= (ManualActivityDTO) element;
                asignarCampos();
                buttonActions();
            }
        });

    }

    public ManualActivityDTO getDto(){
        return dto;
    }

    private void buttonActions(){
        Button buttonDelete = findViewById(R.id.buttonBorrar);
        Button buttonSave = findViewById(R.id.buttonGuardar);

        actionDelete(buttonDelete);

        actionSave(buttonSave);
    }

    private void actionSave(Button buttonSave) {
        buttonSave.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

                startMainActivity();
                finish();
            }
        });
    }

    private void actionDelete(Button buttonDelete) {
        Context cont = this;
        buttonDelete.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                new DeleteDialog(cont, model, dialog);

            }
        });
    }



    private void startMainActivity(){
        dialog.show();
        Intent intent = new Intent(this,MainActivity.class);
        startActivity(intent);
    }

    @Override
    public void onBackPressed(){

        startMainActivity();
    }

    private void asignarCampos(){
        TextView distance = findViewById(R.id.textViewDistanceValueDetails);
        distance.setText(dto.getDistance() +" km");
        TextView speed = findViewById(R.id.textViewVelocityValueDetails);
        speed.setText(dto.getSpeedMean() + " km/h");
        TextView minKm = findViewById(R.id.textViewMinKmValueDetails);
        minKm.setText(dto.getMinKmMean());
        TextView time = findViewById(R.id.textViewTimeValueDetails);
        time.setText(dto.getFormatHour());

        TextView titulo = findViewById(R.id.textViewTitulo);
        String fecha = day + "/"+(Integer.parseInt(month)+1)+"/"+year;
        titulo.setText("ACTIVIDAD: "+ fecha+  " "+hour);
    }



    @Override
    protected void onDestroy(){

        if (dialog.isShowing()){
            dialog.dismiss();
        }
        super.onDestroy();
    }



}