package com.example.trainingPartner.controller.dialog;

import android.app.Activity;
import android.content.Context;
import android.text.InputFilter;
import android.widget.EditText;
import android.widget.TextView;

import com.example.mapbox2.R;
import com.example.trainingPartner.controller.fragment.MainWindowFragment;
import com.example.trainingPartner.model.DailyInformationModel;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Clase que simula el dialog de cambio de altura
 */
public class ChangeHeightDialog extends ChangeDataDialog{
    public ChangeHeightDialog(Context cont, DailyInformationModel model) {
        super(cont, model);
    }

    /**
     * Comprueba que el formato de la altura sea correcto
     * @param text
     * @return
     */
    @Override
    public boolean isCorrectFormat(EditText text) {
        String txt = String.valueOf(text.getText());
        String regularExpr = "\\d{1}(\\.\\d{1,2})";
        Pattern pattern = Pattern.compile(regularExpr);
        Matcher matcher = pattern.matcher(txt);
        return matcher.matches() && inTheLimits(text);
    }

    /**
     * Pone el error de que el formato es el incorrecto
     * @param text
     */
    @Override
    public void setFormatAndInfo(EditText text) {
        text.setHint("X.XX");
        format.setText("Formato: X.XX m");
        textInformation.setText("Introduce el nuevo valor para la altura");
        InputFilter[] filterArray = new InputFilter[1];
        filterArray[0] = new InputFilter.LengthFilter(4);
        text.setFilters(filterArray);

    }

    private boolean inTheLimits(EditText text){
        String s = text.getText().toString();
        Double d = Double.parseDouble(s);
        return d <= 2.50 && d >= 0.60;
    }

    @Override
    public void saveData(String data) {
        model.saveHeight(data);
        TextView tx = ((Activity) context).findViewById(R.id.textViewHeight);
        Double d = Double.parseDouble(data);
        tx.setText(d + " m");
        MainWindowFragment.dto.setHeight(Double.parseDouble(data));
        MainWindowFragment.dto.calcularLongitudPaso();

    }


}
