package com.example.trainingPartner.controller.dialog;

import android.app.Dialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.SharedPreferences;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.view.View;
import android.view.Window;
import android.widget.Button;
import android.widget.NumberPicker;

import com.example.mapbox2.R;
import com.example.trainingPartner.controller.fragment.HistoryFragment;

import java.util.Calendar;

/**
 * Dialog para seleccionar el mes y el año en el fragmento del historial
 */
public class MonthYearPickerDialog  {
    private Calendar cal = Calendar.getInstance();


    private Context context;
    private Dialog dialog;
    private SharedPreferences sharedPref;

    private NumberPicker yearPicker;
    private NumberPicker monthPicker;
    private HistoryFragment hs;


    public MonthYearPickerDialog(Context context, SharedPreferences sha, HistoryFragment hs){
        this.context = context;
        dialog = new Dialog(context);
        dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
        dialog.setContentView(R.layout.dialog_date);
        dialog.getWindow().setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
        yearPicker = (NumberPicker) dialog.findViewById(R.id.picker_year);
        monthPicker = (NumberPicker) dialog.findViewById(R.id.picker_month);
        sharedPref = sha;
        this.hs = hs;

        setPickers();
        dialog.setOnCancelListener(new DialogInterface.OnCancelListener() {
            @Override
            public void onCancel(DialogInterface dialogInterface) {
                getValues();
                dialogInterface.dismiss();
            }
        });
        dialog.show();
    }

    /**
     * Crea los pickers
     */
    private void setPickers(){
        int year = cal.get(Calendar.YEAR);
        yearPicker.setMinValue(year-20);
        yearPicker.setMaxValue(year);
        yearPicker.setValue(sharedPref.getInt("year", year));

        monthPicker.setDisplayedValues(new String[]{"Enero","Febrero", "Marzo", "Abril", "Mayo", "Junio", "Julio", "Agosto", "Septiembre",
        "Ocubre", "Noviembre", "Diciembre"});
        monthPicker.setMinValue(0);
        monthPicker.setMaxValue(11);
        monthPicker.setValue(sharedPref.getInt("month", cal.get(Calendar.MONTH)));

        Button button = dialog.findViewById(R.id.buttonSelectDate);
        button.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                getValues();
                dialog.dismiss();
            }
        });

    }

    /**
     * Consigue los valores de los pickers
     */
    private void getValues(){
        int month = monthPicker.getValue();
        int year = yearPicker.getValue();
        SharedPreferences.Editor editor = sharedPref.edit();
        editor.putInt("month", month);
        editor.putInt("year", year);
        editor.apply();
        setCalendarFragment();

    }

    public int getMonth(){
        return sharedPref.getInt("month", cal.get(Calendar.MONTH));
    }

    public int getYear(){
        return sharedPref.getInt("year",cal.get(Calendar.YEAR));
    }

    private void setCalendarFragment(){
        hs.setCalendar(getYear(), getMonth());
    }

}
