package com.example.trainingPartner.controller.fragment;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.example.trainingPartner.model.MyCallbBack;
import com.example.trainingPartner.utils.ListAdapt;
import com.example.mapbox2.R;
import com.example.trainingPartner.utils.Utils;
import com.example.trainingPartner.dto.DayInformationDTO;
import com.example.trainingPartner.dto.activityDto.ActivityDTO;
import com.example.trainingPartner.model.DailyInformationModel;

import java.util.Calendar;
import java.util.List;

public class HistoricalDayFragment extends Fragment {
    private View v;
    private DailyInformationModel model;
    private int year, month, day;
    private List<ActivityDTO> activityDTOList;

    public  HistoricalDayFragment(int day, int month, int year){
        this.year = year;
        this.month = month;
        this.day = day;
    }


    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.MONTH, month);
        calendar.set(Calendar.DAY_OF_MONTH, day);
        calendar.set(Calendar.YEAR, year);
        String username = Utils.getUserName(getActivity());
        model = new DailyInformationModel(calendar, username);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        v = inflater.inflate(R.layout.fragment_historicalday, container, false);
        setActivities();
        setCaloriesAndSteps();
        setTitle();
        return v;
    }

    private void setActivities(){
        model.getActivitiesOfTheDay(new MyCallbBack<List<ActivityDTO>>() {
            @Override
            public void onCallback(List<ActivityDTO> element) {
                activityDTOList= element;
                ListAdapt adapter = new ListAdapt(activityDTOList, getActivity());
                RecyclerView recyclerView = v.findViewById(R.id.recyclerViewHistoricalDay);
                recyclerView.setHasFixedSize(true);
                recyclerView.setLayoutManager(new LinearLayoutManager(getActivity()));
                recyclerView.setAdapter(adapter);
            }
        });

    }

    private void setCaloriesAndSteps(){
        model.getStepsAndCalories(new MyCallbBack<DayInformationDTO>() {
            @Override
            public void onCallback(DayInformationDTO element) {
                View customLayout = v.findViewById(R.id.steps_hist);
                TextView textView = customLayout.findViewById(R.id.textViewSteps);
                textView.setText(Utils.setStepsString(element.steps));

                View customLayoutCal = v.findViewById(R.id.calorie_hist);
                TextView textViewCal = customLayoutCal.findViewById(R.id.textViewCalories);
                textViewCal.setText(Utils.setStepsString((int)element.calories));
            }
        });

    }

    public String setTitle(){
        TextView title = v.findViewById(R.id.textViewTitleHistoricalDay);
        TextView subTi = v.findViewById(R.id.textViewHistoricalTitle);
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.YEAR, year);
        calendar.set(Calendar.MONTH, month);
        calendar.set(Calendar.DAY_OF_MONTH, day);
        int day = calendar.get(Calendar.DAY_OF_WEEK);
        String[] days = {"Domingo", "Lunes", "Martes", "Miércoles", "Jueves", "Viernes", "Sabado"};
        String dayOfWeek = days[day-1];
        title.setText(days[day-1]);
        String sub = this.day +"/"+(month +1)+"/"+ year;

        subTi.setText(sub);

        return dayOfWeek + " "+ sub;
    }


}
