package com.example.trainingPartner.controller.fragment;


import android.app.ProgressDialog;
import android.content.Context;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.text.style.ForegroundColorSpan;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.core.content.ContextCompat;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentTransaction;

import com.example.mapbox2.R;
import com.example.trainingPartner.controller.MainActivity;
import com.example.trainingPartner.model.MyCallbBack;
import com.example.trainingPartner.utils.Utils;
import com.example.trainingPartner.controller.dialog.MonthYearPickerDialog;
import com.example.trainingPartner.model.HistoryModel;
import com.prolificinteractive.materialcalendarview.CalendarDay;
import com.prolificinteractive.materialcalendarview.MaterialCalendarView;
import com.prolificinteractive.materialcalendarview.DayViewDecorator;
import com.prolificinteractive.materialcalendarview.DayViewFacade;
import com.prolificinteractive.materialcalendarview.OnDateSelectedListener;
import com.prolificinteractive.materialcalendarview.OnMonthChangedListener;

import java.util.Calendar;
import java.util.Collection;
import java.util.HashSet;

public class HistoryFragment extends Fragment {

    private View v;
    private SharedPreferences sharedPref;
    private MaterialCalendarView calendarView;
    private HistoryModel model;
    private TextView dateText;
    private int y, m;

    private int selectedDay, selectedMonth, selectedYear;
    private ProgressDialog dialog;
    public int daysUsed;
    private HashSet<CalendarDay> enableDays;


    public HistoryFragment(){
        y = Calendar.getInstance().get(Calendar.YEAR);
        m =  Calendar.getInstance().get(Calendar.MONTH);

    }
    public HistoryFragment(ProgressDialog dialog){
        this.dialog=dialog;
        y = Calendar.getInstance().get(Calendar.YEAR);
        m =  Calendar.getInstance().get(Calendar.MONTH);
    }

    public int getDaysUsed(){
        return daysUsed;
    }


    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        String username = Utils.getUserName(getActivity());
        System.out.println(getActivity().getLocalClassName());
        System.out.println(username);
        model = new HistoryModel(username);
    }




    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        v = inflater.inflate(R.layout.fragment_history, container, false);
        buttonAction();

        sharedPref = getActivity().getPreferences(Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = sharedPref.edit();
        editor.putInt("month", m);
        editor.putInt("year", y);
        dateText = v.findViewById(R.id.textViewDate);
        editor.apply();
        setCalendar(y,m);

        calendarView.setOnMonthChangedListener(new OnMonthChangedListener() {
            @Override
            public void onMonthChanged(MaterialCalendarView widget, CalendarDay date) {
                y = date.getYear();
                m =  date.getMonth();
                System.out.println(y+" "+m);
                desActivateDates();
                v.findViewById(R.id.buttonVisualize).setVisibility(View.INVISIBLE);
                widget.clearSelection();
                dateText.setText("");

            }
        });

        calendarView.setOnDateChangedListener(new OnDateSelectedListener() {
            @Override
            public void onDateSelected(@NonNull MaterialCalendarView widget, @NonNull CalendarDay date, boolean selected) {
                v.findViewById(R.id.buttonVisualize).setVisibility(View.VISIBLE);
                dateText.setText(date.getDay() +"/"+(date.getMonth()+1)+"/"+date.getYear());
                selectedDay = date.getDay();
                selectedMonth=date.getMonth();
                selectedYear=date.getYear();
            }
        });
        if (dialog!=null)
            dialog.dismiss();
        return v;
    }

    public void desActivateDates()  {
        System.out.println(y);
        System.out.println(m);
        Context context = getActivity().getApplicationContext();
        model.getDays(y, m, new MyCallbBack<HashSet<CalendarDay>>() {
            private int en;
            @Override
            public void onCallback(HashSet<CalendarDay> calendarDays) {
                enableDays = calendarDays;
                setDaysUsed(calendarDays.size());
                calendarView.addDecorator(new Decorator(enableDays));
                calendarView.addDecorator(new DecoratorActivate(enableDays, context));
                System.out.println(daysUsed);
                System.out.println(getDaysUsed());
            }
        });



    }

    private void setDaysUsed(int a){
        this.daysUsed = a;
    }
    public HashSet<CalendarDay> getEnableDays(){
        return enableDays;
    }
    private void buttonAction() {
        Button button = v.findViewById(R.id.buttonDate);
        button.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                startDialog();
            }
        });

        Button buttonSee = v.findViewById(R.id.buttonVisualize);
        buttonSee.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                MainActivity activity = (MainActivity) getActivity();
                activity.setPreviousItem(R.id.history);
                Fragment fragment = new HistoricalDayFragment(selectedDay,selectedMonth,selectedYear);
                openFragment(fragment);
            }
        });
    }


    private void startDialog(){

        new MonthYearPickerDialog(getContext(),sharedPref, this);
    }

    private void openFragment(Fragment fragment) {
        FragmentTransaction transaction = getActivity().getSupportFragmentManager().beginTransaction();
        //this is a helper class that replaces the container with the fragment. You can replace or add fragments.
        transaction.replace(R.id.container, fragment);
        transaction.addToBackStack(null); //if you add fragments it will be added to the backStack. If you replace the fragment it will add only the last fragment
        transaction.commit(); // commit() performs the action
    }

    public void setCalendar(int year, int month){
        calendarView = v.findViewById(R.id.calendarView2);
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.YEAR, year);
        calendar.set(Calendar.MONTH, month);
        this.y = year;
        this.m = month;
        desActivateDates();
        calendarView.setCurrentDate(calendar.getTime());

    }


}

class Decorator implements DayViewDecorator{

    private final Calendar calendar = Calendar.getInstance();
    private final HashSet<CalendarDay> disabledDays;

    public Decorator(Collection<CalendarDay> disabledDays) {
        this.disabledDays = new HashSet<>(disabledDays);

    }

    @Override
    public boolean shouldDecorate(CalendarDay day) {
        return !disabledDays.contains(day);
    }

    @Override
    public void decorate(DayViewFacade view) {
        view.setDaysDisabled(true);

    }
}



class DecoratorActivate implements DayViewDecorator{

    private final Calendar calendar = Calendar.getInstance();
    private final HashSet<CalendarDay> disabledDays;
    private Context context;

    public DecoratorActivate(Collection<CalendarDay> disabledDays, Context context) {
        this.disabledDays = new HashSet<>(disabledDays);
        this.context = context;
    }

    @Override
    public boolean shouldDecorate(CalendarDay day) {
        return disabledDays.contains(day);
    }

    @Override
    public void decorate(DayViewFacade view) {
        view.setDaysDisabled(false);
        view.addSpan(new ForegroundColorSpan(ContextCompat.getColor(context,R.color.white)));
    }


}