package com.example.trainingPartner.controller.fragment;

import android.app.ProgressDialog;
import android.graphics.Bitmap;
import android.os.Bundle;

import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.example.trainingPartner.model.MyCallbBack;
import com.example.trainingPartner.utils.ListAdapt;
import com.example.mapbox2.R;
import com.example.trainingPartner.utils.Utils;
import com.example.trainingPartner.controller.dialog.ChangeHeightDialog;
import com.example.trainingPartner.controller.dialog.ChangeWeightDialog;
import com.example.trainingPartner.controller.dialog.SignOutDialog;
import com.example.trainingPartner.dto.UserInformationDTO;
import com.example.trainingPartner.dto.activityDto.ActivityDTO;
import com.example.trainingPartner.model.DailyInformationModel;

import com.theartofdev.edmodo.cropper.CropImage;
import com.theartofdev.edmodo.cropper.CropImageView;

import java.io.ByteArrayOutputStream;
import java.util.List;


public class MainWindowFragment extends Fragment {
    private DailyInformationModel model;
    public static UserInformationDTO dto;
    private View v;
    private List<ActivityDTO> activityDTOList;
    private ProgressDialog dialog;

    public MainWindowFragment(){

    }
    public MainWindowFragment(ProgressDialog dialog){
        this.dialog = dialog;
    }


    public DailyInformationModel getModel(){
        return model;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        String username = Utils.getUserName(getActivity());
        model = new DailyInformationModel(username);
        model.getData(new MyCallbBack<UserInformationDTO>() {
            @Override
            public void onCallback(UserInformationDTO element) {
                dto = element;
                setInfo();
                setImage();

                ImageButton buttonProfile = v.findViewById(R.id.imageButtonProfile);
                buttonProfile.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        startCropActivity();
                    }
                });
                setChangeDataActions();
                setActivities();
                signOut();

            }
        });

    }

    @Override
    public void onResume() {
        super.onResume();
    }



    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        v = inflater.inflate(R.layout.fragment_main_window, container, false);

        return v;
    }

    private void setChangeDataActions(){
        LinearLayout height = v.findViewById(R.id.linearHeight);
        height.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

                new ChangeHeightDialog(getContext(), model);

            }
        });
        LinearLayout weight = v.findViewById(R.id.linearWeight);
        weight.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                new ChangeWeightDialog(getContext(), model);
            }
        });
    }

    private void signOut(){
        ImageView signOut = v.findViewById(R.id.imageViewSignOut);
        signOut.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                new SignOutDialog(getContext());
            }
        });
    }

    public void setActivities(){
        model.getActivitiesOfTheDay(new MyCallbBack<List<ActivityDTO>>() {
            @Override
            public void onCallback(List<ActivityDTO> element) {
                activityDTOList=element;
                ListAdapt adapter = new ListAdapt(activityDTOList, getActivity());
                RecyclerView recyclerView = v.findViewById(R.id.recyclerView);
                recyclerView.setHasFixedSize(true);
                recyclerView.setLayoutManager(new LinearLayoutManager(getActivity()));
                recyclerView.setAdapter(adapter);
            }
        });

    }

    public void setImage(){
        model.getImage(new MyCallbBack<Bitmap>() {
            @Override
            public void onCallback(Bitmap bmp) {
                Bitmap b3 = Utils.getRoundedCroppedBitmap(bmp);
                ByteArrayOutputStream outStream = new ByteArrayOutputStream();
                ImageButton buttonProfile = v.findViewById(R.id.imageButtonProfile);
                buttonProfile.setImageBitmap(b3);
                if (dialog!=null)
                    dialog.dismiss();
            }
        });

    }

    public UserInformationDTO getUserInfo(){
        return dto;
    }
    public List<ActivityDTO> getActivities(){
        return activityDTOList;
    }




    private void startCropActivity(){
        CropImage.activity()
                .setGuidelines(CropImageView.Guidelines.ON).setMinCropResultSize(400,400)
                .setMaxCropResultSize(4000,4000).setAspectRatio(1,1)
                .setCropShape(CropImageView.CropShape.OVAL).start(getActivity());


    }


    private void setInfo(){
        TextView name = v.findViewById(R.id.textViewName);
        name.setText(dto.getName()+ " "+ dto.getSurname());
        TextView weight = v.findViewById(R.id.textViewWeight);
        weight.setText(dto.getWeight() +" kg");
        TextView height = v.findViewById(R.id.textViewHeight);
        height.setText(dto.getHeight() + " m");
    }


}