package com.example.trainingPartner.controller.service;

import android.app.ActivityManager;
import android.app.Notification;
import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.os.Binder;
import android.os.Build;
import android.os.Handler;
import android.os.HandlerThread;
import android.os.IBinder;
import android.util.Log;
import android.widget.Toast;

import androidx.core.app.NotificationCompat;

import com.example.mapbox2.R;
import com.example.trainingPartner.controller.MainActivity;

public class StepCounterService extends Service implements SensorEventListener {

    private static final int STEPS_CONSTANT = 40;

    private static final String CHANNEL_ID = "channel_02";
    private static final int NOTIFICATION_ID = 987654;
    public static final String STEPS = "STEPS";
    public static final String STEPS_EXTRA = "STEPS_EXTRA";
    private boolean mChangingConfiguration = false;
    private Handler mServiceHandler;
    private static final String TAG = StepCounterService.class.getSimpleName();
    private NotificationManager mNotificationManager;
    SensorManager sensorManager;
    private double magnitudePrevious =0;
    private Integer steps;
    private Intent intent;
    private Sensor countSensor;

    public StepCounterService() {
    }
    private final IBinder mBinder = new StepCounterService.LocalBinder();
    @Override
    public IBinder onBind(Intent intent) {
        stopForeground(true);
        mChangingConfiguration = false;
        return mBinder;
    }

    @Override
    public void onRebind(Intent intent) {
        // Called when a client (MainActivity in case of this sample) returns to the foreground
        // and binds once again with this service. The service should cease to be a foreground
        // service when that happens.
        Log.i(TAG, "in onRebind()");
        stopForeground(true);
        mChangingConfiguration = false;
        super.onRebind(intent);
    }

    @Override
    public boolean onUnbind(Intent intent) {

        Log.i(TAG, "Last client unbound from service");

        // Called when the last client (MainActivity in case of this sample) unbinds from this
        // service. If this method is called due to a configuration change in MainActivity, we
        // do nothing. Otherwise, we make this service a foreground service.
        if (!mChangingConfiguration) {
            Log.i(TAG, "Starting foreground service");

            startForeground(NOTIFICATION_ID, getNotification());

        }
        return true; // Ensures onRebind() is called when a client re-binds.
    }


    @Override
    public void onCreate() {
        super.onCreate();
       setUpSensor();

    }

    private void setUpSensor(){
        HandlerThread handlerThread = new HandlerThread(TAG);
        handlerThread.start();
        mServiceHandler = new Handler(handlerThread.getLooper());
        mServiceHandler.post(updateBroadcastData);
        sensorManager = (SensorManager) getSystemService(Context.SENSOR_SERVICE);
        steps = MainActivity.sharedPreferences.getInt("stepCount", 0);
        mNotificationManager = (NotificationManager) getSystemService(NOTIFICATION_SERVICE);
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            CharSequence name = getString(R.string.app_name);
            // Create the channel for the notification
            NotificationChannel mChannel =
                    new NotificationChannel(CHANNEL_ID, name, NotificationManager.IMPORTANCE_DEFAULT);

            // Set the Notification Channel for the Notification Manager.
            mNotificationManager.createNotificationChannel(mChannel);
        }
    }

    public class LocalBinder extends Binder {
        public StepCounterService getService() {
            return StepCounterService.this;
        }
    }

    /**
     * Returns true if this is a foreground service.
     *
     * @param context The {@link Context}.
     */
    public boolean serviceIsRunningInForeground(Context context) {
        ActivityManager manager = (ActivityManager) context.getSystemService(
                Context.ACTIVITY_SERVICE);
        for (ActivityManager.RunningServiceInfo service : manager.getRunningServices(
                Integer.MAX_VALUE)) {
            if (getClass().getName().equals(service.service.getClassName())) {
                if (service.foreground) {
                    return true;
                }
            }
        }
        return false;
    }

    private void registerSensor(){
        countSensor = sensorManager.getDefaultSensor(Sensor.TYPE_ACCELEROMETER);
        if(countSensor != null){
            System.out.println("REGISTRANDO");
            sensorManager.registerListener(this, countSensor, SensorManager.SENSOR_DELAY_NORMAL);
        }else{
            Toast.makeText(this, "Sensor not found", Toast.LENGTH_SHORT).show();
        }
    }

    private void unRegisterSensor(){
        sensorManager.unregisterListener(this);
    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        registerSensor();

        return Service.START_NOT_STICKY;

    }

    @Override
    public void onSensorChanged(SensorEvent event) {
        if(event!=null){
            //Se accede al sensor de acelerómetro y se consiguen los movimientos en los 3 ejes
            float x_acc = event.values[0];
            float y_acc = event.values[1];
            float z_acc = event.values[2];

            double magnitude = Math.sqrt(x_acc*x_acc + y_acc*y_acc * z_acc*z_acc);
            double magnitudeDelta = magnitude -magnitudePrevious;
            magnitudePrevious = magnitude;
            //Si ha habido un cambio superior a STEP_CONSTANT, se suma un paso
            if (magnitudeDelta>STEPS_CONSTANT){
                steps++;
                SharedPreferences.Editor editor =MainActivity.sharedPreferences.edit();
                editor.putInt("stepCount", steps);
                editor.apply();
            }

        }
    }



    private void sendStepsToTheActivity() {
        Intent intent = new Intent(STEPS);
        intent.putExtra(STEPS_EXTRA, steps);
        sendBroadcast(intent);
    }

    private Notification getNotification() {
        Intent intent = new Intent(this, StepCounterService.class);

        CharSequence text = "PASOS: " +steps.toString();

        // The PendingIntent that leads to a call to onStartCommand() in this service.
        PendingIntent servicePendingIntent = PendingIntent.getService(this, 0, intent,
                PendingIntent.FLAG_UPDATE_CURRENT);

        // The PendingIntent to launch activity.
        PendingIntent activityPendingIntent = PendingIntent.getActivity(this, 0,
                new Intent(this, MainActivity.class), 0);

        NotificationCompat.Builder builder = new NotificationCompat.Builder(this)
                .setContentText(text)
                .setContentTitle("RUNNING PARTNER")
                .setOngoing(true)
                .setPriority(Notification.PRIORITY_LOW)
                .setSmallIcon(R.mipmap.ic_launcher)
                .setTicker(text).setVibrate(new long[]{0L})
                .setWhen(System.currentTimeMillis());

        // Set the Channel ID for Android O.
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            builder.setChannelId(CHANNEL_ID); // Channel ID
        }


        return builder.build();
    }


    @Override
    public void onAccuracyChanged(Sensor sensor, int accuracy) {

    }

    @Override
    public void onDestroy(){
        mServiceHandler.removeCallbacksAndMessages(null);
    }

    public void requestSteps(int t) {
        this.steps=t;
        intent= new Intent(getApplicationContext(), StepCounterService.class);
        startService(intent);
    }

    public void stopRequest() {
        sensorManager.unregisterListener(this);
        stopService(intent);

        mServiceHandler.removeCallbacks(updateBroadcastData);
    }



    private Runnable updateBroadcastData = new Runnable() {
        public void run() {
            if (serviceIsRunningInForeground(getApplicationContext())) {
                mNotificationManager.notify(NOTIFICATION_ID, getNotification());
                System.out.println("ACTUALIZANDO");
            }else{
                sendStepsToTheActivity();
            }
            mServiceHandler.postDelayed(this, 1000);

        }
    };


}



