package com.example.trainingPartner.controller.service;

import android.app.Activity;
import android.app.ActivityManager;
import android.app.Notification;
import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.app.ProgressDialog;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.content.res.Configuration;
import android.location.Location;
import android.os.Binder;
import android.os.Build;
import android.os.Handler;
import android.os.HandlerThread;
import android.os.IBinder;
import android.os.Looper;
import android.util.Log;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.core.app.NotificationCompat;
import androidx.localbroadcastmanager.content.LocalBroadcastManager;

import com.example.mapbox2.R;
import com.example.trainingPartner.utils.Utils;
import com.example.trainingPartner.controller.RecordActivity;
import com.example.trainingPartner.dto.PointDTO;
import com.google.android.gms.location.FusedLocationProviderClient;
import com.google.android.gms.location.LocationAvailability;
import com.google.android.gms.location.LocationCallback;
import com.google.android.gms.location.LocationRequest;
import com.google.android.gms.location.LocationResult;
import com.google.android.gms.location.LocationServices;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.OnFailureListener;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.android.gms.tasks.Task;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

public class TrackingService extends Service {

    public static final String REQUESTPERMISSION = "REQUEST_PERMISSION";
    private static final String TRACKING_NOTIFICATION = "TRACKING_NOTIFICATION";
    public static String TRACKING_DISTANCE = "TRACKING_DISTANCE";
    public static String TRACKING_SPEED = "TRACKING_SPEED";
    public static String TRACKING_COORDINATES = "TRACKING_COORDINATES";
    public static String SPEED_EXTRA = "SPEED_EXTRA";
    public static String DISTANCE_EXTRA = "DISTANCE_EXTRA";
    public static String COORDINATES_EXTRA = "COORDINATES_EXTRA";
    public static String PARCIALS_EXTRA = "PARCIALS_EXTRA";
    public static String TIME_EXTRA = "timeExtra";
    public static String TIMER_UPDATED = "timerUpdated";

    private HashMap<String, Double> velocityPerKilometer;
    private Timer timer ;
    private Location currentLocation;
    private Location previousLocation=null;
    private boolean firstLocation = true;
    private int decimasAntiguas=0;
    private int decimasNuevas =0;
    private TimeTask timeTask;
    private String text= "";
    private ProgressDialog progress;
    private Activity activity;

    private double kilometers=0;
    private boolean firstDistance=true;

    private double timeInitial=0;
    private List<PointDTO> route;

    public static final String PACKAGE_NAME =
            "com.google.android.gms.location.sample.locationupdatesforegroundservice";

    private static final String TAG = TrackingService.class.getSimpleName();

    /**
     * The name of the channel for notifications.
     */
    private static final String CHANNEL_ID = "channel_01";

    public static final String ACTION_BROADCAST = PACKAGE_NAME + ".broadcast";

    public static final String EXTRA_LOCATION = PACKAGE_NAME + ".location";

    private final IBinder mBinder = new LocalBinder();

    /**
     * The desired interval for location updates. Inexact. Updates may be more or less frequent.
     */
    private static final long UPDATE_INTERVAL_IN_MILLISECONDS = 10000;

    /**
     * The fastest rate for active location updates. Updates will never be more frequent
     * than this value.
     */
    private static final long FASTEST_UPDATE_INTERVAL_IN_MILLISECONDS =
            10000 / 2;

    /**
     * The identifier for the notification displayed for the foreground service.
     */
    private static final int NOTIFICATION_ID = 12345678;

    /**
     * Used to check whether the bound activity has really gone away and not unbound as part of an
     * orientation change. We create a foreground service notification only if the former takes
     * place.
     */
    private boolean mChangingConfiguration = false;
    private boolean stopped = false;

    private NotificationManager mNotificationManager;

    /**
     */
    private LocationRequest mLocationRequest;

    /**
     * Provides access to the Fused Location Provider API.
     */
    private FusedLocationProviderClient mFusedLocationClient;

    /**
     * Callback for changes in location.
     */
    private LocationCallback mLocationCallback;

    private Handler mServiceHandler;

    /**
     * The current location.
     */
    private Location mLocation;




    public TrackingService() {
    }

    @Override
    public void onCreate() {
        mFusedLocationClient = LocationServices.getFusedLocationProviderClient(this);

        mLocationCallback = new LocationCallback() {
            @Override
            public void onLocationResult(LocationResult locationResult) {
                super.onLocationResult(locationResult);

                onNewLocation(locationResult.getLastLocation());
            }
        };
        route = new ArrayList<>();
        velocityPerKilometer = new HashMap<>();

        createLocationRequest();
        progress = RecordActivity.progress;
        waitForLocation(progress);
        getLastLocation();

        HandlerThread handlerThread = new HandlerThread(TAG);
        handlerThread.start();
        mServiceHandler = new Handler(handlerThread.getLooper());
        mNotificationManager = (NotificationManager) getSystemService(NOTIFICATION_SERVICE);

        // Android O requires a Notification Channel.
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            CharSequence name = getString(R.string.app_name);
            // Create the channel for the notification
            NotificationChannel mChannel =
                    new NotificationChannel(CHANNEL_ID, name, NotificationManager.IMPORTANCE_DEFAULT);

            // Set the Notification Channel for the Notification Manager.
            mNotificationManager.createNotificationChannel(mChannel);
        }
    }

    private void waitForLocation(ProgressDialog progress) {

        try {
            OnSuccessListener listener = new OnSuccessListener<LocationAvailability>() {
                @Override
                public void onSuccess(LocationAvailability locationAvailability) {
                    if (locationAvailability.isLocationAvailable()){
                        progress.dismiss();
                    }else{
                        waitForLocation(progress);
                    }
                }
            };

            OnFailureListener listener2 = new OnFailureListener() {
                @Override
                public void onFailure(@NonNull Exception e) {
                    Intent intent = new Intent(REQUESTPERMISSION);
                    sendBroadcast(intent);
                }
            };

             mFusedLocationClient.getLocationAvailability().addOnFailureListener(listener2);
             mFusedLocationClient.getLocationAvailability().addOnSuccessListener(listener);
        } catch (SecurityException unlikely) {
            Log.e(TAG, "Lost location permission." + unlikely);
        }
    }



    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {

        // Tells the system to not try to recreate the service after it has been killed.
        return START_NOT_STICKY;
    }

    @Override
    public void onConfigurationChanged(Configuration newConfig) {
        super.onConfigurationChanged(newConfig);
        mChangingConfiguration = true;
    }

    @Override
    public IBinder onBind(Intent intent) {
        // Called when a client (MainActivity in case of this sample) comes to the foreground
        // and binds with this service. The service should cease to be a foreground service
        // when that happens.
        Log.i(TAG, "in onBind()");
        stopForeground(true);
        mChangingConfiguration = false;
        return mBinder;
    }

    @Override
    public void onRebind(Intent intent) {
        // Called when a client (MainActivity in case of this sample) returns to the foreground
        // and binds once again with this service. The service should cease to be a foreground
        // service when that happens.
        Log.i(TAG, "in onRebind()");
        stopForeground(true);
        mChangingConfiguration = false;
        super.onRebind(intent);
    }

    @Override
    public boolean onUnbind(Intent intent) {
        Log.i(TAG, "Last client unbound from service");

        // Called when the last client (MainActivity in case of this sample) unbinds from this
        // service. If this method is called due to a configuration change in MainActivity, we
        // do nothing. Otherwise, we make this service a foreground service.
        if (!mChangingConfiguration && Utils.requestingLocationUpdates(this)) {
            Log.i(TAG, "Starting foreground service");

            startForeground(NOTIFICATION_ID, getNotification());
        }
        return true; // Ensures onRebind() is called when a client re-binds.
    }

    @Override
    public void onDestroy() {
        mServiceHandler.removeCallbacksAndMessages(null);
        if (mLocation != null)
            route.add(new PointDTO(mLocation.getLatitude(), mLocation.getLongitude()));
        removeLocationUpdates(true);
        sendRouteToTheActivity();
        if (timer!= null)
            timer.cancel();
    }

    /**
     * Makes a request for location updates. Note that in this sample we merely log the
     * {@link SecurityException}.
     */
    public void requestLocationUpdates(double t) {
        timer = new Timer();
        timeTask = new TimeTask(t);
        timer.scheduleAtFixedRate(timeTask, 0, 1000);
        Log.i(TAG, "Requesting location updates");
        Utils.setRequestingLocationUpdates(this, true);
        startService(new Intent(getApplicationContext(), TrackingService.class));

        try {
            mFusedLocationClient.requestLocationUpdates(mLocationRequest,
                    mLocationCallback, Looper.myLooper());
        } catch (SecurityException unlikely) {
            Utils.setRequestingLocationUpdates(this, false);
            Log.e(TAG, "Lost location permission. Could not request updates. " + unlikely);
        }
    }

    /**
     * Removes location updates. Note that in this sample we merely log the
     * {@link SecurityException}.
     */
    public void removeLocationUpdates(boolean condition) {

        if (timer!= null)
            timer.cancel();
        Log.i(TAG, "Removing location updates");
        try {
            route.add(new PointDTO(0, 0));
            stopped=true;
            mFusedLocationClient.removeLocationUpdates(mLocationCallback);
            Utils.setRequestingLocationUpdates(this, false);
            if (condition)
                stopSelf();
        } catch (SecurityException unlikely) {
            Utils.setRequestingLocationUpdates(this, true);
            Log.e(TAG, "Lost location permission. Could not remove updates. " + unlikely);
        }
    }

    /**
     * Returns the {@link NotificationCompat} used as part of the foreground service.
     */
    private Notification getNotification() {
        Intent intent = new Intent(this, TrackingService.class);

        CharSequence text = Utils.getLocationText(mLocation);
        String title = "ACTIVIDAD EN CURSO";
        String contentText = "TIEMPO: "+ Utils.returnStringTime(timeTask.getTime()) + "\t\tDISTANCIA: " + Utils.roundTwoDecimals(kilometers) +" km";
        NotificationCompat.Builder builder = new NotificationCompat.Builder(this)
                .setContentText(contentText)
                .setContentTitle(title)
                .setOngoing(true)
                .setPriority(Notification.PRIORITY_DEFAULT)
                .setSmallIcon(R.mipmap.ic_launcher)
                .setTicker(contentText)
                .setWhen(System.currentTimeMillis());

        // Set the Channel ID for Android O.
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            builder.setChannelId(CHANNEL_ID); // Channel ID
        }

        return builder.build();
    }

    private void getLastLocation() {
        try {
            mFusedLocationClient.getLastLocation()
                    .addOnCompleteListener(new OnCompleteListener<Location>() {
                        @Override
                        public void onComplete(@NonNull Task<Location> task) {
                            if (task.isSuccessful() && task.getResult() != null) {
                                mLocation = task.getResult();
                                previousLocation = task.getResult();
                            } else {
                                Log.w(TAG, "Failed to get location.");
                            }
                        }
                    });
        } catch (SecurityException unlikely) {
            Log.e(TAG, "Lost location permission." + unlikely);
        }
    }

    private void onNewLocation(Location location) {
        Log.i(TAG, "New location: " + location);
        mLocation = location;
        currentLocation = location;
        System.out.println("PREVIOUS LOCATION: "+previousLocation);
        System.out.println(currentLocation);
        if (stopped){
            previousLocation=location;
            stopped=false;
        }
        if (currentLocation==null || previousLocation==null){
            //En este momento se ha perdido la ubicación.
            previousLocation=currentLocation;
            progress.show();
            return;
        }
        //Aquí se ha encontrado la ubicación
        if(progress.isShowing())
            progress.dismiss();
        double d = distancia(previousLocation, currentLocation);
        if (firstLocation && d>0.07){
            //Por si hay un desfase al comenzar la actividad.
            d = 0;
            firstLocation=false;
        }
        kilometers += d; //Suma la distancia
        saveRoute(location, kilometers); //Comprueba si hay que actualizar los parciales y los puntos geográficos
        sendDataToActivity(kilometers); //Manda los datos a la actividad
        double speed = velocidad(location);
        sendSpeedToActivity(speed); //Manda la velocidad a la actividad

        // Envía localizaciones nuevas
        Intent intent = new Intent(ACTION_BROADCAST);
        intent.putExtra(EXTRA_LOCATION, location);
        LocalBroadcastManager.getInstance(getApplicationContext()).sendBroadcast(intent);

        // Actualiza la notificación.
        if (serviceIsRunningInForeground(this)) {
            mNotificationManager.notify(NOTIFICATION_ID, getNotification());
        }
        previousLocation=location;
    }

    /**
     * Sets the location request parameters.
     */
    private void createLocationRequest() {
        mLocationRequest = new LocationRequest();
        mLocationRequest.setInterval(UPDATE_INTERVAL_IN_MILLISECONDS);
        mLocationRequest.setFastestInterval(FASTEST_UPDATE_INTERVAL_IN_MILLISECONDS);
        mLocationRequest.setPriority(LocationRequest.PRIORITY_HIGH_ACCURACY);
    }

    /**
     * Class used for the client Binder.  Since this service runs in the same process as its
     * clients, we don't need to deal with IPC.
     */
    public class LocalBinder extends Binder {
        public TrackingService getService() {
            return TrackingService.this;
        }
    }

    /**
     * Returns true if this is a foreground service.
     *
     * @param context The {@link Context}.
     */
    public boolean serviceIsRunningInForeground(Context context) {
        ActivityManager manager = (ActivityManager) context.getSystemService(
                Context.ACTIVITY_SERVICE);
        for (ActivityManager.RunningServiceInfo service : manager.getRunningServices(
                Integer.MAX_VALUE)) {
            if (getClass().getName().equals(service.service.getClassName())) {
                if (service.foreground) {
                    return true;
                }
            }
        }
        return false;
    }

    private class TimeTask extends TimerTask {
        private Double time;
        private TimeTask(Double time){
            this.time = time;
        }

        @Override
        public void run() {
            Intent intent = new Intent(TIMER_UPDATED);
            time++;
            if (serviceIsRunningInForeground(getApplicationContext())) {
                mNotificationManager.notify(NOTIFICATION_ID, getNotification());
            }
            intent.putExtra(TIME_EXTRA, time);
            sendBroadcast(intent);
        }

        public double getTime(){
            return time;
        }
    }

    private int getKilometer(double numero){
        int n = (int) numero;
        return n;
    }
    private void sendRouteToTheActivity() {
        Intent intent = new Intent(TRACKING_COORDINATES);
        intent.putExtra(COORDINATES_EXTRA, (Serializable) route);
        intent.putExtra(PARCIALS_EXTRA, velocityPerKilometer);
        sendBroadcast(intent);

    }

    private void sendDataToActivity(double kilometers) {
        Intent intent = new Intent(TRACKING_DISTANCE);
        intent.putExtra(DISTANCE_EXTRA,kilometers);
        sendBroadcast(intent);

    }

    private void sendSpeedToActivity(double velocidad) {
        Intent intent = new Intent(TRACKING_SPEED);
        intent.putExtra(SPEED_EXTRA, velocidad);
        sendBroadcast(intent);

    }
    private int getDecimas(double numero){
        double parteDecimal = (numero %1)*10;
        int parteEntera = (int)parteDecimal;
        return parteEntera;
    }



    private double velocidad(Location location){
        double speed = location.getSpeed();
        double s = speed*3.6;
        return Utils.roundTwoDecimals(s);

    }


    private double distancia(Location start, Location end){
        double km = 0;
        double m = Math.abs(start.distanceTo(end)/1000);
        km = Utils.roundTwoDecimals(m);
        return km;
    }




    private void saveRoute(Location location, double km) {
        int decimas = getDecimas(km);
        if (decimas -decimasAntiguas>=1){ //Aquí se comprueba si se han recorrido 100 metros
            decimasAntiguas=decimas;
            route.add(new PointDTO(location.getLatitude(), location.getLongitude()));
        }else if (decimasAntiguas>decimas){ //Aquí es que se ha recorrido un kilómetro
            double time = timeTask.getTime() - timeInitial;
            timeInitial = timeTask.getTime();
            //Guardado del kilometro
            velocityPerKilometer.put(getKilometer(km)+"", time);
            decimasAntiguas=decimas;
        }
    }




}