package com.example.trainingPartner.dto;

import com.example.trainingPartner.utils.Utils;

import java.time.LocalDate;
import java.time.Period;
import java.time.ZoneId;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

public class UserInformationDTO {
    public static final boolean MALE = true;
    public static final boolean FEMALE = false;

    private static final double STEP_MALE = 0.415;
    private static final double STEP_FEMALE = 0.413;


    private static final int EDAD1 = 20;
    private static final int EDAD2 = 40;
    private static final int EDAD3 = 50;
    private static final int EDAD4 = 60;

    private String name;
    private String username;
    private Date birthDate;

    private Double weight;
    private Double height;
    private boolean sex;
    private String photo;
    private String password;
    private String surname;
    private double longitud_paso;


    public UserInformationDTO(String name,String username, String surname, Date birth, double weight, double height, boolean sex,
                              String foto, String password){
        this.name = name;
        this.username=username;
        birthDate = birth;

        this.weight=weight;
        this.height=height;
        this.sex=sex;
        this.password= password;
        this.surname = surname;
        setPhoto(foto);
        calcularLongitudPaso();
    }

    public void setHeight(double h){
        this.height= h;
    }

    public void setWeight(double w){
        this.weight= w;
    }

    public void setPhoto(String photo){
        if (photo == null){
            photo = "default";
        }
        this.photo = photo;
    }

    public String getName() {
        return name;
    }

    public String getSurname(){return surname;}

    public void setName(String name) {
        this.name = name;
    }

    public String getUsername() {
        return username;
    }

    public void setUsername(String username) {
        this.username = username;
    }

    public Date getBirthDate() {
        return birthDate;
    }

    public Double getWeight() {
        return weight;
    }

    public Double getHeight() {
        return height;
    }

    public boolean isSex() {
        return sex;
    }

    public String getPhoto() {
        return photo;
    }

    private String encriptar(){
        String encriptada = Utils.getMD5(password);
        return encriptada;
    }

    public double getLongitud_paso(){
        return longitud_paso;
    }

    public Map<String, Object> getInformation(){
        Map<String,Object> info = new HashMap<>();
        info.put("Name", name);
        info.put("Birth", birthDate);
        info.put("Height", height/100);
        info.put("Weight", weight);
        info.put("Sex", sex);
        info.put("foto", photo);
        info.put("Username", username);
        info.put("numberOfActivities", 0);
        info.put("Surname",surname);
        info.put("password",  encriptar());

        return info;
    }

    private int getEdad(){

        if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.O) {
            LocalDate b=birthDate.toInstant().atZone(ZoneId.systemDefault()).toLocalDate();
            Period periodo = Period.between(b, LocalDate.now());
            int edad = periodo.getYears();
            return edad;
        }
        return 0;
    }


    public void calcularLongitudPaso(){
        double longitud_paso;
        if (sex)
            longitud_paso = height* STEP_MALE;
        else
            longitud_paso = height * STEP_FEMALE;
        longitud_paso *=100; //Convertimos a centimetros
        int edad = getEdad();
        if (edad<EDAD1){
            longitud_paso+=2;
        }else if (edad>=EDAD1 && edad<=(EDAD2-1)){
            longitud_paso+=0;
        }else if (edad>=EDAD2 && edad<=(EDAD3-1)){
            longitud_paso-=1;
        }else if (edad>=EDAD3 && edad<=(EDAD4-1)){
            longitud_paso-=2;
        }else if (edad>=EDAD4){
            longitud_paso-=3;
        }
        if (weight>70){
            longitud_paso+=1;
        }else if (weight<55){
            longitud_paso-=1;
        }
        this.longitud_paso= longitud_paso/100;
    }
}
