package com.example.trainingPartner.dto.activityDto;

import com.example.trainingPartner.utils.Utils;
import com.example.trainingPartner.dto.PointDTO;
import com.mapbox.geojson.Point;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class LocationActivityDTO extends ActivityDTO {
    public List<PointDTO> coordinates;
    public Map<String, Double> kilometers;

    public LocationActivityDTO(List<PointDTO> coordinates, Map<String, Double> kilometers, Double distance,
                               Double time, Calendar calendar){
        super(distance, time, ActivityDTO.AUTOMATIC, calendar);
        this.coordinates=coordinates;
        this.kilometers=kilometers;

    }

    public LocationActivityDTO(List<PointDTO> coordinates, Map<String, Double> kilometers, Double distance, Calendar calendar){
        this(coordinates,kilometers, distance,0.0, calendar );


    }

    public LocationActivityDTO(List<PointDTO> coordinates, Double time, Calendar calendar){
        this(coordinates, new HashMap<>(), 0.0,time, calendar);
    }

    public Map<String, Double> getPartials(){
        return kilometers;
    }


    public List<PointDTO> getCoordinates(){
        return coordinates;
    }

    public List<Point> getMapBoxPoints(){
        List<Point> points = new ArrayList<>();
        for (PointDTO point : coordinates){
            points.add(Point.fromLngLat(point.longitude, point.latitude));
        }
        return points;
    }

    private double setTime(Map<String,Double> kilometers){
        double time =0.0;
        for (double value: kilometers.values()){
            time+=value;
        }
        return time;
    }

    public Point getMiddlePoint(){
        List<Point> puntos = getMapBoxPoints();
        List<Point> withoutCero = removeCero(puntos);
        int middle = withoutCero.size()/2;
        return withoutCero.get(middle);
    }

    private List<Point> removeCero(List<Point> original){
        if (original.contains(Point.fromLngLat(0,0))){
            original.remove(Point.fromLngLat(0,0));
            return removeCero(original);
        }else{
            return original;
        }
    }





    public String getDistanceKilometer(int km){
        if (km == getPartials().size()-1){
            return Utils.roundTwoDecimals(distance-km) +" m";
        }
        return "1 km";

    }

    public String getSpeedKilometer(int km, double time){
        if (km == getPartials().size()-1){
            if (distance-km > 0.01){
                double d = distance-km;
                System.out.println("DISTANCE:  "+ d);
                double speed = calculateSpeed(time, d);
                return Utils.convertToMinKm(speed);
            }else{
                return "--:--";
            }

        }
        return Utils.returnStringTimePartials(time);
    }

    @Override
    public Map<String, Object> setValuesToSave(){
        Map<String, Object> actividad = new HashMap<>();
        actividad.put("distance", getDistance());
        actividad.put("kilometers", getPartials());
        actividad.put("coordinates", getCoordinates());
        actividad.put("time", getTime());
        actividad.put("type", getType());
        return actividad;
    }





}
