package com.example.trainingPartner.model;

import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.AsyncTask;

import androidx.annotation.NonNull;

import com.example.trainingPartner.utils.Utils;
import com.example.trainingPartner.dto.DayInformationDTO;
import com.example.trainingPartner.dto.UserInformationDTO;
import com.example.trainingPartner.dto.activityDto.ActivityDTO;
import com.example.trainingPartner.dto.activityDto.ManualActivityDTO;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.OnFailureListener;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.firestore.DocumentSnapshot;
import com.google.firebase.firestore.QuerySnapshot;
import com.google.firebase.firestore.SetOptions;
import com.google.firebase.storage.FirebaseStorage;
import com.google.firebase.storage.StorageReference;
import com.google.firebase.storage.UploadTask;

import java.io.ByteArrayOutputStream;
import java.lang.invoke.ConstantCallSite;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

public class DailyInformationModel extends Model{
    private UserInformationDTO dto;
    private FirebaseStorage storage ;
    private Bitmap bmp;
    private String month;
    private String year;
    private String day;
    private Calendar calendar;


    public DailyInformationModel(String user){
        super(user);
        storage = FirebaseStorage.getInstance("gs://pruebamapbox-3672b.appspot.com");

        calendar = Calendar.getInstance();
        month = calendar.get(Calendar.MONTH)+"";
        year = calendar.get(Calendar.YEAR)+"";
        day = calendar.get(Calendar.DAY_OF_MONTH)+"";
    }

    public DailyInformationModel(Calendar calendar, String user){
        super(user);

        storage = FirebaseStorage.getInstance("gs://pruebamapbox-3672b.appspot.com");
        this.calendar=calendar;
        month = calendar.get(Calendar.MONTH)+"";
        year = calendar.get(Calendar.YEAR)+"";
        day = calendar.get(Calendar.DAY_OF_MONTH)+"";
    }

    public void getData(MyCallbBack<UserInformationDTO> callBack) {
        db.collection("usuarios").document(username).get().addOnFailureListener(new OnFailureListener() {
            @Override
            public void onFailure(@NonNull Exception e) {
                try {
                    throw new Exception(e);
                } catch (Exception ex) {
                    ex.printStackTrace();
                }
            }
        }).addOnSuccessListener(new OnSuccessListener<DocumentSnapshot>() {
            @Override
            public void onSuccess(DocumentSnapshot documentSnapshot) {
                try{
                    Double height = documentSnapshot.getDouble("Height");
                    String name = documentSnapshot.getString("Name");
                    boolean sex = documentSnapshot.getBoolean("Sex");
                    String username = documentSnapshot.getString("Username");
                    String surname = documentSnapshot.getString("Surname");
                    double weight = documentSnapshot.getDouble("Weight");
                    Date birth = documentSnapshot.getDate("Birth");
                    String foto = documentSnapshot.getString("foto");
                    String pssw = documentSnapshot.getString("password");
                    dto = new UserInformationDTO(name, username,surname, birth, weight, height, sex, foto, pssw);
                    callBack.onCallback(dto);
                }catch(NullPointerException e){
                    //Por si se han eliminado las Shared
                }

            }
        });
    }



    public void saveImage(Bitmap bitmap, MyCallbBack<UserInformationDTO> callbBack){
        StorageReference storageRef = storage.getReference();
        StorageReference mountainsRef = storageRef.child("users/"+dto.getUsername()+".png");

        dto.setPhoto(dto.getUsername());
        db.collection("usuarios").document(dto.getUsername()).update("foto", dto.getPhoto());
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        bitmap.compress(Bitmap.CompressFormat.JPEG, 100, baos);
        byte[] data = baos.toByteArray();

        UploadTask uploadTask = mountainsRef.putBytes(data);
        uploadTask.addOnFailureListener(new OnFailureListener() {
            @Override
            public void onFailure(@NonNull Exception exception) {
                // Handle unsuccessful uploads
            }
        }).addOnSuccessListener(new OnSuccessListener<UploadTask.TaskSnapshot>() {
            @Override
            public void onSuccess(UploadTask.TaskSnapshot taskSnapshot) {
                // taskSnapshot.getMetadata() contains file metadata such as size, content-type, etc.
                // ...

                callbBack.onCallback(dto);
            }
        });
    }

    public void getImage(MyCallbBack<Bitmap> callbBack) {
        StorageReference storageRef = storage.getReference();

        StorageReference islandRef = storageRef.child("users/"+dto.getPhoto()+".png");


        final long ONE_MEGABYTE = 1024 * 1024;
        Task<byte[]> ref = islandRef.getBytes(ONE_MEGABYTE).addOnFailureListener(new OnFailureListener() {
            @Override
            public void onFailure(@NonNull Exception exception) {
                // Handle any errors
            }
        }).addOnCompleteListener(new OnCompleteListener<byte[]>() {
            @Override
            public void onComplete(@NonNull Task<byte[]> task) {
                try{
                    byte[] data = task.getResult();
                    BitmapFactory.Options options = new BitmapFactory.Options();
                    options.inMutable = true;
                    bmp = BitmapFactory.decodeByteArray(data, 0, data.length, options);
                    callbBack.onCallback(bmp);
                }catch (Exception e){
                    StorageReference islandRef = storageRef.child("users/"+"default"+".png");
                    islandRef.getBytes(ONE_MEGABYTE).addOnFailureListener(new OnFailureListener() {
                        @Override
                        public void onFailure(@NonNull Exception e) {

                        }
                    }).addOnCompleteListener(new OnCompleteListener<byte[]>() {
                        @Override
                        public void onComplete(@NonNull Task<byte[]> task) {
                            byte[] data = task.getResult();
                            BitmapFactory.Options options = new BitmapFactory.Options();
                            options.inMutable = true;
                            bmp = BitmapFactory.decodeByteArray(data, 0, data.length, options);
                            callbBack.onCallback(bmp);
                        }
                    });
                }

            }
        });

    }



    public void getActivitiesOfTheDay(MyCallbBack<List<ActivityDTO>> callbBack){
        Task<QuerySnapshot> collection =  db.collection("usuarios").document(username).
                collection("año").document(year).collection(month).
                document(day).collection("actividades").get().addOnFailureListener(new OnFailureListener() {
                    @Override
                    public void onFailure(@NonNull Exception e) {
                        try {
                            throw new Exception(e);
                        } catch (Exception ex) {
                            ex.printStackTrace();
                        }
                    }
                }).addOnSuccessListener(new OnSuccessListener<QuerySnapshot>() {
                    @Override
                    public void onSuccess(QuerySnapshot documents) {
                        try{
                            List<ActivityDTO> list = new ArrayList<>();
                            for (DocumentSnapshot document : documents) {
                                String s = document.getId();
                                boolean type = document.getBoolean("type");

                                double distance = document.getDouble("distance");
                                double time = document.getDouble("time");
                                ActivityDTO activity ;
                                if (type == ActivityDTO.MANUAL){//Significa que es manual
                                    activity = new ManualActivityDTO(distance, time, s, calendar);
                                }else{
                                    activity = Utils.saveLocationActivity(document, distance, time, calendar);
                                    activity.setHour(s);
                                }
                                list.add(activity);
                            }
                            callbBack.onCallback(list);
                        }catch(NullPointerException e){

                        }

                    }
                });



    }


    public void saveWeight(String weight){
        double w = Double.parseDouble(weight);
        dto.setWeight(w);
        HashMap<String, Double> weightMap = new HashMap<>();
        weightMap.put("Weight", w);
        db.collection("usuarios").document(username).set(weightMap, SetOptions.merge());

    }



    public void saveHeight(String height){

        double h = Double.parseDouble(height);
        dto.setHeight(h);
        HashMap<String, Double> heightMap = new HashMap<>();
        heightMap.put("Height", h);
        db.collection("usuarios").document(username).set(heightMap, SetOptions.merge());

    }


    public void saveStepsAndCalories(int actualY, int actualM, int actualD, int steps, int calories) {
        HashMap<String, Integer> st = new HashMap<>();
        st.put("steps", steps);
        st.put("calories", calories);
        db.collection("usuarios").document(username).
                collection("año").document(actualY+"").collection(actualM+"").
                document(actualD+"").set(st, SetOptions.merge());
        saveRegistry(actualY, actualM, actualD);
    }

    public void saveRegistry(int actualY, int actualM, int actualD){
        HashMap<String, Integer> regis = new HashMap<>();
        regis.put("year", actualY);
        regis.put("month", actualM);
        regis.put("day", actualD);
        db.collection("registro").document(username).set(regis);
    }

    public void getStepsAndCalories(MyCallbBack<DayInformationDTO> callback) {
        Task<DocumentSnapshot> document = db.collection("usuarios").document(username).
        collection("año").document(year).collection(month).document(day).get().addOnFailureListener(new OnFailureListener() {
            @Override
            public void onFailure(@NonNull Exception e) {
                try {
                    throw new Exception( e);
                } catch (Exception ex) {
                    ex.printStackTrace();
                }
            }
        }).addOnSuccessListener(new OnSuccessListener<DocumentSnapshot>() {
                    @Override
                    public void onSuccess(DocumentSnapshot documentSnapshot) {
                        int st;
                        int cl;
                        try {
                            long steps = (long) documentSnapshot.get("steps");
                            long calories = (long) documentSnapshot.get("calories");
                            st = (int) steps;
                            cl = (int) calories;
                        }catch(NullPointerException e){
                            st =0;
                            cl=0;
                        }
                        DayInformationDTO day = new DayInformationDTO(st, cl);
                        callback.onCallback(day);
                    }
                });
    }


    public void getLastConnection(MyCallbBack<Calendar> callback) {
        Task<DocumentSnapshot> document = db.collection("registro").document(username).get().addOnFailureListener(new OnFailureListener() {
                    @Override
                    public void onFailure(@NonNull Exception e) {
                        try {
                            throw new Exception( e);
                        } catch (Exception ex) {
                            ex.printStackTrace();
                        }
                    }
                }).addOnSuccessListener(new OnSuccessListener<DocumentSnapshot>() {
            @Override
            public void onSuccess(DocumentSnapshot documentSnapshot) {
                try{
                    long y = documentSnapshot.getLong("year");
                    long m = documentSnapshot.getLong("month");
                    long d = documentSnapshot.getLong("day");
                    Calendar calendar = Calendar.getInstance();
                    calendar.set(Calendar.YEAR, (int) y);
                    calendar.set(Calendar.MONTH, (int) m);
                    calendar.set(Calendar.DAY_OF_MONTH, (int) d);
                    callback.onCallback(calendar);
                }catch(NullPointerException e){
                    Calendar calendar1 = Calendar.getInstance();
                    calendar1.set(Calendar.YEAR, -1);
                    callback.onCallback(calendar);
                }
            }
        });
        
    }
}
