package com.example.trainingPartner.utils;


import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.example.mapbox2.R;
import com.example.trainingPartner.controller.TrainDetailsActivity;
import com.example.trainingPartner.controller.TrainDetailsManualActivity;
import com.example.trainingPartner.dto.activityDto.ActivityDTO;

import java.util.Collections;
import java.util.List;

public class ListAdapt extends RecyclerView.Adapter<ListAdapt.ViewHolder> {
    private List<ActivityDTO> list;
    private LayoutInflater mInflater;
    private Context context;

    public ListAdapt(List<ActivityDTO> lista, Context cont){
        this.mInflater= LayoutInflater.from(cont);
        this.list=lista;
        Collections.reverse(list);
        this.context=cont;
    }


    @NonNull
    @Override
    public ListAdapt.ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = mInflater.inflate(R.layout.activity_element, null, false);
        return new ListAdapt.ViewHolder(view);
    }

    @Override
    public void onBindViewHolder(@NonNull ListAdapt.ViewHolder holder, int position) {
        holder.bindData(list.get(position));
    }

    @Override
    public int getItemCount() {
        return list.size();
    }

    public class ViewHolder extends RecyclerView.ViewHolder{
        ImageView image;
        TextView title, distance, time;
        View itemView;


        public ViewHolder(@NonNull View itemView) {
            super(itemView);
            this.itemView = itemView;
            image = itemView.findViewById(R.id.image_element);
            title = itemView.findViewById(R.id.title_element);
            distance = itemView.findViewById(R.id.distance_element);
            time = itemView.findViewById(R.id.time_element);

        }

        void bindData(final ActivityDTO element){
            Intent intent;
            String t = "";
            if (element.getType()){
                t="MANUAL\t\t\t";
                itemView.findViewById(R.id.relative).setBackgroundColor(context.getResources().getColor(R.color.red));
                image.setColorFilter(context.getResources().getColor(R.color.black));
                intent = new Intent(context, TrainDetailsManualActivity.class);
            }else{
                t="AUTOMÁTICO\t\t";
                itemView.findViewById(R.id.relative).setBackgroundColor(context.getResources().getColor(R.color.gray));
                image.setColorFilter(context.getResources().getColor(R.color.black));
                intent = new Intent(context, TrainDetailsActivity.class);
            }
            intent.putExtra("hora", element.getHour());
            intent.putExtra("year",element.getYear());
            intent.putExtra("month",element.getMonth());
            intent.putExtra("day",element.getDay());
            String h = element.getHour();
            title.setText( t + h.substring(0, h.length() - 3));
            distance.setText(element.getDistance() + " km");
            System.out.println(Utils.returnStringTime(element.getTime()) );
            time.setText(Utils.returnStringTime(element.getTime()) );
            itemView.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    Activity activity = (Activity) context;
                    activity.finish();
                    context.startActivity(intent);
                }
            });
        }
    }
}
