package com.example.trainingPartner.utils;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.SharedPreferences;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.PorterDuff;
import android.graphics.PorterDuffXfermode;
import android.graphics.Rect;
import android.graphics.RectF;
import android.location.Location;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.Uri;
import android.preference.PreferenceManager;
import android.view.View;
import android.view.ViewGroup;
import android.view.inputmethod.InputMethodManager;
import android.widget.EditText;

import com.example.mapbox2.R;
import com.example.trainingPartner.dto.PointDTO;
import com.example.trainingPartner.dto.activityDto.ActivityDTO;
import com.example.trainingPartner.dto.activityDto.LocationActivityDTO;
import com.google.android.gms.tasks.Task;
import com.google.firebase.firestore.CollectionReference;
import com.google.firebase.firestore.DocumentSnapshot;
import com.google.firebase.firestore.QuerySnapshot;

import java.io.FileNotFoundException;
import java.math.BigInteger;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class Utils {

    public static final String KEY_REQUESTING_LOCATION_UPDATES = "requesting_location_updates";

    public static String returnStringTime(double time){
        int resultInt = (int)time;
        int horas = resultInt%86400 /3600;
        int minutes = resultInt%86400 %3600/60;
        int seconds = resultInt%86400%3600%60;

        return  String.format("%02d:%02d:%02d", horas, minutes,seconds);
    }

    public static String returnStringTimePartials(double time){
        int resultInt = (int)time;
        int horas = (resultInt%86400 /3600)*60;
        int minutes = resultInt%86400 %3600/60;
        int seconds = resultInt%86400%3600%60;


        return  String.format("%02d:%02d min/km", minutes+horas,seconds);
    }

    /**
     * Convierte de km/h a min/km
     * @param speed
     * @return
     */
    public static String convertToMinKm(double speed){
        if(speed <1)
            return "--:-- min/km";
        double sp = (60/speed); //Velocidad en min/km
        int min = (int)sp;
        int sec = (int)((sp -min)*100);
        int sec2 = (sec*60)/100;
        if (sec2>60){
            int minMas = sec/60;
            min += minMas;
            sec = sec-60;
        }
        String seconds = sec2+"";
        if (sec<10 || sec2<10){
            seconds = "0"+sec2;
        }
        if (min<10){
            return  "0"+min+":"+seconds+ " min/km";
        }
        return  min+":"+seconds+ " min/km";
    }

    public static Bitmap decodeUri(Context c, Uri uri, final int requiredSize)
            throws FileNotFoundException {
        BitmapFactory.Options o = new BitmapFactory.Options();
        o.inJustDecodeBounds = true;
        BitmapFactory.decodeStream(c.getContentResolver().openInputStream(uri), null, o);

        int width_tmp = o.outWidth
                , height_tmp = o.outHeight;
        int scale = 1;

        while(true) {
            if(width_tmp / 2 < requiredSize || height_tmp / 2 < requiredSize)
                break;
            width_tmp /= 2;
            height_tmp /= 2;
            scale *= 2;
        }

        BitmapFactory.Options o2 = new BitmapFactory.Options();
        o2.inSampleSize = scale;
        return BitmapFactory.decodeStream(c.getContentResolver().openInputStream(uri), null, o2);
    }

    public static Bitmap getRoundedCroppedBitmap(Bitmap bitmap) {
        int widthLight = bitmap.getWidth();
        int heightLight = bitmap.getHeight();

        Bitmap output = Bitmap.createBitmap(bitmap.getWidth(), bitmap.getHeight(),
                Bitmap.Config.ARGB_8888);

        Canvas canvas = new Canvas(output);
        Paint paintColor = new Paint();
        paintColor.setFlags(Paint.ANTI_ALIAS_FLAG);

        RectF rectF = new RectF(new Rect(0, 0, widthLight, heightLight));

        canvas.drawRoundRect(rectF, widthLight / 2, heightLight / 2, paintColor);

        Paint paintImage = new Paint();
        paintImage.setXfermode(new PorterDuffXfermode(PorterDuff.Mode.SRC_ATOP));
        canvas.drawBitmap(bitmap, 0, 0, paintImage);

        return output;
    }



    /**
     * Returns true if requesting location updates, otherwise returns false.
     *
     * @param context The {@link Context}.
     */
    public static boolean requestingLocationUpdates(Context context) {
        return PreferenceManager.getDefaultSharedPreferences(context)
                .getBoolean(KEY_REQUESTING_LOCATION_UPDATES, false);
    }

    /**
     * Stores the location updates state in SharedPreferences.
     * @param requestingLocationUpdates The location updates state.
     */
    public static void setRequestingLocationUpdates(Context context, boolean requestingLocationUpdates) {
        PreferenceManager.getDefaultSharedPreferences(context)
                .edit()
                .putBoolean(KEY_REQUESTING_LOCATION_UPDATES, requestingLocationUpdates)
                .apply();
    }

    /**
     * Returns the {@code location} object as a human readable string.
     * @param location  The {@link Location}.
     */
    public static String getLocationText(Location location) {
        return location == null ? "Unknown location" :
                "(" + location.getLatitude() + ", " + location.getLongitude() + ")";
    }

    public static String getLocationTitle(Context context) {
        return context.getString(R.string.location_updated,
                DateFormat.getDateTimeInstance().format(new Date()));
    }




    public static double roundTwoDecimals(double number){
        return Math.round(number*100)/100d;
    }


    public static ActivityDTO saveLocationActivity(DocumentSnapshot documentSnapshot, double distance, double time, Calendar calendar){
        List<HashMap<String, Double>> cordenadas = (List<HashMap<String, Double>>)documentSnapshot.get("coordinates");
        List<PointDTO> coordinates = setCoordinates(cordenadas);
        Map<String ,Double> kilometers = (Map<String ,Double>)documentSnapshot.get("kilometers");
        ActivityDTO actividadDTO;
        if (kilometers.size()>0){
            actividadDTO = new LocationActivityDTO(coordinates,kilometers, distance, time,calendar);
        }else{
            actividadDTO = new LocationActivityDTO(coordinates,time, calendar);
        }
        System.out.println(distance +" "+ coordinates + " "+ kilometers);
        return actividadDTO;
    }



    private static List<PointDTO> setCoordinates(List<HashMap<String, Double>> coordenadas){
        List<PointDTO> points = new ArrayList<>();
        for (HashMap<String,Double> point : coordenadas){
            points.add(new PointDTO(point.get("latitude"), point.get("longitude")));
        }
        return points;
    }

    public static String setStepsString(int steps){
        if (steps > 1000){
            int t = steps / 1000;
            int p = (steps-(t*1000))/100;
            String s = t + "."+p + "K";
            return s;
        }
        return steps+"";
    }

    public static Calendar getCalendar(String day, String month, String year){
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.DAY_OF_MONTH, Integer.parseInt(day));
        calendar.set(Calendar.MONTH, Integer.parseInt(month));
        calendar.set(Calendar.YEAR, Integer.parseInt(year));
        return calendar;
    }

    public static String getMD5(String input) {
        try {
            MessageDigest md = MessageDigest.getInstance("MD5");
            byte[] messageDigest = md.digest(input.getBytes());
            BigInteger number = new BigInteger(1, messageDigest);
            String hashtext = number.toString(16);

            while (hashtext.length() < 32) {
                hashtext = "0" + hashtext;
            }
            return hashtext;
        }
        catch (NoSuchAlgorithmException e) {
            throw new RuntimeException(e);
        }
    }

    public static boolean matchesWeight(EditText text){
        String txt = String.valueOf(text.getText());
        String regularExpr = "\\d{1,3}(\\.\\d{1,2})?";
        Pattern pattern = Pattern.compile(regularExpr);
        Matcher matcher = pattern.matcher(txt);
        return matcher.matches();
    }

    public static String getUserName(Context context){
        SharedPreferences userPref = context.getSharedPreferences("users",Context.MODE_PRIVATE);
        String username = userPref.getString("username", "None");
        return username;
    }

    public static void deleteSharedPreferences(Activity context){
        SharedPreferences userPref = context.getSharedPreferences("users",Context.MODE_PRIVATE);
        userPref.edit().remove("username").commit();
        SharedPreferences pref = context.getPreferences(Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = pref.edit();
        editor.clear();
        editor.apply();

    }

    public static void deleteUserSharedPreferences(Activity context){
        SharedPreferences userPref = context.getSharedPreferences("users",Context.MODE_PRIVATE);
        userPref.edit().remove("username").commit();


    }

    public static void deleteStepsSharedPreferences(Activity context){
        SharedPreferences userPref = context.getSharedPreferences("stepCount",Context.MODE_PRIVATE);
        userPref.edit().remove("stepCount").commit();
        SharedPreferences pref = context.getPreferences(Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = pref.edit();
        editor.clear();
        editor.apply();

    }

    public static boolean hasText(int id, Activity context){
        EditText txt = context.findViewById(id);
        return hasStringText(txt.getText().toString());
    }

    public static boolean hasStringText(String txt){
        if (txt== null)
            return false;
        return txt.replaceAll("\\s","").length()>0;
    }

    public static boolean hasConnection(Context context){

        ConnectivityManager cm =
                (ConnectivityManager)context.getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo activeNetwork = cm.getActiveNetworkInfo();
        boolean isConnected = activeNetwork != null &&
                activeNetwork.isConnectedOrConnecting();

        return isConnected;
    }

    public static void showDialogConnection(Context context){
        ProgressDialog  dialog = new ProgressDialog(context);
        dialog.setMessage("Sin conexión a internet...");
        dialog.setCancelable(false);
        dialog.show();
        while (!hasConnection(context)){
            dialog.show();
        }
        dialog.dismiss();
    }

    public static void deleteCollection(CollectionReference collection) {
        try {
            // retrieve a small batch of documents to avoid out-of-memory errors

            int deleted = 0;
            // future.get() blocks on document retrieval
            Task<QuerySnapshot> collections = collection.get();
            while (!collections.isComplete()){

            }
            List<DocumentSnapshot> documents = collections.getResult().getDocuments();
            System.out.println("DOCUMENTS:" + documents);
            for (DocumentSnapshot document : documents) {
                document.getReference().delete();
                ++deleted;
            }
        } catch (Exception e) {
            System.err.println("Error deleting collection : " + e.getMessage());
        }
    }


    public static void deActivateTouchingOutside(Activity a ){
        ViewGroup container = a.findViewById(android.R.id.content);
        View.OnClickListener listener = new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                InputMethodManager imm = (InputMethodManager) a.getSystemService(Context.INPUT_METHOD_SERVICE);
                View view = a.getCurrentFocus();
                if (view != null) {
                    v.clearFocus();
                    imm.hideSoftInputFromWindow(view.getWindowToken(), 0);
                }
            }
        };

        for (int i = 0; i < container.getChildCount(); i++) {
            View child = container.getChildAt(i);
            if (!(child instanceof EditText)) {
                child.setOnClickListener(listener);
            }
        }

    }








}
